import { SqlQuery } from '../sql';
import { Column } from './column';
export interface QueryResultValue {
    header: readonly Column[];
    rows: readonly any[][];
    query?: Record<string, unknown>;
    sqlQuery?: SqlQuery;
    queryId?: string;
    sqlQueryId?: string;
    resultContext?: Record<string, any>;
    queryDuration?: number;
}
export declare class QueryResult {
    static BLANK: QueryResult;
    static jsonParse: (str: string) => any;
    static shouldIncludeTimestamp(queryPayload: Record<string, unknown>): boolean;
    static hasHeader(queryPayload: Record<string, unknown>): boolean;
    static hasTypeHeader(queryPayload: Record<string, unknown>, header: Record<string, string>): boolean;
    static hasSqlTypeHeader(queryPayload: Record<string, unknown>, header: Record<string, string>): boolean;
    static fromQueryAndRawResult(queryPayload: Record<string, unknown>, data: unknown, header?: Record<string, string>): QueryResult;
    static fromRawResult(data: unknown, includeTimestampIfExists?: boolean, hasHeader?: boolean, hasTypeHeader?: boolean, hasSqlTypeHeader?: boolean): QueryResult;
    static fromObjectArray(array: Record<string, any>[], hasHeader?: boolean, hasTypes?: boolean): QueryResult;
    readonly header: readonly Column[];
    readonly rows: readonly any[][];
    readonly query?: Record<string, unknown>;
    readonly sqlQuery?: SqlQuery;
    readonly queryId?: string;
    readonly sqlQueryId?: string;
    readonly resultContext?: Record<string, any>;
    readonly queryDuration?: number;
    constructor(value: QueryResultValue);
    valueOf(): QueryResultValue;
    changeQueryDuration(queryDuration: number): QueryResult;
    attachQuery(query: Record<string, unknown>, sqlQuery?: SqlQuery): QueryResult;
    attachQueryId(queryId: string | undefined, sqlQueryId?: string): QueryResult;
    changeResultContext(resultContext: Record<string, any> | undefined): QueryResult;
    getHeaderNames(): string[];
    isEmpty(): boolean;
    getNumResults(): number;
    toObjectArray(): Record<string, any>[];
    getColumnByIndex(columnIndex: number): any[] | undefined;
    getColumnByName(columnName: string): any[] | undefined;
    getSqlOuterLimit(): number | undefined;
    private guessDateColumnIndexes;
    private inflateDatesForIndexes;
    inflateDatesByGuessing(): QueryResult;
    inflateDatesFromSqlTypes(): QueryResult;
}
//# sourceMappingURL=query-result.d.ts.map