import { LiteralValue, RefName, SqlComparison, SqlOrderByDirection, SqlOrderByExpression, SqlUnary } from '.';
import { SqlBase, Substitutor } from './sql-base';
export interface DecomposeViaOptions {
    flatten?: boolean;
}
export declare abstract class SqlExpression extends SqlBase {
    static parse(input: string | SqlExpression): SqlExpression;
    static maybeParse(input: string | SqlExpression): SqlExpression | undefined;
    static wrap(input: SqlExpression | LiteralValue): SqlExpression;
    static and(...args: (SqlExpression | undefined)[]): SqlExpression;
    static or(...args: (SqlExpression | undefined)[]): SqlExpression;
    static fromTimeExpressionAndInterval(time: SqlExpression, interval: string | string[]): SqlExpression;
    static arrayOfLiterals(xs: LiteralValue[]): SqlExpression;
    _walkHelper(stack: SqlBase[], fn: Substitutor, postorder: boolean): SqlExpression | undefined;
    _walkInner(_nextStack: SqlBase[], _fn: Substitutor, _postorder: boolean): SqlExpression | undefined;
    as(alias: RefName | string | undefined, forceQuotes?: boolean): SqlExpression;
    ifUnnamedAliasAs(alias: RefName | string, forceQuotes?: boolean): SqlExpression;
    convertToTable(): SqlExpression;
    getAliasName(): string | undefined;
    getOutputName(): string | undefined;
    getUnderlyingExpression(): SqlExpression;
    toOrderByExpression(direction?: SqlOrderByDirection): SqlOrderByExpression;
    not(): SqlUnary;
    negate(): SqlExpression;
    equal(rhs: SqlExpression | LiteralValue): SqlComparison;
    unequal(rhs: SqlExpression | LiteralValue): SqlComparison;
    lessThan(rhs: SqlExpression | LiteralValue): SqlComparison;
    greaterThan(rhs: SqlExpression | LiteralValue): SqlComparison;
    lessThanOrEqual(rhs: SqlExpression | LiteralValue): SqlComparison;
    greaterThanOrEqual(rhs: SqlExpression | LiteralValue): SqlComparison;
    isNull(): SqlComparison;
    isNotNull(): SqlComparison;
    in(values: (SqlExpression | LiteralValue)[]): SqlComparison;
    notIn(values: (SqlExpression | LiteralValue)[]): SqlComparison;
    like(rhs: SqlExpression | string, escape?: SqlExpression | string): SqlComparison;
    between(start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    notBetween(start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    betweenSymmetric(start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    notBetweenSymmetric(start: SqlExpression | LiteralValue, end: SqlExpression | LiteralValue): SqlComparison;
    and(expression: SqlExpression): SqlExpression;
    or(expression: SqlExpression): SqlExpression;
    decomposeViaAnd(_options?: DecomposeViaOptions): SqlExpression[];
    decomposeViaOr(_options?: DecomposeViaOptions): SqlExpression[];
    filterAnd(fn: (ex: SqlExpression) => boolean): SqlExpression | undefined;
    removeColumnFromAnd(column: string): SqlExpression | undefined;
    fillPlaceholders(fillWith: (SqlExpression | LiteralValue)[]): SqlExpression;
    cast(asType: string): SqlExpression;
    addFilterToAggregations(filter: SqlExpression, knownAggregations: string[]): SqlExpression;
}
//# sourceMappingURL=sql-expression.d.ts.map