import { SqlBase, SqlBaseValue, SqlTypeDesignator, Substitutor } from '../sql-base';
import { SqlColumnDeclaration, SqlExtendClause, SqlWhereClause } from '../sql-clause';
import { SqlExpression } from '../sql-expression';
import { LiteralValue, SqlLiteral } from '../sql-literal/sql-literal';
import { SqlNamespace } from '../sql-namespace/sql-namespace';
import { SqlType } from '../sql-type/sql-type';
import { SqlWindowSpec } from '../sql-window-spec/sql-window-spec';
import { RefName, SeparatedArray } from '../utils';
export declare type SpecialParen = 'square' | 'none';
export interface SqlFunctionValue extends SqlBaseValue {
    namespace?: SqlNamespace;
    functionName: RefName;
    specialParen?: SpecialParen;
    decorator?: string;
    args?: SeparatedArray<SqlExpression>;
    whereClause?: SqlWhereClause;
    extendClause?: SqlExtendClause;
    windowSpec?: SqlWindowSpec;
}
export declare class SqlFunction extends SqlExpression {
    static type: SqlTypeDesignator;
    static DEFAULT_FILTER_KEYWORD: string;
    static DEFAULT_OVER_KEYWORD: string;
    static COUNT_STAR: SqlFunction;
    static SPECIAL_FUNCTIONS: string[];
    static isValidFunctionName(functionName: string): boolean;
    static isNakedFunction(functionName: string): boolean;
    static simple(functionName: RefName | string, args: (SqlExpression | LiteralValue)[] | SeparatedArray<SqlExpression>, filter?: SqlWhereClause | SqlExpression): SqlFunction;
    static decorated(functionName: RefName | string, decorator: string | undefined, args: (SqlExpression | LiteralValue)[] | SeparatedArray<SqlExpression>, filter?: SqlWhereClause | SqlExpression): SqlFunction;
    static count(arg?: SqlExpression): SqlFunction;
    static countDistinct(arg: SqlExpression): SqlFunction;
    static sum(arg: SqlExpression): SqlFunction;
    static min(arg: SqlExpression): SqlFunction;
    static max(arg: SqlExpression): SqlFunction;
    static avg(arg: SqlExpression): SqlFunction;
    static cast(ex: SqlExpression, asType: SqlType | string): SqlExpression;
    static floor(ex: SqlExpression, timeUnit: string | SqlLiteral): SqlExpression;
    static timeFloor(timestampExpr: SqlExpression, period: string | SqlLiteral, origin?: string | SqlLiteral, timezone?: string | SqlLiteral): SqlFunction;
    static timeCeil(timestampExpr: SqlExpression, period: string | SqlLiteral, origin?: string | SqlLiteral, timezone?: string | SqlLiteral): SqlFunction;
    static timeShift(timestampExpr: SqlExpression, period: string | SqlLiteral, step?: number | SqlLiteral, timezone?: string | SqlLiteral): SqlFunction;
    static array(...exs: (LiteralValue | SqlExpression)[]): SqlExpression;
    static stringFormat(format: string | SqlLiteral, ...args: (SqlExpression | LiteralValue)[]): SqlFunction;
    static regexpLike(ex: SqlExpression, pattern: string | SqlLiteral): SqlFunction;
    readonly namespace?: SqlNamespace;
    readonly functionName: RefName;
    readonly specialParen?: SpecialParen;
    readonly args?: SeparatedArray<SqlExpression>;
    readonly decorator?: string;
    readonly whereClause?: SqlWhereClause;
    readonly extendClause?: SqlExtendClause;
    readonly windowSpec?: SqlWindowSpec;
    constructor(options: SqlFunctionValue);
    valueOf(): SqlFunctionValue;
    protected _toRawString(): string;
    changeNamespace(namespace: SqlNamespace | undefined): this;
    getNamespaceName(): string | undefined;
    changeNamespaceName(namespace: string | undefined): this;
    getEffectiveFunctionName(): string;
    getEffectiveDecorator(): string | undefined;
    changeArgs(args: SeparatedArray<SqlExpression>): this;
    numArgs(): number;
    getArgArray(): readonly SqlExpression[];
    getArg(index: number): SqlExpression | undefined;
    getArgAsString(index: number): string | undefined;
    getArgAsNumber(index: number): number | undefined;
    getArgAsNumberOrBigint(index: number): number | bigint | undefined;
    changeWhereClause(whereClause: SqlWhereClause | undefined): this;
    changeWhereExpression(whereExpression: SqlExpression | undefined): this;
    addWhereExpression(whereExpression: SqlExpression): this;
    getWhereExpression(): SqlExpression | undefined;
    getEffectiveWhereExpression(): SqlExpression;
    changeExtendClause(extendClause: SqlExtendClause | undefined): this;
    isAggregation(knownAggregations: string[]): boolean;
    _walkInner(nextStack: SqlBase[], fn: Substitutor, postorder: boolean): SqlExpression | undefined;
    clearOwnSeparators(): this;
    resetOwnKeywords(): any;
    isCountStar(): boolean;
    getCastType(): SqlType | undefined;
    getColumnDeclarations(): readonly SqlColumnDeclaration[] | undefined;
    changeColumnDeclarations(columnDeclarations: readonly SqlColumnDeclaration[] | undefined): this;
}
//# sourceMappingURL=sql-function.d.ts.map