import { SqlBase, SqlBaseValue, SqlTypeDesignator } from '../sql-base';
import { DecomposeViaOptions, SqlExpression } from '../sql-expression';
export declare type LiteralValue = null | boolean | number | bigint | string | Date;
export interface SqlLiteralValue extends SqlBaseValue {
    value: LiteralValue;
    stringValue?: string;
}
export declare class SqlLiteral extends SqlExpression {
    static type: SqlTypeDesignator;
    static DEFAULT_TIMESTAMP_KEYWORD: string;
    static DEFAULT_NULL_KEYWORD: string;
    static DEFAULT_FALSE_KEYWORD: string;
    static DEFAULT_TRUE_KEYWORD: string;
    static NULL: SqlLiteral;
    static FALSE: SqlLiteral;
    static TRUE: SqlLiteral;
    static ZERO: SqlLiteral;
    static create(value: LiteralValue | SqlLiteral): SqlLiteral;
    static double(value: number): SqlLiteral;
    static maybe(value: any): SqlLiteral | undefined;
    static index(n: number): SqlLiteral;
    static direct(v: string | SqlLiteral): SqlLiteral;
    static escapeLiteralString(str: string): string;
    static dateToTimestampValue(date: Date): string;
    static _equalsLiteral(expression: SqlBase, value: number): boolean;
    readonly value: LiteralValue;
    readonly stringValue?: string;
    constructor(options: SqlLiteralValue);
    valueOf(): SqlLiteralValue;
    getEffectiveStringValue(): string;
    protected _toRawString(): string;
    resetOwnKeywords(): this;
    isIndex(): boolean;
    getIndexValue(): number;
    incrementIndex(amount?: number): SqlLiteral;
    prettyTrim(maxLength: number): this;
    decomposeViaAnd(_options?: DecomposeViaOptions): SqlExpression[];
    isInteger(): boolean;
    isDate(): boolean;
    getNumberValue(): number | undefined;
    getNumberOrBigintValue(): number | bigint | undefined;
    getStringValue(): string | undefined;
    getDateValue(): Date | undefined;
}
//# sourceMappingURL=sql-literal.d.ts.map