import { SqlBase, SqlBaseValue, SqlTypeDesignator, Substitutor } from '../sql-base';
import { SqlClusteredByClause, SqlFromClause, SqlGroupByClause, SqlHavingClause, SqlInsertClause, SqlJoinPart, SqlLimitClause, SqlOffsetClause, SqlOrderByClause, SqlOrderByDirection, SqlOrderByExpression, SqlPartitionedByClause, SqlWhereClause, SqlWithClause, SqlWithPart } from '../sql-clause';
import { SqlReplaceClause } from '../sql-clause/sql-replace-clause/sql-replace-clause';
import { SqlExpression } from '../sql-expression';
import { SqlLiteral } from '../sql-literal/sql-literal';
import { SqlTable } from '../sql-table/sql-table';
import { SeparatedArray } from '../utils';
export declare type SqlQueryDecorator = 'ALL' | 'DISTINCT';
export declare type InsertIndex = number | 'last-grouping' | 'last';
export interface ExpressionInfo {
    expression: SqlExpression;
    selectIndex: number;
    outputColumn?: string;
    orderByExpression?: SqlOrderByExpression;
}
export interface AddSelectOptions {
    insertIndex?: InsertIndex;
    addToGroupBy?: 'start' | 'end';
    addToOrderBy?: 'start' | 'end';
    orderByExpression?: SqlOrderByExpression;
    direction?: SqlOrderByDirection;
}
export interface SqlQueryValue extends SqlBaseValue {
    explain?: boolean;
    insertClause?: SqlInsertClause;
    replaceClause?: SqlReplaceClause;
    withClause?: SqlWithClause;
    decorator?: SqlQueryDecorator;
    selectExpressions?: SeparatedArray<SqlExpression>;
    fromClause?: SqlFromClause;
    whereClause?: SqlWhereClause;
    groupByClause?: SqlGroupByClause;
    havingClause?: SqlHavingClause;
    orderByClause?: SqlOrderByClause;
    limitClause?: SqlLimitClause;
    offsetClause?: SqlOffsetClause;
    partitionedByClause?: SqlPartitionedByClause;
    clusteredByClause?: SqlClusteredByClause;
    unionQuery?: SqlQuery;
}
export declare class SqlQuery extends SqlExpression {
    static type: SqlTypeDesignator;
    static readonly DEFAULT_EXPLAIN_PLAN_FOR_KEYWORD = "EXPLAIN PLAN FOR";
    static readonly DEFAULT_SELECT_KEYWORD = "SELECT";
    static readonly DEFAULT_UNION_KEYWORD = "UNION ALL";
    static create(from: string | SqlExpression | SqlFromClause): SqlQuery;
    static from(from: string | SqlExpression | SqlFromClause): SqlQuery;
    static parse(input: string | SqlQuery): SqlQuery;
    static maybeParse(input: string | SqlQuery): SqlQuery | undefined;
    static getSelectExpressionOutput(selectExpression: SqlExpression, i: number): string;
    static isPhonyOutputName(name: string): boolean;
    readonly explain?: boolean;
    readonly insertClause?: SqlInsertClause;
    readonly replaceClause?: SqlReplaceClause;
    readonly withClause?: SqlWithClause;
    readonly decorator?: SqlQueryDecorator;
    readonly selectExpressions?: SeparatedArray<SqlExpression>;
    readonly fromClause?: SqlFromClause;
    readonly whereClause?: SqlWhereClause;
    readonly groupByClause?: SqlGroupByClause;
    readonly havingClause?: SqlHavingClause;
    readonly orderByClause?: SqlOrderByClause;
    readonly limitClause?: SqlLimitClause;
    readonly partitionedByClause?: SqlPartitionedByClause;
    readonly clusteredByClause?: SqlClusteredByClause;
    readonly offsetClause?: SqlOffsetClause;
    readonly unionQuery?: SqlQuery;
    constructor(options: SqlQueryValue);
    valueOf(): SqlQueryValue;
    protected _toRawString(): string;
    changeExplain(explain: boolean): this;
    changeInsertClause(insertClause: SqlInsertClause | undefined): this;
    changeReplaceClause(replaceClause: SqlReplaceClause | undefined): this;
    changeWithClause(withClause: SqlWithClause | undefined): this;
    changeSelectExpressions(selectExpressions: SeparatedArray<SqlExpression> | SqlExpression[] | undefined): this;
    changeFromClause(fromClause: SqlFromClause | undefined): this;
    changeFromExpressions(fromExpressions: SeparatedArray<SqlExpression> | SqlExpression[] | undefined): this;
    changeWhereClause(whereClause: SqlWhereClause | undefined): this;
    changeWhereExpression(whereExpression: SqlExpression | undefined): this;
    changeGroupByClause(groupByClause: SqlGroupByClause | undefined): this;
    changeGroupByExpressions(groupByExpressions: SeparatedArray<SqlExpression> | SqlExpression[] | undefined): this;
    changeHavingClause(havingClause: SqlHavingClause | undefined): this;
    changeHavingExpression(havingExpression: SqlExpression | undefined): this;
    changeOrderByClause(orderByClause: SqlOrderByClause | undefined): this;
    changeOrderByExpressions(orderByExpressions: SeparatedArray<SqlOrderByExpression> | SqlOrderByExpression[] | undefined): this;
    changeOrderByExpression(orderByExpression: SqlOrderByExpression | undefined): this;
    changeLimitClause(limitClause: SqlLimitClause | undefined): this;
    changeLimitValue(limitValue: SqlLiteral | number | undefined): this;
    changeOffsetClause(offsetClause: SqlOffsetClause | undefined): this;
    changeOffsetValue(offsetValue: SqlLiteral | number | undefined): this;
    changePartitionedByClause(partitionedByClause: SqlPartitionedByClause | undefined): this;
    changeClusteredByClause(clusteredByClause: SqlClusteredByClause | undefined): this;
    changeClusteredByExpressions(clusteredByExpressions: SeparatedArray<SqlExpression> | SqlExpression[] | undefined): this;
    changeUnionQuery(unionQuery: SqlQuery | undefined): this;
    _walkInner(nextStack: SqlBase[], fn: Substitutor, postorder: boolean): SqlQuery | undefined;
    clearOwnSeparators(): this;
    makeExplain(): this;
    getInsertIntoTable(): SqlTable | undefined;
    changeInsertIntoTable(table: SqlTable | string | undefined): this;
    getReplaceIntoTable(): SqlTable | undefined;
    changeReplaceIntoTable(table: SqlTable | string | undefined): this;
    getIngestTable(): SqlTable | undefined;
    getWithParts(): readonly SqlWithPart[];
    changeWithParts(withParts: SeparatedArray<SqlWithPart> | SqlWithPart[] | undefined): this;
    prependWith(name: string, query: SqlQuery): this;
    flattenWith(): SqlQuery;
    isValidSelectIndex(selectIndex: number): boolean;
    getSelectExpressionsArray(): readonly SqlExpression[];
    getSelectExpressionForIndex(selectIndex: number): SqlExpression | undefined;
    hasStarInSelect(): boolean;
    getOutputColumns(): string[];
    getSelectIndexesForColumn(column: string): number[];
    getGroupedSelectIndexesForColumn(column: string): number[];
    getSelectIndexForOutputColumn(outputColumn: string): number;
    private expressionRefersToSelectIndex;
    getSelectIndexForExpression(ex: SqlExpression, allowAliasReferences: boolean): number;
    isRealOutputColumnAtSelectIndex(selectIndex: number): boolean;
    isGroupedSelectIndex(selectIndex: number): boolean;
    isGroupedOutputColumn(outputColumn: string): boolean;
    getGroupedSelectExpressions(): SqlExpression[];
    getGroupedOutputColumns(): string[];
    isAggregateSelectIndex(selectIndex: number): boolean;
    isAggregateOutputColumn(outputColumn: string): boolean;
    getAggregateSelectExpressions(): SqlExpression[];
    getAggregateOutputColumns(): string[];
    private decodeInsertIndex;
    addSelect(ex: SqlExpression, options?: AddSelectOptions): this;
    changeSelect(selectIndex: number, ex: SqlExpression): this;
    removeSelectIndex(selectIndex: number): this;
    removeSelectIndexes(selectIndexes: number[]): this;
    removeOutputColumn(outputColumn: string): this;
    hasFrom(): boolean;
    getFromExpressions(): readonly SqlExpression[];
    getFirstFromExpression(): SqlExpression | undefined;
    getFirstTableName(): string | undefined;
    getFirstSchema(): string | undefined;
    hasJoin(): boolean;
    getJoins(): readonly SqlJoinPart[];
    addJoin(join: SqlJoinPart): this;
    addLeftJoin(table: SqlExpression, onExpression: SqlExpression | SqlExpression[]): this;
    addRightJoin(table: SqlExpression, onExpression: SqlExpression | SqlExpression[]): this;
    addInnerJoin(table: SqlExpression, onExpression: SqlExpression | SqlExpression[]): this;
    addFullJoin(table: SqlExpression, onExpression: SqlExpression | SqlExpression[]): this;
    addCrossJoin(table: SqlExpression): this;
    removeAllJoins(): this;
    hasWhere(): boolean;
    getWhereExpression(): SqlExpression | undefined;
    getEffectiveWhereExpression(): SqlExpression;
    addWhere(ex: SqlExpression): this;
    removeColumnFromWhere(column: string): this;
    hasGroupBy(): boolean;
    getGroupingExpressionInfos(): ExpressionInfo[];
    getGroupingExpressions(): SqlExpression[];
    addGroupBy(ex: SqlExpression): this;
    hasHaving(): boolean;
    getHavingExpression(): SqlExpression | undefined;
    getEffectiveHavingExpression(): SqlExpression;
    addHaving(ex: SqlExpression): this;
    removeFromHaving(column: string): this;
    hasOrderBy(): boolean;
    getOrderByExpressions(): readonly SqlOrderByExpression[];
    getOrderByForSelectIndex(selectIndex: number): SqlOrderByExpression | undefined;
    getOrderByForExpression(ex: SqlExpression): SqlOrderByExpression | undefined;
    getOrderByForOutputColumn(outputColumn: string): SqlOrderByExpression | undefined;
    getOrderedSelectExpressions(): SqlExpression[];
    getOrderedOutputColumns(): string[];
    removeOrderByForSelectIndex(selectIndex: number): this;
    removeOrderByForOutputColumn(outputColumn: string): this;
    addOrderBy(orderBy: SqlOrderByExpression): this;
    hasLimit(): boolean;
    combineWithLimitClause(otherLimitClause: SqlLimitClause): SqlQuery;
    hasOffset(): boolean;
    combineWithOffsetClause(otherOffsetClause: SqlOffsetClause): SqlQuery;
}
//# sourceMappingURL=sql-query.d.ts.map