"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __esm = (fn, res) => function __init() {
  return fn && (res = (0, fn[__getOwnPropNames(fn)[0]])(fn = 0)), res;
};
var __commonJS = (cb, mod) => function __require() {
  return mod || (0, cb[__getOwnPropNames(cb)[0]])((mod = { exports: {} }).exports, mod), mod.exports;
};
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target, mod));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/utils.ts
function cleanObject(obj) {
  const cleanObj = {};
  for (const k in obj) {
    const v = obj[k];
    if (typeof v === "string")
      cleanObj[k] = v;
  }
  return cleanObj;
}
function filterMap(xs, f) {
  return xs.map(f).filter((x) => typeof x !== "undefined");
}
function objectMap(obj, f) {
  const ret = {};
  for (const k in obj) {
    const v = f(obj[k], k);
    if (typeof v !== "undefined") {
      ret[k] = v;
    }
  }
  return ret;
}
function dedupe(array, keyFn = String) {
  const seen = /* @__PURE__ */ new Map();
  const deduped = [];
  array.forEach((d) => {
    const key = keyFn(d);
    if (seen.has(key))
      return;
    seen.set(key, true);
    deduped.push(d);
  });
  return deduped;
}
function isEmptyArray(x) {
  return Array.isArray(x) && !x.length;
}
function compact(xs) {
  return xs.filter(Boolean);
}
function sane(_x) {
  const str = String.raw.apply(String, arguments);
  const match = /^\n( *)/m.exec(str);
  if (!match)
    throw new Error("sane string must start with a \\n is:" + str);
  const spaces = match[1].length;
  let lines = str.split("\n");
  lines.shift();
  lines = lines.map((line) => line.substr(spaces));
  if (lines[lines.length - 1] === "")
    lines.pop();
  return lines.join("\n").replace(/\\`/g, "`").replace(/\\\{/g, "{").replace(/\\\\/g, "\\");
}
var init_utils = __esm({
  "src/utils.ts"() {
    "use strict";
  }
});

// src/sql/utils/general/general.ts
function trimString(str, maxLength) {
  if (str.length < maxLength)
    return str;
  return str.substr(0, Math.max(maxLength - 3, 1)) + "...";
}
function clampIndex(index, min, max) {
  return Math.max(min, Math.min(max, Math.floor(index)));
}
function normalizeIndex(index, n) {
  return index < 0 ? index + n : index;
}
function insert(array, index, value) {
  const n = array.length;
  if (index < 0 || n < index)
    throw new Error(`insert index out of bounds (${index})`);
  return array.slice(0, index).concat([value], array.slice(index, n));
}
function getEscapedRegExp() {
  ESCAPED.lastIndex = 0;
  return ESCAPED;
}
function needsUnicodeEscape(str) {
  return getEscapedRegExp().test(str);
}
function sqlEscapeUnicode(str) {
  return str.replace(/\\/g, "\\\\").replace(getEscapedRegExp(), (a) => `\\${("0000" + a.charCodeAt(0).toString(16)).slice(-4)}`);
}
var ESCAPED;
var init_general = __esm({
  "src/sql/utils/general/general.ts"() {
    "use strict";
    ESCAPED = /[\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g;
  }
});

// src/sql/allowed-functions.ts
var ALLOWED_FUNCTIONS;
var init_allowed_functions = __esm({
  "src/sql/allowed-functions.ts"() {
    "use strict";
    ALLOWED_FUNCTIONS = [
      "ABS",
      "ALL",
      "ARRAY",
      "AVG",
      "CARDINALITY",
      "CAST",
      "CEIL",
      "CEILING",
      "CHARACTER_LENGTH",
      "CHAR_LENGTH",
      "CLASSIFIER",
      "COALESCE",
      "COLLECT",
      "CONVERT",
      "COUNT",
      "COVAR_POP",
      "COVAR_SAMP",
      "CUME_DIST",
      "CURSOR",
      "DENSE_RANK",
      "DISTINCT",
      "ELEMENT",
      "EXISTS",
      "EXP",
      "EXTRACT",
      "FIRST_VALUE",
      "FLOOR",
      "FUSION",
      "GROUPING",
      "HOUR",
      "JSON_ARRAY",
      "JSON_ARRAYAGG",
      "JSON_EXISTS",
      "JSON_OBJECT",
      "JSON_OBJECTAGG",
      "JSON_QUERY",
      "JSON_VALUE",
      "LAG",
      "LAST_VALUE",
      "LEAD",
      "LEFT",
      "LN",
      "LOCALTIME",
      "LOCALTIMESTAMP",
      "LOWER",
      "MATCH_NUMBER",
      "MAX",
      "MIN",
      "MINUTE",
      "MOD",
      "MONTH",
      "MULTISET",
      "NEXT",
      "NOT",
      "NTH_VALUE",
      "NTILE",
      "NULLIF",
      "OCTET_LENGTH",
      "OVERLAY",
      "PERCENT_RANK",
      "PERIOD",
      "POSITION",
      "POWER",
      "PREV",
      "RANK",
      "REGR_COUNT",
      "REGR_SXX",
      "REGR_SYY",
      "RIGHT",
      "ROW",
      "ROW_NUMBER",
      "SECOND",
      "SQRT",
      "STDDEV_POP",
      "STDDEV_SAMP",
      "STREAM",
      "SUBSTRING",
      "SUM",
      "TABLE",
      "TRANSLATE",
      "TRIM",
      "TRUNCATE",
      "UNNEST",
      "UPPER",
      "USER",
      "VAR_POP",
      "VAR_SAMP",
      "YEAR"
    ];
  }
});

// src/sql/reserved-keywords.ts
var RESERVED_KEYWORDS, RESERVED_ALIASES;
var init_reserved_keywords = __esm({
  "src/sql/reserved-keywords.ts"() {
    "use strict";
    RESERVED_KEYWORDS = [
      "ABS",
      "ALL",
      "ALLOCATE",
      "ALLOW",
      "ALTER",
      "AND",
      "ANY",
      "ARE",
      "ARRAY",
      "ARRAY_MAX_CARDINALITY",
      "AS",
      "ASENSITIVE",
      "ASYMMETRIC",
      "AT",
      "ATOMIC",
      "AUTHORIZATION",
      "AVG",
      "BEGIN",
      "BEGIN_FRAME",
      "BEGIN_PARTITION",
      "BETWEEN",
      "BIGINT",
      "BINARY",
      "BIT",
      "BLOB",
      "BOOLEAN",
      "BOTH",
      "BY",
      "CALL",
      "CALLED",
      "CARDINALITY",
      "CASCADED",
      "CASE",
      "CAST",
      "CEIL",
      "CEILING",
      "CHAR",
      "CHARACTER",
      "CHARACTER_LENGTH",
      "CHAR_LENGTH",
      "CHECK",
      "CLASSIFIER",
      "CLOB",
      "CLOSE",
      "CLUSTERED",
      "COALESCE",
      "COLLATE",
      "COLLECT",
      "COLUMN",
      "COMMIT",
      "CONDITION",
      "CONNECT",
      "CONSTRAINT",
      "CONTAINS",
      "CONVERT",
      "CORR",
      "CORRESPONDING",
      "COUNT",
      "COVAR_POP",
      "COVAR_SAMP",
      "CREATE",
      "CROSS",
      "CUBE",
      "CUME_DIST",
      "CURRENT",
      "CURRENT_DATE",
      "CURRENT_ROW",
      "CURRENT_TIME",
      "CURRENT_TIMESTAMP",
      "CURRENT_TRANSFORM_GROUP_FOR_TYPE",
      "CURSOR",
      "CYCLE",
      "DATE",
      "DAY",
      "DEALLOCATE",
      "DEC",
      "DECIMAL",
      "DECLARE",
      "DEFAULT",
      "DEFINE",
      "DELETE",
      "DENSE_RANK",
      "DEREF",
      "DESCRIBE",
      "DETERMINISTIC",
      "DISALLOW",
      "DISCONNECT",
      "DISTINCT",
      "DOUBLE",
      "DROP",
      "DYNAMIC",
      "EACH",
      "ELEMENT",
      "ELSE",
      "EMPTY",
      "END",
      "END_FRAME",
      "END_PARTITION",
      "EQUALS",
      "ESCAPE",
      "EVERY",
      "EXCEPT",
      "EXEC",
      "EXECUTE",
      "EXISTS",
      "EXP",
      "EXPLAIN",
      "EXTEND",
      "EXTERNAL",
      "EXTRACT",
      "FALSE",
      "FETCH",
      "FILTER",
      "FIRST_VALUE",
      "FLOAT",
      "FLOOR",
      "FOR",
      "FOREIGN",
      "FRAME_ROW",
      "FREE",
      "FROM",
      "FULL",
      "FUNCTION",
      "FUSION",
      "GET",
      "GLOBAL",
      "GRANT",
      "GROUP",
      "GROUPING",
      "GROUPS",
      "HAVING",
      "HOLD",
      "HOUR",
      "IDENTITY",
      "IMPORT",
      "IN",
      "INDICATOR",
      "INITIAL",
      "INNER",
      "INOUT",
      "INSENSITIVE",
      "INSERT",
      "INT",
      "INTEGER",
      "INTERSECT",
      "INTERSECTION",
      "INTERVAL",
      "INTO",
      "IS",
      "JOIN",
      "JSON_ARRAY",
      "JSON_ARRAYAGG",
      "JSON_EXISTS",
      "JSON_OBJECT",
      "JSON_OBJECTAGG",
      "JSON_QUERY",
      "JSON_VALUE",
      "LAG",
      "LANGUAGE",
      "LARGE",
      "LAST_VALUE",
      "LATERAL",
      "LEAD",
      "LEADING",
      "LEFT",
      "LIKE",
      "LIKE_REGEX",
      "LIMIT",
      "LN",
      "LOCAL",
      "LOCALTIME",
      "LOCALTIMESTAMP",
      "LOWER",
      "MATCH",
      "MATCHES",
      "MATCH_NUMBER",
      "MATCH_RECOGNIZE",
      "MAX",
      "MEASURES",
      "MEMBER",
      "MERGE",
      "METHOD",
      "MIN",
      "MINUS",
      "MINUTE",
      "MOD",
      "MODIFIES",
      "MODULE",
      "MONTH",
      "MULTISET",
      "NATIONAL",
      "NATURAL",
      "NCHAR",
      "NCLOB",
      "NEW",
      "NEXT",
      "NO",
      "NONE",
      "NORMALIZE",
      "NOT",
      "NTH_VALUE",
      "NTILE",
      "NULL",
      "NULLIF",
      "NUMERIC",
      "OCCURRENCES_REGEX",
      "OCTET_LENGTH",
      "OF",
      "OFFSET",
      "OLD",
      "OMIT",
      "ON",
      "ONE",
      "ONLY",
      "OPEN",
      "OR",
      "ORDER",
      "OUT",
      "OUTER",
      "OVER",
      "OVERLAPS",
      "OVERLAY",
      "PARAMETER",
      "PARTITION",
      "PARTITIONED",
      "PATTERN",
      "PER",
      "PERCENT",
      "PERCENTILE_CONT",
      "PERCENTILE_DISC",
      "PERCENT_RANK",
      "PERIOD",
      "PERMUTE",
      "PORTION",
      "POSITION",
      "POSITION_REGEX",
      "POWER",
      "PRECEDES",
      "PRECISION",
      "PREPARE",
      "PREV",
      "PRIMARY",
      "PROCEDURE",
      "RANGE",
      "RANK",
      "READS",
      "REAL",
      "RECURSIVE",
      "REF",
      "REFERENCES",
      "REFERENCING",
      "REGR_AVGX",
      "REGR_AVGY",
      "REGR_COUNT",
      "REGR_INTERCEPT",
      "REGR_SLOPE",
      "REGR_SXX",
      "REGR_SXY",
      "REGR_SYY",
      "RELEASE",
      "RESET",
      "RESULT",
      "RETURN",
      "RETURNS",
      "REVOKE",
      "RIGHT",
      "ROLLBACK",
      "ROLLUP",
      "ROW",
      "ROWS",
      "ROW_NUMBER",
      "RUNNING",
      "SAVEPOINT",
      "SCOPE",
      "SCROLL",
      "SEARCH",
      "SECOND",
      "SEEK",
      "SELECT",
      "SENSITIVE",
      "SET",
      "SHOW",
      "SIMILAR",
      "SKIP",
      "SMALLINT",
      "SOME",
      "SPECIFIC",
      "SPECIFICTYPE",
      "SQL",
      "SQLEXCEPTION",
      "SQLSTATE",
      "SQLWARNING",
      "SQRT",
      "START",
      "STATIC",
      "STDDEV_POP",
      "STDDEV_SAMP",
      "STREAM",
      "SUBMULTISET",
      "SUBSET",
      "SUBSTRING",
      "SUBSTRING_REGEX",
      "SUCCEEDS",
      "SUM",
      "SYMMETRIC",
      "SYSTEM",
      "SYSTEM_TIME",
      "TABLE",
      "TABLESAMPLE",
      "THEN",
      "TIME",
      "TIMESTAMP",
      "TIMEZONE_HOUR",
      "TIMEZONE_MINUTE",
      "TINYINT",
      "TO",
      "TRAILING",
      "TRANSLATE",
      "TRANSLATE_REGEX",
      "TRANSLATION",
      "TREAT",
      "TRIGGER",
      "TRIM",
      "TRIM_ARRAY",
      "TRUE",
      "TRUNCATE",
      "UESCAPE",
      "UNION",
      "UNIQUE",
      "UNKNOWN",
      "UNNEST",
      "UPDATE",
      "UPPER",
      "UPSERT",
      "USING",
      "VALUE",
      "VALUES",
      "VALUE_OF",
      "VARBINARY",
      "VARCHAR",
      "VARYING",
      "VAR_POP",
      "VAR_SAMP",
      "VERSIONING",
      "WHEN",
      "WHENEVER",
      "WHERE",
      "WIDTH_BUCKET",
      "WINDOW",
      "WITH",
      "WITHIN",
      "WITHOUT",
      "YEAR"
    ];
    RESERVED_ALIASES = [
      "CURRENT_CATALOG",
      "CURRENT_DEFAULT_TRANSFORM_GROUP",
      "CURRENT_PATH",
      "CURRENT_ROLE",
      "CURRENT_SCHEMA",
      "CURRENT_USER",
      "SESSION_USER",
      "SYSTEM_USER",
      "USER"
    ];
  }
});

// src/sql/utils/ref-name/ref-name.ts
var VALID_NAKED_NAME_REGEXP, reservedKeywordLookup, reservedAliasLookup, allowedFunctionLookup, _RefName, RefName;
var init_ref_name = __esm({
  "src/sql/utils/ref-name/ref-name.ts"() {
    "use strict";
    init_allowed_functions();
    init_reserved_keywords();
    init_general();
    VALID_NAKED_NAME_REGEXP = /^[a-z_]\w*$/i;
    reservedKeywordLookup = {};
    for (const r of RESERVED_KEYWORDS) {
      reservedKeywordLookup[r] = true;
    }
    reservedAliasLookup = {};
    for (const r of RESERVED_ALIASES) {
      reservedAliasLookup[r] = true;
    }
    allowedFunctionLookup = {};
    for (const r of ALLOWED_FUNCTIONS) {
      allowedFunctionLookup[r] = true;
    }
    _RefName = class {
      static isReservedKeyword(k) {
        return Boolean(reservedKeywordLookup[k.toUpperCase()]);
      }
      static isReservedAlias(k) {
        return _RefName.isReservedKeyword(k) || Boolean(reservedAliasLookup[k.toUpperCase()]);
      }
      static isReservedFunctionName(k) {
        return _RefName.isReservedKeyword(k) && !allowedFunctionLookup[k.toUpperCase()];
      }
      static needsQuotes(name) {
        return !VALID_NAKED_NAME_REGEXP.test(name) || _RefName.isReservedKeyword(name);
      }
      static needsQuotesAlias(name) {
        return !VALID_NAKED_NAME_REGEXP.test(name) || _RefName.isReservedAlias(name);
      }
      static needsQuotesFunctionName(name) {
        return !VALID_NAKED_NAME_REGEXP.test(name) || _RefName.isReservedFunctionName(name);
      }
      static maybe(name, forceQuotes = true) {
        if (!name)
          return;
        return _RefName.create(name, forceQuotes);
      }
      static create(name, forceQuotes = true) {
        if (name instanceof _RefName)
          return name;
        return new _RefName({ name, quotes: forceQuotes || _RefName.needsQuotes(name) });
      }
      static alias(name, forceQuotes = true) {
        if (name instanceof _RefName)
          return name;
        return new _RefName({ name, quotes: forceQuotes || _RefName.needsQuotesAlias(name) });
      }
      static functionName(name, forceQuotes = false) {
        if (name instanceof _RefName)
          return name;
        return new _RefName({ name, quotes: forceQuotes || _RefName.needsQuotesFunctionName(name) });
      }
      constructor(options) {
        this.name = options.name;
        this.quotes = options.quotes;
      }
      valueOf() {
        return {
          name: this.name,
          quotes: this.quotes
        };
      }
      toString() {
        const { name, quotes } = this;
        if (!quotes)
          return name;
        const quoteEscapedName = name.replace(/"/g, '""');
        if (needsUnicodeEscape(quoteEscapedName)) {
          return `U&"${sqlEscapeUnicode(quoteEscapedName)}"`;
        } else {
          return `"${quoteEscapedName}"`;
        }
      }
      changeName(name) {
        return _RefName.create(name, this.quotes);
      }
      changeNameAsAlias(name) {
        return _RefName.alias(name, this.quotes);
      }
      changeNameAsFunctionName(name) {
        return _RefName.functionName(name, this.quotes);
      }
      prettyTrim(maxLength) {
        return this.changeName(trimString(this.name, maxLength));
      }
    };
    RefName = _RefName;
    RefName.RESERVED_KEYWORDS = RESERVED_KEYWORDS;
    RefName.RESERVED_ALIASES = RESERVED_ALIASES;
    RefName.ALLOWED_FUNCTIONS = ALLOWED_FUNCTIONS;
  }
});

// src/sql/utils/separated-array/separated-array.ts
var SeparatedArray;
var init_separated_array = __esm({
  "src/sql/utils/separated-array/separated-array.ts"() {
    "use strict";
    init_utils2();
    SeparatedArray = class {
      static fromArray(xs, separator) {
        if (xs instanceof SeparatedArray)
          return xs;
        const separators = [];
        if (separator) {
          for (let i = 1; i < xs.length; i++) {
            separators.push(separator);
          }
        }
        return new SeparatedArray(xs, separators);
      }
      static fromSingleValue(x) {
        return new SeparatedArray([x], []);
      }
      static fromTwoValuesWithSeparator(value0, separator, value1) {
        return new SeparatedArray([value0, value1], [separator]);
      }
      constructor(values, separators) {
        separators = separators || [];
        if (!values.length) {
          throw new Error("must have at least one value");
        }
        if (values.length <= separators.length) {
          throw new Error(`invalid values (${values.length}) or separator length (${separators.length})`);
        }
        this.values = values;
        this.separators = separators;
      }
      toString(defaultSeparator = Separator.COMMA) {
        const { values, separators } = this;
        const lastIndex = values.length - 1;
        return values.map((v, i) => String(v) + (i < lastIndex ? String(separators[i] || defaultSeparator) : "")).join("");
      }
      length() {
        return this.values.length;
      }
      get(index) {
        return this.values[normalizeIndex(index, this.values.length)];
      }
      first() {
        return this.values[0];
      }
      change(index, newValue) {
        index = normalizeIndex(index, this.values.length);
        return this.map((value, i) => i === index ? newValue : value);
      }
      map(callbackfn) {
        return new SeparatedArray(this.values.map(callbackfn), this.separators);
      }
      filter(fn) {
        const { values, separators } = this;
        const filteredValues = [];
        const filteredSeparators = [];
        let skippedFirst = false;
        for (let i = 0; i < values.length; i++) {
          const value = values[i];
          if (fn(value, i)) {
            filteredValues.push(value);
            if (i > 0)
              filteredSeparators.push(separators[i - 1]);
          } else if (i === 0) {
            skippedFirst = true;
          }
        }
        if (skippedFirst && filteredSeparators.length) {
          filteredSeparators.shift();
        }
        if (!filteredValues.length)
          return;
        return new SeparatedArray(filteredValues, filteredSeparators);
      }
      filterMap(callbackfn) {
        return this.map(callbackfn).filter(Boolean);
      }
      remove(index) {
        if (index < 0)
          index += this.values.length;
        return this.filter((_x, i) => i !== index);
      }
      insert(index, value) {
        const { values, separators } = this;
        index = clampIndex(normalizeIndex(index, values.length), 0, values.length);
        const sn = separators.length;
        const separator = separators[clampIndex(index - 1, 0, sn - 1)];
        return new SeparatedArray(insert(values, index, value), insert(separators, Math.min(index, sn), separator));
      }
      prepend(value) {
        return this.insert(0, value);
      }
      append(value) {
        return this.insert(Infinity, value);
      }
      clearSeparators() {
        return new SeparatedArray(this.values, []);
      }
      clearSeparatorsMatching(match) {
        let hasUnmatched = false;
        const newSeparators = this.separators.map((separator) => {
          if (typeof separator === "string" && separator === match || separator instanceof Separator && separator.separator === match) {
            return;
          } else {
            hasUnmatched = true;
            return separator;
          }
        });
        return new SeparatedArray(this.values, hasUnmatched ? newSeparators : []);
      }
    };
  }
});

// src/sql/utils/separator/separator.ts
var Separator;
var init_separator = __esm({
  "src/sql/utils/separator/separator.ts"() {
    "use strict";
    Separator = class {
      static symmetricSpace(separator) {
        return new Separator({ left: " ", separator, right: " " });
      }
      static rightSpace(separator) {
        return new Separator({ separator, right: " " });
      }
      constructor(options) {
        this.left = options.left || "";
        this.right = options.right || "";
        this.separator = options.separator;
      }
      toString() {
        return [this.left, this.separator, this.right].join("");
      }
    };
    Separator.COMMA = Separator.rightSpace(",");
    Separator.COMMA_NEW_LINE = new Separator({ separator: ",", right: "\n" });
  }
});

// src/sql/utils/index.ts
var init_utils2 = __esm({
  "src/sql/utils/index.ts"() {
    "use strict";
    init_general();
    init_ref_name();
    init_separated_array();
    init_separator();
  }
});

// src/sql/parser/index.js
var require_parser = __commonJS({
  "src/sql/parser/index.js"(exports) {
    "use strict";
    var sql = (init_sql(), __toCommonJS(sql_exports));
    var p = function() {
      "use strict";
      function peg$subclass(child, parent) {
        function ctor() {
          this.constructor = child;
        }
        ctor.prototype = parent.prototype;
        child.prototype = new ctor();
      }
      function peg$SyntaxError(message, expected, found, location) {
        this.message = message;
        this.expected = expected;
        this.found = found;
        this.location = location;
        this.name = "SyntaxError";
        if (typeof Error.captureStackTrace === "function") {
          Error.captureStackTrace(this, peg$SyntaxError);
        }
      }
      peg$subclass(peg$SyntaxError, Error);
      peg$SyntaxError.buildMessage = function(expected, found) {
        var DESCRIBE_EXPECTATION_FNS = {
          literal: function(expectation) {
            return '"' + literalEscape(expectation.text) + '"';
          },
          "class": function(expectation) {
            var escapedParts = "", i;
            for (i = 0; i < expectation.parts.length; i++) {
              escapedParts += expectation.parts[i] instanceof Array ? classEscape(expectation.parts[i][0]) + "-" + classEscape(expectation.parts[i][1]) : classEscape(expectation.parts[i]);
            }
            return "[" + (expectation.inverted ? "^" : "") + escapedParts + "]";
          },
          any: function(expectation) {
            return "any character";
          },
          end: function(expectation) {
            return "end of input";
          },
          other: function(expectation) {
            return expectation.description;
          }
        };
        function hex(ch) {
          return ch.charCodeAt(0).toString(16).toUpperCase();
        }
        function literalEscape(s) {
          return s.replace(/\\/g, "\\\\").replace(/"/g, '\\"').replace(/\0/g, "\\0").replace(/\t/g, "\\t").replace(/\n/g, "\\n").replace(/\r/g, "\\r").replace(/[\x00-\x0F]/g, function(ch) {
            return "\\x0" + hex(ch);
          }).replace(/[\x10-\x1F\x7F-\x9F]/g, function(ch) {
            return "\\x" + hex(ch);
          });
        }
        function classEscape(s) {
          return s.replace(/\\/g, "\\\\").replace(/\]/g, "\\]").replace(/\^/g, "\\^").replace(/-/g, "\\-").replace(/\0/g, "\\0").replace(/\t/g, "\\t").replace(/\n/g, "\\n").replace(/\r/g, "\\r").replace(/[\x00-\x0F]/g, function(ch) {
            return "\\x0" + hex(ch);
          }).replace(/[\x10-\x1F\x7F-\x9F]/g, function(ch) {
            return "\\x" + hex(ch);
          });
        }
        function describeExpectation(expectation) {
          return DESCRIBE_EXPECTATION_FNS[expectation.type](expectation);
        }
        function describeExpected(expected2) {
          var descriptions = new Array(expected2.length), i, j;
          for (i = 0; i < expected2.length; i++) {
            descriptions[i] = describeExpectation(expected2[i]);
          }
          descriptions.sort();
          if (descriptions.length > 0) {
            for (i = 1, j = 1; i < descriptions.length; i++) {
              if (descriptions[i - 1] !== descriptions[i]) {
                descriptions[j] = descriptions[i];
                j++;
              }
            }
            descriptions.length = j;
          }
          switch (descriptions.length) {
            case 1:
              return descriptions[0];
            case 2:
              return descriptions[0] + " or " + descriptions[1];
            default:
              return descriptions.slice(0, -1).join(", ") + ", or " + descriptions[descriptions.length - 1];
          }
        }
        function describeFound(found2) {
          return found2 ? '"' + literalEscape(found2) + '"' : "end of input";
        }
        return "Expected " + describeExpected(expected) + " but " + describeFound(found) + " found.";
      };
      function peg$parse(input, options) {
        options = options !== void 0 ? options : {};
        var peg$FAILED = {}, peg$startRuleFunctions = { Start: peg$parseStart }, peg$startRuleFunction = peg$parseStart, peg$c0 = function(initial, thing, final) {
          if (initial)
            thing = thing.changeSpace("initial", initial);
          if (final)
            thing = thing.changeSpace("final", final);
          return thing;
        }, peg$c1 = function(expression, alias, columns) {
          if (!alias)
            return expression;
          var value = { expression };
          var spacing = value.spacing = {};
          var keywords = value.keywords = {};
          var as = alias[0];
          if (as) {
            spacing.preAs = as[0];
            keywords.as = as[1];
          } else {
            keywords.as = "";
          }
          if (columns) {
            spacing.preColumns = columns[0];
            value.columns = columns[1];
          }
          spacing.preAlias = alias[1];
          value.alias = alias[2];
          return new sql.SqlAlias(value);
        }, peg$c2 = "=>", peg$c3 = peg$literalExpectation("=>", false), peg$c4 = function(label, preArrow, postArrow, expression) {
          return new sql.SqlLabeledExpression({
            label,
            spacing: {
              preArrow,
              postArrow
            },
            expression
          });
        }, peg$c5 = function(extend, postLeftParen, head, tail, postColumnDeclarations) {
          return new sql.SqlExtendClause({
            keywords: {
              extend: extend ? extend[0] : ""
            },
            columnDeclarations: makeSeparatedArray(head, tail),
            spacing: {
              postExtend: extend ? extend[1] : void 0,
              postLeftParen,
              postColumnDeclarations
            }
          });
        }, peg$c6 = function(column, postColumn, columnType) {
          return new sql.SqlColumnDeclaration({
            column,
            spacing: {
              postColumn
            },
            columnType
          });
        }, peg$c7 = function(explain, insertClause, replaceClause, heart, orderByClause, limitClause, offsetClause, partitionedByClause, clusteredByClause, union) {
          var value = {};
          var keywords = value.keywords = {};
          var spacing = value.spacing = {};
          if (explain) {
            value.explain = true;
            keywords.explainPlanFor = explain[0];
            spacing.postExplainPlanFor = explain[1];
          }
          if (insertClause && replaceClause) {
            error("Can not have both an INSERT and a REPLACE clause");
          }
          if (insertClause) {
            value.insertClause = insertClause[0];
            spacing.postInsertClause = insertClause[1];
          }
          if (replaceClause) {
            value.replaceClause = replaceClause[0];
            spacing.postReplaceClause = replaceClause[1];
          }
          var withQueryMode = heart.length === 7;
          if (withQueryMode) {
            value.withClause = heart[0];
            spacing.postWithClause = heart[1];
            value.query = heart[4].addParens(heart[3], heart[5]);
          } else {
            var withClause = heart[0];
            if (withClause) {
              value.withClause = withClause[0];
              spacing.postWithClause = withClause[1];
            }
            var subQuery = heart[1];
            Object.assign(value, subQuery.value);
            Object.assign(keywords, subQuery.keywords);
            Object.assign(spacing, subQuery.spacing);
          }
          if (partitionedByClause) {
            spacing.prePartitionedByClause = partitionedByClause[0];
            value.partitionedByClause = partitionedByClause[1];
          }
          if (clusteredByClause) {
            spacing.preClusteredByClause = clusteredByClause[0];
            value.clusteredByClause = clusteredByClause[1];
          }
          if (orderByClause) {
            spacing.preOrderByClause = orderByClause[0];
            value.orderByClause = orderByClause[1];
          }
          if (limitClause) {
            spacing.preLimitClause = limitClause[0];
            value.limitClause = limitClause[1];
          }
          if (offsetClause) {
            spacing.preOffsetClause = offsetClause[0];
            value.offsetClause = offsetClause[1];
          }
          if (union) {
            spacing.preUnion = union[0];
            keywords.union = union[1].unionKeyword;
            spacing.postUnion = union[1].postUnion;
            value.unionQuery = union[1].unionQuery;
          }
          return withQueryMode ? new sql.SqlWithQuery(value) : new sql.SqlQuery(value);
        }, peg$c8 = function(select, fromClause, whereClause, groupByClause, havingClause) {
          var value = {};
          var keywords = {};
          var spacing = {};
          keywords.select = select.selectKeyword;
          spacing.postSelect = select.postSelect;
          value.decorator = select.decorator;
          keywords.decorator = select.decoratorKeyword;
          spacing.postDecorator = select.postDecorator;
          value.selectExpressions = select.selectExpressions;
          if (fromClause) {
            spacing.preFromClause = fromClause[0];
            value.fromClause = fromClause[1];
          }
          if (whereClause) {
            spacing.preWhereClause = whereClause[0];
            value.whereClause = whereClause[1];
          }
          if (groupByClause) {
            spacing.preGroupByClause = groupByClause[0];
            value.groupByClause = groupByClause[1];
          }
          if (havingClause) {
            spacing.preHavingClause = havingClause[0];
            value.havingClause = havingClause[1];
          }
          return {
            value,
            keywords,
            spacing
          };
        }, peg$c9 = function(insert2, postInsert, into, postInto, table, columns) {
          var value = {
            table,
            keywords: {
              insert: insert2,
              into
            },
            spacing: {
              postInsert,
              postInto
            }
          };
          if (columns) {
            value.spacing.preColumns = columns[0];
            value.columns = columns[1];
          }
          return new sql.SqlInsertClause(value);
        }, peg$c10 = function(replace, postReplace, into, postInto, table, columns, preOverwrite, overwrite, postOverwrite, allOrWhere) {
          var value = {
            table,
            keywords: {
              replace,
              into,
              overwrite
            },
            spacing: {
              postReplace,
              postInto,
              preOverwrite,
              postOverwrite
            }
          };
          if (columns) {
            value.spacing.preColumns = columns[0];
            value.columns = columns[1];
          }
          if (typeof allOrWhere === "string") {
            value.keywords.all = allOrWhere;
          } else {
            value.whereClause = allOrWhere;
          }
          return new sql.SqlReplaceClause(value);
        }, peg$c11 = function(withKeyword, postWith, head, tail) {
          return new sql.SqlWithClause({
            withParts: makeSeparatedArray(head, tail),
            keywords: {
              "with": withKeyword
            },
            spacing: {
              postWith
            }
          });
        }, peg$c12 = function(table, postTable, columns, asKeyword, postAs, query) {
          var value = {
            table,
            query
          };
          var spacing = value.spacing = {
            postTable,
            postAs
          };
          var keywords = value.keywords = {
            as: asKeyword
          };
          if (columns) {
            value.columns = columns[0];
            spacing.postColumns = columns[1];
          }
          return new sql.SqlWithPart(value);
        }, peg$c13 = function(postLeftParen, head, tail, preRightParen) {
          return new sql.SqlColumnList({
            columns: makeSeparatedArray(head, tail)
          }).addParens(postLeftParen, preRightParen);
        }, peg$c14 = function(selectKeyword, postSelect, decorator, head, tail) {
          var ret = {
            selectKeyword,
            postSelect,
            selectExpressions: makeSeparatedArray(head, tail)
          };
          if (decorator) {
            ret.decorator = decorator[0].toUpperCase();
            ret.decoratorKeyword = decorator[0];
            ret.postDecorator = decorator[1];
          }
          return ret;
        }, peg$c15 = function(from, postFrom, head, tail, join) {
          return new sql.SqlFromClause({
            expressions: makeSeparatedArray(head, tail).map(function(ex) {
              return ex.convertToTable();
            }),
            joinParts: join ? join[1] : void 0,
            spacing: {
              postFrom,
              preJoin: join ? join[0] : void 0
            },
            keywords: {
              from
            }
          });
        }, peg$c16 = function(head, tail) {
          return makeSeparatedArray(head, tail);
        }, peg$c17 = function(joinType, join, postJoin, table, on) {
          var value = {
            table: table.convertToTable()
          };
          var spacing = value.spacing = {
            postJoin
          };
          var keywords = value.keywords = {
            join
          };
          if (joinType) {
            var joinTypeUpper = joinType[0].toUpperCase();
            var m = joinTypeUpper.match(/^LEFT|RIGHT|FULL/);
            if (m)
              joinTypeUpper = m[0];
            value.joinType = joinTypeUpper;
            keywords.joinType = joinType[0];
            spacing.postJoinType = joinType[1];
          }
          if (on) {
            spacing.preOn = on[0];
            keywords.on = on[1];
            spacing.postOn = on[2];
            value.onExpression = on[3];
          }
          return new sql.SqlJoinPart(value);
        }, peg$c18 = "left", peg$c19 = peg$literalExpectation("LEFT", true), peg$c20 = "right", peg$c21 = peg$literalExpectation("RIGHT", true), peg$c22 = "full", peg$c23 = peg$literalExpectation("FULL", true), peg$c24 = "inner", peg$c25 = peg$literalExpectation("INNER", true), peg$c26 = "cross", peg$c27 = peg$literalExpectation("CROSS", true), peg$c28 = function(where, postWhere, expression) {
          return new sql.SqlWhereClause({
            expression,
            spacing: {
              postWhere
            },
            keywords: {
              where
            }
          });
        }, peg$c29 = function(groupBy, postGroupBy, decorator, ex) {
          var value = {
            spacing: {
              postGroupBy
            },
            keywords: {
              groupBy
            }
          };
          if (decorator) {
            var d = decorator[0].toUpperCase();
            value.decorator = d.startsWith("GROUPING") ? "GROUPING SETS" : d;
            value.spacing.postDecorator = decorator[1];
            value.keywords.decorator = decorator[0];
          }
          if (Array.isArray(ex)) {
            value.innerParens = true;
            value.spacing.postLeftParen = ex[1];
            if (ex[2]) {
              value.expressions = ex[2][0];
              value.spacing.postExpressions = ex[2][1];
            }
          } else {
            value.expressions = ex;
          }
          return new sql.SqlGroupByClause(value);
        }, peg$c30 = function(postLeftParen) {
          return new sql.SqlRecord({
            keywords: { row: "" },
            spacing: {
              postLeftParen
            }
          });
        }, peg$c31 = function(having, postHaving, expression) {
          return new sql.SqlHavingClause({
            expression,
            spacing: {
              postHaving
            },
            keywords: {
              having
            }
          });
        }, peg$c32 = function(orderBy, postOrderBy, head, tail) {
          return new sql.SqlOrderByClause({
            expressions: makeSeparatedArray(head, tail),
            spacing: {
              postOrderBy
            },
            keywords: {
              orderBy
            }
          });
        }, peg$c33 = function(expression, direction) {
          var value = {
            expression
          };
          var spacing = value.spacing = {};
          var keywords = value.keywords = {};
          if (direction) {
            spacing.preDirection = direction[0];
            value.direction = direction[1].toUpperCase();
            keywords.direction = direction[1];
          }
          return new sql.SqlOrderByExpression(value);
        }, peg$c34 = function(limit, postLimit, limitLiteral) {
          return new sql.SqlLimitClause({
            limit: limitLiteral,
            spacing: {
              postLimit
            },
            keywords: {
              limit
            }
          });
        }, peg$c35 = function(offset, postOffset, offsetLiteral) {
          return new sql.SqlOffsetClause({
            offset: offsetLiteral,
            spacing: {
              postOffset
            },
            keywords: {
              offset
            }
          });
        }, peg$c36 = function(partitionedBy, postPartitionedBy, ex) {
          var value = {};
          var spacing = value.spacing = {
            postPartitionedBy
          };
          var keywords = value.keywords = {
            partitionedBy
          };
          if (Array.isArray(ex)) {
            var ex1 = ex[1];
            keywords.all = ex[0] + (ex1 ? ex1.join("") : "");
          } else {
            value.expression = ex;
          }
          return new sql.SqlPartitionedByClause(value);
        }, peg$c37 = function(clusteredBy, postClusteredBy, head, tail) {
          return new sql.SqlClusteredByClause({
            expressions: makeSeparatedArray(head, tail),
            spacing: {
              postClusteredBy
            },
            keywords: {
              clusteredBy
            }
          });
        }, peg$c38 = function(unionKeyword, postUnion, unionQuery) {
          return {
            unionKeyword,
            postUnion,
            unionQuery
          };
        }, peg$c39 = function(head, tail) {
          return maybeMakeMulti("OR", head, tail);
        }, peg$c40 = function(head, tail) {
          return maybeMakeMulti("AND", head, tail);
        }, peg$c41 = function(op, postOp, argument) {
          return new sql.SqlUnary({
            op: op.toUpperCase(),
            argument,
            spacing: {
              postOp
            },
            keywords: {
              op
            }
          });
        }, peg$c42 = function(lhs, rhs) {
          if (!rhs)
            return lhs;
          var preOp = rhs[0];
          var opRhs = rhs[1];
          return new sql.SqlComparison({
            lhs,
            op: opRhs.op,
            decorator: opRhs.decorator,
            rhs: opRhs.rhs,
            negated: Boolean(opRhs.notKeyword),
            spacing: {
              preOp: opRhs.preOp ? opRhs.preOp : preOp,
              postOp: opRhs.postOp,
              not: opRhs.preOp ? preOp : opRhs.notSpacing,
              postDecorator: opRhs.postDecorator
            },
            keywords: {
              op: opRhs.opKeyword,
              not: opRhs.notKeyword,
              decorator: opRhs.decoratorKeyword
            }
          });
        }, peg$c43 = function(op, postOp, rhs) {
          const ret = {
            op: op === "!=" ? "<>" : op,
            opKeyword: op,
            postOp
          };
          if (Array.isArray(rhs)) {
            var decorator = rhs[0];
            var decoratorUpper = decorator.toUpperCase();
            ret.decorator = decoratorUpper === "SOME" ? "ANY" : decoratorUpper;
            ret.decoratorKeyword = decorator;
            ret.postDecorator = rhs[1];
            ret.rhs = rhs[2];
          } else {
            ret.rhs = rhs;
          }
          return ret;
        }, peg$c44 = "=", peg$c45 = peg$literalExpectation("=", false), peg$c46 = "<>", peg$c47 = peg$literalExpectation("<>", false), peg$c48 = "!=", peg$c49 = peg$literalExpectation("!=", false), peg$c50 = ">=", peg$c51 = peg$literalExpectation(">=", false), peg$c52 = "<=", peg$c53 = peg$literalExpectation("<=", false), peg$c54 = "<", peg$c55 = peg$literalExpectation("<", false), peg$c56 = ">", peg$c57 = peg$literalExpectation(">", false), peg$c58 = function(op, postOp, not, rhs) {
          return {
            op: op.toUpperCase(),
            opKeyword: op,
            postOp,
            rhs,
            notKeyword: not ? not[0] : void 0,
            notSpacing: not ? not[1] : void 0
          };
        }, peg$c59 = function(op, postOp, rhs) {
          return {
            op: op.toUpperCase(),
            opKeyword: op,
            postOp,
            rhs
          };
        }, peg$c60 = function(op, postOp, symmetricKeyword, start, preAnd, andKeyword, postAnd, end) {
          var value = {
            start,
            end,
            spacing: {
              preAnd,
              postAnd
            },
            keywords: {
              and: andKeyword
            }
          };
          if (symmetricKeyword) {
            value.symmetric = true;
            value.keywords.symmetric = symmetricKeyword[0];
            value.spacing.postSymmetric = symmetricKeyword[1];
          }
          return {
            op: op.toUpperCase(),
            opKeyword: op,
            postOp,
            rhs: new sql.SqlBetweenPart(value)
          };
        }, peg$c61 = function(op, postOp, like, escape) {
          return {
            op: op.toUpperCase(),
            opKeyword: op,
            postOp,
            rhs: escape ? new sql.SqlLikePart({
              like,
              escape: escape[3],
              spacing: {
                preEscape: escape[0],
                postEscape: escape[2]
              },
              keywords: {
                escape: escape[1]
              }
            }) : like
          };
        }, peg$c62 = function(notKeyword, preOp, opRhs) {
          return Object.assign({}, opRhs, {
            notKeyword,
            preOp
          });
        }, peg$c63 = "+", peg$c64 = peg$literalExpectation("+", false), peg$c65 = function(head, tail) {
          return maybeMakeMulti("+", head, tail);
        }, peg$c66 = "-", peg$c67 = peg$literalExpectation("-", false), peg$c68 = function(head, tail) {
          return maybeMakeMulti("-", head, tail);
        }, peg$c69 = "*", peg$c70 = peg$literalExpectation("*", false), peg$c71 = function(head, tail) {
          return maybeMakeMulti("*", head, tail);
        }, peg$c72 = "/", peg$c73 = peg$literalExpectation("/", false), peg$c74 = function(head, tail) {
          return maybeMakeMulti("/", head, tail);
        }, peg$c75 = /^[+\-]/, peg$c76 = peg$classExpectation(["+", "-"], false, false), peg$c77 = function(op, postOp, argument) {
          return new sql.SqlUnary({
            op,
            argument,
            spacing: {
              postOp
            },
            keywords: {
              op
            }
          });
        }, peg$c78 = "||", peg$c79 = peg$literalExpectation("||", false), peg$c80 = function(head, tail) {
          return maybeMakeMulti("||", head, tail);
        }, peg$c81 = function(caseToken, postCase, caseExpression, head, tail, elseValue, preEnd, end) {
          return new sql.SqlCase({
            caseExpression: caseExpression ? caseExpression[0] : void 0,
            whenThenParts: makeSeparatedArray(head, tail),
            elseExpression: elseValue ? elseValue[3] : void 0,
            spacing: {
              postCase,
              postCaseExpression: caseExpression ? caseExpression[1] : void 0,
              preElse: elseValue ? elseValue[0] : void 0,
              postElse: elseValue ? elseValue[2] : void 0,
              preEnd
            },
            keywords: {
              "case": caseToken,
              "else": elseValue ? elseValue[1] : void 0,
              end
            }
          });
        }, peg$c82 = function(when, postWhen, whenHead, whenTail, postWhenExpressions, then, postThen, thenExpression) {
          return new sql.SqlWhenThenPart({
            whenExpressions: makeSeparatedArray(whenHead, whenTail),
            thenExpression,
            spacing: {
              postWhen,
              postWhenExpressions,
              postThen
            },
            keywords: {
              when,
              then
            }
          });
        }, peg$c83 = "_", peg$c84 = peg$literalExpectation("_", false), peg$c85 = function(interval, postInterval, intervalValue, postIntervalValue, unit) {
          return new sql.SqlInterval({
            intervalValue,
            unit,
            spacing: {
              postInterval,
              postIntervalValue
            },
            keywords: {
              interval
            }
          });
        }, peg$c86 = function(unit) {
          return sql.SqlLiteral.direct(unit);
        }, peg$c87 = "second", peg$c88 = peg$literalExpectation("SECOND", true), peg$c89 = "minute", peg$c90 = peg$literalExpectation("MINUTE", true), peg$c91 = "hour", peg$c92 = peg$literalExpectation("HOUR", true), peg$c93 = "day", peg$c94 = peg$literalExpectation("DAY", true), peg$c95 = "week", peg$c96 = peg$literalExpectation("WEEK", true), peg$c97 = "month", peg$c98 = peg$literalExpectation("MONTH", true), peg$c99 = "quarter", peg$c100 = peg$literalExpectation("QUARTER", true), peg$c101 = "year", peg$c102 = peg$literalExpectation("YEAR", true), peg$c103 = "epoch", peg$c104 = peg$literalExpectation("EPOCH", true), peg$c105 = "microsecond", peg$c106 = peg$literalExpectation("MICROSECOND", true), peg$c107 = "millisecond", peg$c108 = peg$literalExpectation("MILLISECOND", true), peg$c109 = "dow", peg$c110 = peg$literalExpectation("DOW", true), peg$c111 = "isodow", peg$c112 = peg$literalExpectation("ISODOW", true), peg$c113 = "doy", peg$c114 = peg$literalExpectation("DOY", true), peg$c115 = "isoyear", peg$c116 = peg$literalExpectation("ISOYEAR", true), peg$c117 = "decade", peg$c118 = peg$literalExpectation("DECADE", true), peg$c119 = "century", peg$c120 = peg$literalExpectation("CENTURY", true), peg$c121 = "millennium", peg$c122 = peg$literalExpectation("MILLENNIUM", true), peg$c123 = ".", peg$c124 = peg$literalExpectation(".", false), peg$c125 = function(functionName, namespaceExtra, preLeftParen, postLeftParen, decorator, head, tail, postArguments, filter, window, extend) {
          var value = {};
          var spacing = value.spacing = {
            preLeftParen,
            postLeftParen
          };
          var keywords = value.keywords = {};
          if (namespaceExtra) {
            value.namespace = new sql.SqlNamespace({ refName: functionName });
            spacing.postNamespace = namespaceExtra[0];
            spacing.postDot = namespaceExtra[2];
            value.functionName = namespaceExtra[3];
          } else {
            value.functionName = functionName;
          }
          if (decorator) {
            value.decorator = decorator[0].toUpperCase();
            keywords.decorator = decorator[0];
            spacing.postDecorator = decorator[1];
          }
          if (head) {
            value.args = makeSeparatedArray(head, tail);
            spacing.postArguments = postArguments;
          }
          if (filter) {
            spacing.preFilter = filter[0];
            keywords.filter = filter[1].filterKeyword;
            spacing.postFilter = filter[1].postFilter;
            value.whereClause = filter[1].whereClause;
          }
          if (window) {
            spacing.preOver = window[0];
            keywords.over = window[1];
            spacing.postOver = window[2];
            value.windowSpec = window[3];
          }
          if (extend) {
            value.spacing.preExtend = extend[0];
            value.extendClause = extend[1];
          }
          return new sql.SqlFunction(value);
        }, peg$c126 = function(functionName) {
          return sql.SqlFunction.isNakedFunction(functionName);
        }, peg$c127 = function(functionName) {
          return new sql.SqlFunction({
            functionName: new sql.RefName({ name: functionName, quotes: false }),
            specialParen: "none"
          });
        }, peg$c128 = '"', peg$c129 = peg$literalExpectation('"', false), peg$c130 = function(functionName, preLeftParen, postLeftParen, postArguments, filter) {
          var value = {
            functionName: makeFunctionName(functionName)
          };
          var spacing = value.spacing = {
            preLeftParen,
            postLeftParen
          };
          var keywords = value.keywords = {};
          value.args = sql.SeparatedArray.fromSingleValue(sql.SqlStar.PLAIN);
          spacing.postArguments = postArguments;
          if (filter) {
            spacing.preFilter = filter[0];
            keywords.filter = filter[1].filterKeyword;
            spacing.postFilter = filter[1].postFilter;
            value.whereClause = filter[1].whereClause;
          }
          return new sql.SqlFunction(value);
        }, peg$c131 = function(functionName, preLeftParen, postLeftParen, expr, separator, type, postArguments) {
          return new sql.SqlFunction({
            functionName: makeFunctionName(functionName),
            args: new sql.SeparatedArray([expr, type], [separator]),
            spacing: {
              preLeftParen,
              postLeftParen,
              postArguments
            }
          });
        }, peg$c132 = function(functionName, preLeftParen, postLeftParen, unit, separator, expr, postArguments) {
          var unitLiteral = new sql.SqlLiteral({
            value: unit.toUpperCase(),
            stringValue: unit
          });
          return new sql.SqlFunction({
            functionName: makeFunctionName(functionName),
            args: new sql.SeparatedArray([unitLiteral, expr], [separator]),
            spacing: {
              preLeftParen,
              postLeftParen,
              postArguments
            }
          });
        }, peg$c133 = function(functionName, preLeftParen, postLeftParen, decorator, expr, from, postArguments) {
          var value = {
            functionName: makeFunctionName(functionName),
            args: from ? new sql.SeparatedArray([expr, from[1]], [from[0]]) : sql.SeparatedArray.fromSingleValue(expr)
          };
          var spacing = value.spacing = {
            preLeftParen,
            postLeftParen,
            postArguments
          };
          var keywords = value.keywords = {};
          if (decorator) {
            value.decorator = decorator[0].toUpperCase();
            keywords.decorator = decorator[0];
            spacing.postDecorator = decorator[1];
          }
          return new sql.SqlFunction(value);
        }, peg$c134 = function(functionName, preLeftParen, postLeftParen, expr, separator, unit, postArguments) {
          return new sql.SqlFunction({
            functionName: makeFunctionName(functionName),
            args: new sql.SeparatedArray([expr, unit], [separator]),
            spacing: {
              preLeftParen,
              postLeftParen,
              postArguments
            }
          });
        }, peg$c135 = function(functionName, preLeftParen, postLeftParen, unit, tail, postArguments) {
          var value = {
            functionName: makeFunctionName(functionName)
          };
          var spacing = value.spacing = {
            preLeftParen,
            postLeftParen
          };
          value.args = makeSeparatedArray(unit, tail);
          spacing.postArguments = postArguments;
          return new sql.SqlFunction(value);
        }, peg$c136 = function(functionName, preLeftParen, postLeftParen, expr1, inSeparator, expr2, extra, postArguments) {
          var args = extra ? new sql.SeparatedArray([expr1, expr2, extra[1]], [inSeparator, extra[0]]) : new sql.SeparatedArray([expr1, expr2], [inSeparator]);
          return new sql.SqlFunction({
            functionName: makeFunctionName(functionName),
            args,
            spacing: {
              preLeftParen,
              postLeftParen,
              postArguments
            }
          });
        }, peg$c137 = "[", peg$c138 = peg$literalExpectation("[", false), peg$c139 = "]", peg$c140 = peg$literalExpectation("]", false), peg$c141 = function(functionName, preLeftParen, postLeftParen, head, tail, postArguments) {
          var value = {
            functionName: makeFunctionName(functionName),
            specialParen: "square"
          };
          var spacing = value.spacing = {
            preLeftParen,
            postLeftParen
          };
          if (head) {
            value.args = makeSeparatedArray(head, tail);
            spacing.postArguments = postArguments;
          }
          return new sql.SqlFunction(value);
        }, peg$c142 = function(filterKeyword, postFilter, postLeftParen, whereClause, preRightParen) {
          return {
            filterKeyword,
            postFilter,
            whereClause: whereClause.addParens(postLeftParen, preRightParen)
          };
        }, peg$c143 = function(postLeftParen, windowName, orderByClause) {
          var value = {};
          var spacing = value.spacing = {
            postLeftParen
          };
          if (windowName) {
            value.windowName = windowName[0];
            spacing.postWindowName = windowName[1];
          }
          if (orderByClause) {
            value.orderByClause = orderByClause[0];
            spacing.postOrderBy = orderByClause[1];
          }
          return new sql.SqlWindowSpec(value);
        }, peg$c144 = ",", peg$c145 = peg$literalExpectation(",", false), peg$c146 = function(left, right) {
          return new sql.Separator({
            left,
            separator: ",",
            right
          });
        }, peg$c147 = function(left, separator, right) {
          return new sql.Separator({
            left,
            separator,
            right
          });
        }, peg$c148 = function(record) {
          return record.unwrapIfSingleton();
        }, peg$c149 = function(row, postLeftParen, head, tail, postExpressions) {
          var value = {
            expressions: makeSeparatedArray(head, tail),
            spacing: {
              postLeftParen,
              postExpressions
            }
          };
          if (row) {
            value.keywords = { row: row[0] };
            value.spacing.postRow = row[1];
          } else {
            value.keywords = { row: "" };
          }
          return new sql.SqlRecord(value);
        }, peg$c150 = function(leftSpacing, ex, rightSpacing) {
          return ex.addParens(leftSpacing, rightSpacing);
        }, peg$c151 = function(values, postValues, head, tail, orderByClause, limitClause, offsetClause) {
          var value = {
            records: makeSeparatedArray(head, tail),
            keywords: {
              values
            }
          };
          var spacing = value.spacing = {
            postValues
          };
          if (orderByClause) {
            spacing.preOrderByClause = orderByClause[0];
            value.orderByClause = orderByClause[1];
          }
          if (limitClause) {
            spacing.preLimitClause = limitClause[0];
            value.limitClause = limitClause[1];
          }
          if (offsetClause) {
            spacing.preOffsetClause = offsetClause[0];
            value.offsetClause = offsetClause[1];
          }
          return new sql.SqlValues(value);
        }, peg$c152 = "?", peg$c153 = peg$literalExpectation("?", false), peg$c154 = function() {
          return new sql.SqlPlaceholder();
        }, peg$c155 = function(lit) {
          return new sql.SqlLiteral(lit);
        }, peg$c156 = function(v) {
          return new sql.SqlLiteral(v);
        }, peg$c157 = function() {
          return new sql.SqlType({ value: text() });
        }, peg$c158 = peg$otherExpectation("Number"), peg$c159 = "e", peg$c160 = peg$literalExpectation("e", true), peg$c161 = function() {
          var str = text();
          return {
            value: str.length > 15 && str.indexOf(".") === -1 ? BigInt(str) : +str,
            stringValue: str
          };
        }, peg$c162 = /^[0-9]/, peg$c163 = peg$classExpectation([["0", "9"]], false, false), peg$c164 = "'", peg$c165 = peg$literalExpectation("'", false), peg$c166 = "''", peg$c167 = peg$literalExpectation("''", false), peg$c168 = /^[^']/, peg$c169 = peg$classExpectation(["'"], true, false), peg$c170 = function(v) {
          return {
            value: v.replace(/''/g, "'"),
            stringValue: text()
          };
        }, peg$c171 = "u&", peg$c172 = peg$literalExpectation("U&", true), peg$c173 = function(v) {
          return {
            value: v.value.replace(/\\(\\|(?:[0-9a-f]{4}))/gi, function(_, s) {
              return s === "\\" ? "\\" : String.fromCharCode(parseInt(s, 16));
            }),
            stringValue: text()
          };
        }, peg$c174 = /^[a-z0-9]/i, peg$c175 = peg$classExpectation([["a", "z"], ["0", "9"]], false, true), peg$c176 = /^[a-z0-9_\-]/i, peg$c177 = peg$classExpectation([["a", "z"], ["0", "9"], "_", "-"], false, true), peg$c178 = function(v) {
          return {
            value: v.value,
            stringValue: text()
          };
        }, peg$c179 = "x'", peg$c180 = peg$literalExpectation("X'", true), peg$c181 = /^[0-9A-F]/i, peg$c182 = peg$classExpectation([["0", "9"], ["A", "F"]], false, true), peg$c183 = function(v) {
          return {
            value: v,
            stringValue: text()
          };
        }, peg$c184 = function(timestamp, postTimestamp, v) {
          return {
            spacing: {
              postTimestamp
            },
            keywords: {
              timestamp
            },
            value: new Date(v.value.replace(" ", "T") + "Z"),
            stringValue: v.stringValue
          };
        }, peg$c185 = function(head, tail) {
          return makeListMap(tail, 1, head);
        }, peg$c186 = function(a, b, c) {
          if (c) {
            return new sql.SqlColumn({
              refName: c[3],
              spacing: {
                postTable: c[0],
                postDot: c[2]
              },
              table: new sql.SqlTable({
                refName: b[3],
                spacing: {
                  postNamespace: b[0],
                  postDot: b[2]
                },
                namespace: new sql.SqlNamespace({
                  refName: a
                })
              })
            });
          } else if (b) {
            return new sql.SqlColumn({
              refName: b[3],
              spacing: {
                postTable: b[0],
                postDot: b[2]
              },
              table: new sql.SqlTable({
                refName: a
              })
            });
          } else {
            return new sql.SqlColumn({
              refName: a
            });
          }
        }, peg$c187 = function(a, b) {
          if (b) {
            return new sql.SqlTable({
              refName: b[3],
              spacing: {
                postTable: b[0],
                postDot: b[2]
              },
              table: new sql.SqlNamespace({
                refName: a
              })
            });
          } else {
            return new sql.SqlTable({
              refName: a
            });
          }
        }, peg$c188 = '""', peg$c189 = peg$literalExpectation('""', false), peg$c190 = /^[^"]/, peg$c191 = peg$classExpectation(['"'], true, false), peg$c192 = function(name) {
          return new sql.RefName({
            name: name.replace(/""/g, '"'),
            quotes: true
          });
        }, peg$c193 = function(v) {
          return new sql.RefName({
            name: v.name.replace(/\\(\\|(?:[0-9a-f]{4}))/gi, function(_, s) {
              return s === "\\" ? "\\" : String.fromCharCode(parseInt(s, 16));
            }),
            quotes: true
          });
        }, peg$c194 = function(name) {
          return !sql.RefName.isReservedKeyword(name);
        }, peg$c195 = function(name) {
          return new sql.RefName({
            name: text(),
            quotes: false
          });
        }, peg$c196 = function(name) {
          return !sql.RefName.isReservedAlias(name);
        }, peg$c197 = function(name) {
          return !sql.RefName.isReservedFunctionName(name);
        }, peg$c198 = /^[a-z_]/i, peg$c199 = peg$classExpectation([["a", "z"], "_"], false, true), peg$c200 = /^[a-z0-9_]/i, peg$c201 = peg$classExpectation([["a", "z"], ["0", "9"], "_"], false, true), peg$c202 = function(a, b) {
          if (!a)
            return sql.SqlStar.PLAIN;
          var last = b || a;
          return new sql.SqlStar({
            spacing: {
              postTable: last[1],
              postDot: last[3]
            },
            table: b ? new sql.SqlTable({
              refName: b[0],
              spacing: {
                postNamespace: a[1],
                postDot: a[3]
              },
              namespace: new sql.SqlNamespace({
                refName: a[0]
              })
            }) : new sql.SqlTable({
              refName: a[0]
            })
          });
        }, peg$c203 = peg$otherExpectation("optional whitespace"), peg$c204 = peg$otherExpectation("mandatory whitespace"), peg$c205 = peg$otherExpectation("pure whitespace"), peg$c206 = "--", peg$c207 = peg$literalExpectation("--", false), peg$c208 = /^[^\n]/, peg$c209 = peg$classExpectation(["\n"], true, false), peg$c210 = /^[\n]/, peg$c211 = peg$classExpectation(["\n"], false, false), peg$c212 = "/*", peg$c213 = peg$literalExpectation("/*", false), peg$c214 = "*/", peg$c215 = peg$literalExpectation("*/", false), peg$c216 = peg$anyExpectation(), peg$c217 = /^[ \t\n\r]/, peg$c218 = peg$classExpectation([" ", "	", "\n", "\r"], false, false), peg$c219 = peg$otherExpectation("("), peg$c220 = "(", peg$c221 = peg$literalExpectation("(", false), peg$c222 = peg$otherExpectation(")"), peg$c223 = ")", peg$c224 = peg$literalExpectation(")", false), peg$c225 = "all", peg$c226 = peg$literalExpectation("ALL", true), peg$c227 = "and", peg$c228 = peg$literalExpectation("AND", true), peg$c229 = "any", peg$c230 = peg$literalExpectation("ANY", true), peg$c231 = "array", peg$c232 = peg$literalExpectation("ARRAY", true), peg$c233 = "as", peg$c234 = peg$literalExpectation("AS", true), peg$c235 = "asc", peg$c236 = peg$literalExpectation("ASC", true), peg$c237 = "between", peg$c238 = peg$literalExpectation("BETWEEN", true), peg$c239 = "both", peg$c240 = peg$literalExpectation("BOTH", true), peg$c241 = "case", peg$c242 = peg$literalExpectation("CASE", true), peg$c243 = "cast", peg$c244 = peg$literalExpectation("CAST", true), peg$c245 = "ceil", peg$c246 = peg$literalExpectation("CEIL", true), peg$c247 = "clustered", peg$c248 = peg$literalExpectation("CLUSTERED", true), peg$c249 = "by", peg$c250 = peg$literalExpectation("BY", true), peg$c251 = "count", peg$c252 = peg$literalExpectation("COUNT", true), peg$c253 = "cube", peg$c254 = peg$literalExpectation("CUBE", true), peg$c255 = "date", peg$c256 = peg$literalExpectation("DATE", true), peg$c257 = "desc", peg$c258 = peg$literalExpectation("DESC", true), peg$c259 = "distinct", peg$c260 = peg$literalExpectation("DISTINCT", true), peg$c261 = "else", peg$c262 = peg$literalExpectation("ELSE", true), peg$c263 = "end", peg$c264 = peg$literalExpectation("END", true), peg$c265 = "escape", peg$c266 = peg$literalExpectation("ESCAPE", true), peg$c267 = "explain", peg$c268 = peg$literalExpectation("EXPLAIN", true), peg$c269 = "plan", peg$c270 = peg$literalExpectation("PLAN", true), peg$c271 = "for", peg$c272 = peg$literalExpectation("FOR", true), peg$c273 = "extend", peg$c274 = peg$literalExpectation("EXTEND", true), peg$c275 = "extract", peg$c276 = peg$literalExpectation("EXTRACT", true), peg$c277 = "false", peg$c278 = peg$literalExpectation("FALSE", true), peg$c279 = function() {
          return { value: false, stringValue: text() };
        }, peg$c280 = "filter", peg$c281 = peg$literalExpectation("FILTER", true), peg$c282 = "floor", peg$c283 = peg$literalExpectation("FLOOR", true), peg$c284 = "from", peg$c285 = peg$literalExpectation("FROM", true), peg$c286 = "group", peg$c287 = peg$literalExpectation("GROUP", true), peg$c288 = "grouping", peg$c289 = peg$literalExpectation("GROUPING", true), peg$c290 = "sets", peg$c291 = peg$literalExpectation("SETS", true), peg$c292 = "having", peg$c293 = peg$literalExpectation("HAVING", true), peg$c294 = "in", peg$c295 = peg$literalExpectation("IN", true), peg$c296 = "insert", peg$c297 = peg$literalExpectation("INSERT", true), peg$c298 = "interval", peg$c299 = peg$literalExpectation("INTERVAL", true), peg$c300 = "into", peg$c301 = peg$literalExpectation("INTO", true), peg$c302 = "is", peg$c303 = peg$literalExpectation("IS", true), peg$c304 = "join", peg$c305 = peg$literalExpectation("JOIN", true), peg$c306 = "leading", peg$c307 = peg$literalExpectation("LEADING", true), peg$c308 = "like", peg$c309 = peg$literalExpectation("LIKE", true), peg$c310 = "limit", peg$c311 = peg$literalExpectation("LIMIT", true), peg$c312 = "not", peg$c313 = peg$literalExpectation("NOT", true), peg$c314 = "null", peg$c315 = peg$literalExpectation("NULL", true), peg$c316 = function() {
          return { value: null, stringValue: text() };
        }, peg$c317 = "offset", peg$c318 = peg$literalExpectation("OFFSET", true), peg$c319 = "on", peg$c320 = peg$literalExpectation("ON", true), peg$c321 = "or", peg$c322 = peg$literalExpectation("OR", true), peg$c323 = "order", peg$c324 = peg$literalExpectation("ORDER", true), peg$c325 = "outer", peg$c326 = peg$literalExpectation("OUTER", true), peg$c327 = "over", peg$c328 = peg$literalExpectation("OVER", true), peg$c329 = "overwrite", peg$c330 = peg$literalExpectation("OVERWRITE", true), peg$c331 = "partitioned", peg$c332 = peg$literalExpectation("PARTITIONED", true), peg$c333 = "position", peg$c334 = peg$literalExpectation("POSITION", true), peg$c335 = "replace", peg$c336 = peg$literalExpectation("REPLACE", true), peg$c337 = "rollup", peg$c338 = peg$literalExpectation("ROLLUP", true), peg$c339 = "row", peg$c340 = peg$literalExpectation("ROW", true), peg$c341 = "select", peg$c342 = peg$literalExpectation("SELECT", true), peg$c343 = "similar", peg$c344 = peg$literalExpectation("SIMILAR", true), peg$c345 = "some", peg$c346 = peg$literalExpectation("SOME", true), peg$c347 = "symmetric", peg$c348 = peg$literalExpectation("SYMMETRIC", true), peg$c349 = "table", peg$c350 = peg$literalExpectation("TABLE", true), peg$c351 = "then", peg$c352 = peg$literalExpectation("THEN", true), peg$c353 = "time", peg$c354 = peg$literalExpectation("TIME", true), peg$c355 = "timestamp", peg$c356 = peg$literalExpectation("TIMESTAMP", true), peg$c357 = "timestampadd", peg$c358 = peg$literalExpectation("TIMESTAMPADD", true), peg$c359 = "timestampdiff", peg$c360 = peg$literalExpectation("TIMESTAMPDIFF", true), peg$c361 = "to", peg$c362 = peg$literalExpectation("TO", true), peg$c363 = "trailing", peg$c364 = peg$literalExpectation("TRAILING", true), peg$c365 = "trim", peg$c366 = peg$literalExpectation("TRIM", true), peg$c367 = "true", peg$c368 = peg$literalExpectation("TRUE", true), peg$c369 = function() {
          return { value: true, stringValue: text() };
        }, peg$c370 = "union", peg$c371 = peg$literalExpectation("UNION", true), peg$c372 = "values", peg$c373 = peg$literalExpectation("VALUES", true), peg$c374 = "when", peg$c375 = peg$literalExpectation("WHEN", true), peg$c376 = "where", peg$c377 = peg$literalExpectation("WHERE", true), peg$c378 = "with", peg$c379 = peg$literalExpectation("WITH", true), peg$currPos = 0, peg$savedPos = 0, peg$posDetailsCache = [{ line: 1, column: 1 }], peg$maxFailPos = 0, peg$maxFailExpected = [], peg$silentFails = 0, peg$result;
        if ("startRule" in options) {
          if (!(options.startRule in peg$startRuleFunctions)) {
            throw new Error(`Can't start parsing from rule "` + options.startRule + '".');
          }
          peg$startRuleFunction = peg$startRuleFunctions[options.startRule];
        }
        function text() {
          return input.substring(peg$savedPos, peg$currPos);
        }
        function location() {
          return peg$computeLocation(peg$savedPos, peg$currPos);
        }
        function expected(description, location2) {
          location2 = location2 !== void 0 ? location2 : peg$computeLocation(peg$savedPos, peg$currPos);
          throw peg$buildStructuredError([peg$otherExpectation(description)], input.substring(peg$savedPos, peg$currPos), location2);
        }
        function error(message, location2) {
          location2 = location2 !== void 0 ? location2 : peg$computeLocation(peg$savedPos, peg$currPos);
          throw peg$buildSimpleError(message, location2);
        }
        function peg$literalExpectation(text2, ignoreCase) {
          return { type: "literal", text: text2, ignoreCase };
        }
        function peg$classExpectation(parts, inverted, ignoreCase) {
          return { type: "class", parts, inverted, ignoreCase };
        }
        function peg$anyExpectation() {
          return { type: "any" };
        }
        function peg$endExpectation() {
          return { type: "end" };
        }
        function peg$otherExpectation(description) {
          return { type: "other", description };
        }
        function peg$computePosDetails(pos) {
          var details = peg$posDetailsCache[pos], p2;
          if (details) {
            return details;
          } else {
            p2 = pos - 1;
            while (!peg$posDetailsCache[p2]) {
              p2--;
            }
            details = peg$posDetailsCache[p2];
            details = {
              line: details.line,
              column: details.column
            };
            while (p2 < pos) {
              if (input.charCodeAt(p2) === 10) {
                details.line++;
                details.column = 1;
              } else {
                details.column++;
              }
              p2++;
            }
            peg$posDetailsCache[pos] = details;
            return details;
          }
        }
        function peg$computeLocation(startPos, endPos) {
          var startPosDetails = peg$computePosDetails(startPos), endPosDetails = peg$computePosDetails(endPos);
          return {
            start: {
              offset: startPos,
              line: startPosDetails.line,
              column: startPosDetails.column
            },
            end: {
              offset: endPos,
              line: endPosDetails.line,
              column: endPosDetails.column
            }
          };
        }
        function peg$fail(expected2) {
          if (peg$currPos < peg$maxFailPos) {
            return;
          }
          if (peg$currPos > peg$maxFailPos) {
            peg$maxFailPos = peg$currPos;
            peg$maxFailExpected = [];
          }
          peg$maxFailExpected.push(expected2);
        }
        function peg$buildSimpleError(message, location2) {
          return new peg$SyntaxError(message, null, null, location2);
        }
        function peg$buildStructuredError(expected2, found, location2) {
          return new peg$SyntaxError(peg$SyntaxError.buildMessage(expected2, found), expected2, found, location2);
        }
        function peg$parseStart() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseSqlQuery();
            if (s2 === peg$FAILED) {
              s2 = peg$parseSqlAlias();
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c0(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlAlias() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseOrExpression();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseAsToken();
              if (s5 !== peg$FAILED) {
                s4 = [s4, s5];
                s3 = s4;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 === peg$FAILED) {
              s3 = null;
            }
            if (s3 !== peg$FAILED) {
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseRefNameAlias();
                if (s5 !== peg$FAILED) {
                  s3 = [s3, s4, s5];
                  s2 = s3;
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseSqlColumnList();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c1(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseGeneralFunctionArgument() {
          var s0;
          s0 = peg$parseSqlLabeledExpression();
          if (s0 === peg$FAILED) {
            s0 = peg$parseSqlAlias();
          }
          return s0;
        }
        function peg$parseSqlLabeledExpression() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseRefNameAlias();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2) === peg$c2) {
                s3 = peg$c2;
                peg$currPos += 2;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c3);
                }
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseOrExpression();
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c4(s1, s2, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlExtendClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseExtendToken();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseOpenParen();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSqlColumnDeclaration();
                if (s4 !== peg$FAILED) {
                  s5 = [];
                  s6 = peg$currPos;
                  s7 = peg$parseCommaSeparator();
                  if (s7 !== peg$FAILED) {
                    s8 = peg$parseSqlColumnDeclaration();
                    if (s8 !== peg$FAILED) {
                      s7 = [s7, s8];
                      s6 = s7;
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                  while (s6 !== peg$FAILED) {
                    s5.push(s6);
                    s6 = peg$currPos;
                    s7 = peg$parseCommaSeparator();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseSqlColumnDeclaration();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parse_();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseCloseParen();
                      if (s7 !== peg$FAILED) {
                        peg$savedPos = s0;
                        s1 = peg$c5(s1, s3, s4, s5, s6);
                        s0 = s1;
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlColumnDeclaration() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseRefName();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlType();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c6(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlQuery() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseExplainPlanForToken();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parseInsertClause();
            if (s3 !== peg$FAILED) {
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s3 = [s3, s4];
                s2 = s3;
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseReplaceClause();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$currPos;
                s5 = peg$currPos;
                s6 = peg$parseWithClause();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parse_();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 === peg$FAILED) {
                  s5 = null;
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseQueryHeart();
                  if (s6 !== peg$FAILED) {
                    s5 = [s5, s6];
                    s4 = s5;
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
                if (s4 === peg$FAILED) {
                  s4 = peg$currPos;
                  s5 = peg$parseWithClause();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parse_();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseOpenParen();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parse_();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parseSqlQuery();
                          if (s9 !== peg$FAILED) {
                            s10 = peg$parse_();
                            if (s10 !== peg$FAILED) {
                              s11 = peg$parseCloseParen();
                              if (s11 !== peg$FAILED) {
                                s5 = [s5, s6, s7, s8, s9, s10, s11];
                                s4 = s5;
                              } else {
                                peg$currPos = s4;
                                s4 = peg$FAILED;
                              }
                            } else {
                              peg$currPos = s4;
                              s4 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s4;
                            s4 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s4;
                          s4 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s4;
                        s4 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s4;
                      s4 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseOrderByClause();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseLimitClause();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$currPos;
                      s8 = peg$parse_();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOffsetClause();
                        if (s9 !== peg$FAILED) {
                          s8 = [s8, s9];
                          s7 = s8;
                        } else {
                          peg$currPos = s7;
                          s7 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                      if (s7 === peg$FAILED) {
                        s7 = null;
                      }
                      if (s7 !== peg$FAILED) {
                        s8 = peg$currPos;
                        s9 = peg$parse_();
                        if (s9 !== peg$FAILED) {
                          s10 = peg$parsePartitionedByClause();
                          if (s10 !== peg$FAILED) {
                            s9 = [s9, s10];
                            s8 = s9;
                          } else {
                            peg$currPos = s8;
                            s8 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s8;
                          s8 = peg$FAILED;
                        }
                        if (s8 === peg$FAILED) {
                          s8 = null;
                        }
                        if (s8 !== peg$FAILED) {
                          s9 = peg$currPos;
                          s10 = peg$parse_();
                          if (s10 !== peg$FAILED) {
                            s11 = peg$parseClusteredByClause();
                            if (s11 !== peg$FAILED) {
                              s10 = [s10, s11];
                              s9 = s10;
                            } else {
                              peg$currPos = s9;
                              s9 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s9;
                            s9 = peg$FAILED;
                          }
                          if (s9 === peg$FAILED) {
                            s9 = null;
                          }
                          if (s9 !== peg$FAILED) {
                            s10 = peg$currPos;
                            s11 = peg$parse_();
                            if (s11 !== peg$FAILED) {
                              s12 = peg$parseUnionClause();
                              if (s12 !== peg$FAILED) {
                                s11 = [s11, s12];
                                s10 = s11;
                              } else {
                                peg$currPos = s10;
                                s10 = peg$FAILED;
                              }
                            } else {
                              peg$currPos = s10;
                              s10 = peg$FAILED;
                            }
                            if (s10 === peg$FAILED) {
                              s10 = null;
                            }
                            if (s10 !== peg$FAILED) {
                              peg$savedPos = s0;
                              s1 = peg$c7(s1, s2, s3, s4, s5, s6, s7, s8, s9, s10);
                              s0 = s1;
                            } else {
                              peg$currPos = s0;
                              s0 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseQueryHeart() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseSelectClause();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s4 = peg$parseFromClause();
              if (s4 !== peg$FAILED) {
                s3 = [s3, s4];
                s2 = s3;
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseSqlWhereClause();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$currPos;
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseGroupByClause();
                  if (s6 !== peg$FAILED) {
                    s5 = [s5, s6];
                    s4 = s5;
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
                if (s4 === peg$FAILED) {
                  s4 = null;
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseHavingClause();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c8(s1, s2, s3, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseInsertClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$parseInsertToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse__();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseIntoToken();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse__();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseSqlTable();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseSqlColumnList();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      peg$savedPos = s0;
                      s1 = peg$c9(s1, s2, s3, s4, s5, s6);
                      s0 = s1;
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseReplaceClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10;
          s0 = peg$currPos;
          s1 = peg$parseReplaceToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse__();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseIntoToken();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse__();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseSqlTable();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseSqlColumnList();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse__();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parseOverwriteToken();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parse__();
                          if (s9 !== peg$FAILED) {
                            s10 = peg$parseAllToken();
                            if (s10 === peg$FAILED) {
                              s10 = peg$parseSqlWhereClause();
                            }
                            if (s10 !== peg$FAILED) {
                              peg$savedPos = s0;
                              s1 = peg$c10(s1, s2, s3, s4, s5, s6, s7, s8, s9, s10);
                              s0 = s1;
                            } else {
                              peg$currPos = s0;
                              s0 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseWithClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseWithToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlWithPart();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseSqlWithPart();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseSqlWithPart();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c11(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlWithPart() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$parseRefName();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseSqlColumnList();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseAsToken();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseSqlQueryInParens();
                    if (s6 !== peg$FAILED) {
                      peg$savedPos = s0;
                      s1 = peg$c12(s1, s2, s3, s4, s5, s6);
                      s0 = s1;
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlColumnList() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseOpenParen();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseRefName();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseRefName();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseRefName();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseCloseParen();
                    if (s6 !== peg$FAILED) {
                      peg$savedPos = s0;
                      s1 = peg$c13(s2, s3, s4, s5);
                      s0 = s1;
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSelectClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$parseSelectToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseAllToken();
              if (s4 === peg$FAILED) {
                s4 = peg$parseDistinctToken();
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSqlStarOrAliasExpression();
                if (s4 !== peg$FAILED) {
                  s5 = [];
                  s6 = peg$currPos;
                  s7 = peg$parseCommaSeparator();
                  if (s7 !== peg$FAILED) {
                    s8 = peg$parseSqlStarOrAliasExpression();
                    if (s8 !== peg$FAILED) {
                      s7 = [s7, s8];
                      s6 = s7;
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                  while (s6 !== peg$FAILED) {
                    s5.push(s6);
                    s6 = peg$currPos;
                    s7 = peg$parseCommaSeparator();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseSqlStarOrAliasExpression();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c14(s1, s2, s3, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlStarOrAliasExpression() {
          var s0;
          s0 = peg$parseSqlStar();
          if (s0 === peg$FAILED) {
            s0 = peg$parseSqlAlias();
          }
          return s0;
        }
        function peg$parseFromClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseFromToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlAlias();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseSqlAlias();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseSqlAlias();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseJoinClauses();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c15(s1, s2, s3, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseJoinClauses() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseSqlJoinPart();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseSqlJoinPart();
              if (s5 !== peg$FAILED) {
                s4 = [s4, s5];
                s3 = s4;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseSqlJoinPart();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c16(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlJoinPart() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseJoinType();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseJoinToken();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSqlAlias();
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseOnToken();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parse_();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOrExpression();
                        if (s9 !== peg$FAILED) {
                          s6 = [s6, s7, s8, s9];
                          s5 = s6;
                        } else {
                          peg$currPos = s5;
                          s5 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s5;
                        s5 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c17(s1, s2, s3, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseJoinType() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c18) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c19);
            }
          }
          if (s2 === peg$FAILED) {
            if (input.substr(peg$currPos, 5).toLowerCase() === peg$c20) {
              s2 = input.substr(peg$currPos, 5);
              peg$currPos += 5;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c21);
              }
            }
            if (s2 === peg$FAILED) {
              if (input.substr(peg$currPos, 4).toLowerCase() === peg$c22) {
                s2 = input.substr(peg$currPos, 4);
                peg$currPos += 4;
              } else {
                s2 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c23);
                }
              }
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            s4 = peg$parse__();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseOuterToken();
              if (s5 !== peg$FAILED) {
                s4 = [s4, s5];
                s3 = s4;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 === peg$FAILED) {
              s3 = null;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          if (s0 === peg$FAILED) {
            if (input.substr(peg$currPos, 5).toLowerCase() === peg$c24) {
              s0 = input.substr(peg$currPos, 5);
              peg$currPos += 5;
            } else {
              s0 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c25);
              }
            }
            if (s0 === peg$FAILED) {
              if (input.substr(peg$currPos, 5).toLowerCase() === peg$c26) {
                s0 = input.substr(peg$currPos, 5);
                peg$currPos += 5;
              } else {
                s0 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c27);
                }
              }
            }
          }
          return s0;
        }
        function peg$parseSqlWhereClause() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseWhereToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOrExpression();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c28(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseGroupByClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseGroupByToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseRollupToken();
              if (s4 === peg$FAILED) {
                s4 = peg$parseCubeToken();
                if (s4 === peg$FAILED) {
                  s4 = peg$parseGroupingSetsToken();
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$currPos;
                s5 = peg$parseOpenParen();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$currPos;
                    s8 = peg$parseGroupByExpressionList();
                    if (s8 !== peg$FAILED) {
                      s9 = peg$parse_();
                      if (s9 !== peg$FAILED) {
                        s8 = [s8, s9];
                        s7 = s8;
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s7;
                      s7 = peg$FAILED;
                    }
                    if (s7 === peg$FAILED) {
                      s7 = null;
                    }
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseCloseParen();
                      if (s8 !== peg$FAILED) {
                        s5 = [s5, s6, s7, s8];
                        s4 = s5;
                      } else {
                        peg$currPos = s4;
                        s4 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s4;
                      s4 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
                if (s4 === peg$FAILED) {
                  s4 = peg$parseGroupByExpressionList();
                }
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c29(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseGroupByExpressionList() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseGroupByExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parseCommaSeparator();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseGroupByExpression();
              if (s5 !== peg$FAILED) {
                s4 = [s4, s5];
                s3 = s4;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parseCommaSeparator();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseGroupByExpression();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c16(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseGroupByExpression() {
          var s0;
          s0 = peg$parseSqlEmptyRecord();
          if (s0 === peg$FAILED) {
            s0 = peg$parseOrExpression();
          }
          return s0;
        }
        function peg$parseSqlEmptyRecord() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseOpenParen();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseCloseParen();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c30(s2);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseHavingClause() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseHavingToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOrExpression();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c31(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseOrderByClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseOrderByToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlOrderByExpression();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseSqlOrderByExpression();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseSqlOrderByExpression();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c32(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlOrderByExpression() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$parseOrExpression();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s4 = peg$parseAscToken();
              if (s4 === peg$FAILED) {
                s4 = peg$parseDescToken();
              }
              if (s4 !== peg$FAILED) {
                s3 = [s3, s4];
                s2 = s3;
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c33(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseLimitClause() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseLimitToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlLiteral();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c34(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseOffsetClause() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseOffsetToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlLiteral();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c35(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parsePartitionedByClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parsePartitionedByToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseTimeUnitLiteral();
              if (s3 === peg$FAILED) {
                s3 = peg$parseOrExpression();
                if (s3 === peg$FAILED) {
                  s3 = peg$currPos;
                  s4 = peg$parseAllToken();
                  if (s4 !== peg$FAILED) {
                    s5 = peg$currPos;
                    s6 = peg$parse__();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseTimeToken();
                      if (s7 !== peg$FAILED) {
                        s6 = [s6, s7];
                        s5 = s6;
                      } else {
                        peg$currPos = s5;
                        s5 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                    if (s5 === peg$FAILED) {
                      s5 = null;
                    }
                    if (s5 !== peg$FAILED) {
                      s4 = [s4, s5];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c36(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseClusteredByClause() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseClusteredByToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOrExpression();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseOrExpression();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseOrExpression();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c37(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnionClause() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseUnionToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlQuery();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c38(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseOrExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseAndExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseOrToken();
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseAndExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseOrToken();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseAndExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c39(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseAndExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseNotExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseAndToken();
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseNotExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseAndToken();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseNotExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c40(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseNotExpression() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseNotToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseNotExpression();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c41(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          if (s0 === peg$FAILED) {
            s0 = peg$parseComparisonExpression();
          }
          return s0;
        }
        function peg$parseComparisonExpression() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$parseAdditionExpression();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s4 = peg$parseComparisonOpRhs();
              if (s4 !== peg$FAILED) {
                s3 = [s3, s4];
                s2 = s3;
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c42(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOpRhs() {
          var s0;
          s0 = peg$parseComparisonOpRhsSimple();
          if (s0 === peg$FAILED) {
            s0 = peg$parseComparisonOpRhsIs();
            if (s0 === peg$FAILED) {
              s0 = peg$parseComparisonOpRhsIn();
              if (s0 === peg$FAILED) {
                s0 = peg$parseComparisonOpRhsBetween();
                if (s0 === peg$FAILED) {
                  s0 = peg$parseComparisonOpRhsLike();
                  if (s0 === peg$FAILED) {
                    s0 = peg$parseComparisonOpRhsNot();
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseComparisonOpRhsSimple() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$parseComparisonOperator();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseAdditionExpression();
              if (s3 === peg$FAILED) {
                s3 = peg$currPos;
                s4 = peg$parseComparisonDecorator();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseSqlQueryInParens();
                    if (s6 !== peg$FAILED) {
                      s4 = [s4, s5, s6];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c43(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOperator() {
          var s0;
          if (input.charCodeAt(peg$currPos) === 61) {
            s0 = peg$c44;
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c45);
            }
          }
          if (s0 === peg$FAILED) {
            if (input.substr(peg$currPos, 2) === peg$c46) {
              s0 = peg$c46;
              peg$currPos += 2;
            } else {
              s0 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c47);
              }
            }
            if (s0 === peg$FAILED) {
              if (input.substr(peg$currPos, 2) === peg$c48) {
                s0 = peg$c48;
                peg$currPos += 2;
              } else {
                s0 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c49);
                }
              }
              if (s0 === peg$FAILED) {
                if (input.substr(peg$currPos, 2) === peg$c50) {
                  s0 = peg$c50;
                  peg$currPos += 2;
                } else {
                  s0 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c51);
                  }
                }
                if (s0 === peg$FAILED) {
                  if (input.substr(peg$currPos, 2) === peg$c52) {
                    s0 = peg$c52;
                    peg$currPos += 2;
                  } else {
                    s0 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c53);
                    }
                  }
                  if (s0 === peg$FAILED) {
                    if (input.charCodeAt(peg$currPos) === 60) {
                      s0 = peg$c54;
                      peg$currPos++;
                    } else {
                      s0 = peg$FAILED;
                      if (peg$silentFails === 0) {
                        peg$fail(peg$c55);
                      }
                    }
                    if (s0 === peg$FAILED) {
                      if (input.charCodeAt(peg$currPos) === 62) {
                        s0 = peg$c56;
                        peg$currPos++;
                      } else {
                        s0 = peg$FAILED;
                        if (peg$silentFails === 0) {
                          peg$fail(peg$c57);
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseComparisonDecorator() {
          var s0;
          s0 = peg$parseAnyToken();
          if (s0 === peg$FAILED) {
            s0 = peg$parseAllToken();
            if (s0 === peg$FAILED) {
              s0 = peg$parseSomeToken();
            }
          }
          return s0;
        }
        function peg$parseComparisonOpRhsIs() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseIsToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseNotToken();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSqlLiteral();
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c58(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOpRhsIn() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseInToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlQueryInParens();
              if (s3 === peg$FAILED) {
                s3 = peg$parseSqlRecord();
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c59(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOpRhsBetween() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$parseBetweenToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseSymmetricToken();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseAdditionExpression();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseAndToken();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse_();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parseAdditionExpression();
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c60(s1, s2, s3, s4, s5, s6, s7, s8);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOpRhsLike() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$parseLikeToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseSimilarToToken();
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseAdditionExpression();
              if (s3 !== peg$FAILED) {
                s4 = peg$currPos;
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseEscapeToken();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseAdditionExpression();
                      if (s8 !== peg$FAILED) {
                        s5 = [s5, s6, s7, s8];
                        s4 = s5;
                      } else {
                        peg$currPos = s4;
                        s4 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s4;
                      s4 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
                if (s4 === peg$FAILED) {
                  s4 = null;
                }
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c61(s1, s2, s3, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseComparisonOpRhsNot() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseNotToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseComparisonOpRhsIn();
              if (s3 === peg$FAILED) {
                s3 = peg$parseComparisonOpRhsBetween();
                if (s3 === peg$FAILED) {
                  s3 = peg$parseComparisonOpRhsLike();
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c62(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseAdditionExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseSubtractionExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 43) {
                s5 = peg$c63;
                peg$currPos++;
              } else {
                s5 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c64);
                }
              }
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseSubtractionExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 43) {
                  s5 = peg$c63;
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c64);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseSubtractionExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c65(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSubtractionExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseMultiplicationExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              s5 = peg$currPos;
              s6 = peg$currPos;
              if (input.charCodeAt(peg$currPos) === 45) {
                s7 = peg$c66;
                peg$currPos++;
              } else {
                s7 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c67);
                }
              }
              if (s7 !== peg$FAILED) {
                s8 = peg$currPos;
                peg$silentFails++;
                if (input.charCodeAt(peg$currPos) === 45) {
                  s9 = peg$c66;
                  peg$currPos++;
                } else {
                  s9 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c67);
                  }
                }
                peg$silentFails--;
                if (s9 === peg$FAILED) {
                  s8 = void 0;
                } else {
                  peg$currPos = s8;
                  s8 = peg$FAILED;
                }
                if (s8 !== peg$FAILED) {
                  s7 = [s7, s8];
                  s6 = s7;
                } else {
                  peg$currPos = s6;
                  s6 = peg$FAILED;
                }
              } else {
                peg$currPos = s6;
                s6 = peg$FAILED;
              }
              if (s6 !== peg$FAILED) {
                s5 = input.substring(s5, peg$currPos);
              } else {
                s5 = s6;
              }
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseMultiplicationExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                s6 = peg$currPos;
                if (input.charCodeAt(peg$currPos) === 45) {
                  s7 = peg$c66;
                  peg$currPos++;
                } else {
                  s7 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c67);
                  }
                }
                if (s7 !== peg$FAILED) {
                  s8 = peg$currPos;
                  peg$silentFails++;
                  if (input.charCodeAt(peg$currPos) === 45) {
                    s9 = peg$c66;
                    peg$currPos++;
                  } else {
                    s9 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c67);
                    }
                  }
                  peg$silentFails--;
                  if (s9 === peg$FAILED) {
                    s8 = void 0;
                  } else {
                    peg$currPos = s8;
                    s8 = peg$FAILED;
                  }
                  if (s8 !== peg$FAILED) {
                    s7 = [s7, s8];
                    s6 = s7;
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s6;
                  s6 = peg$FAILED;
                }
                if (s6 !== peg$FAILED) {
                  s5 = input.substring(s5, peg$currPos);
                } else {
                  s5 = s6;
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseMultiplicationExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c68(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseMultiplicationExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseDivisionExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 42) {
                s5 = peg$c69;
                peg$currPos++;
              } else {
                s5 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c70);
                }
              }
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseDivisionExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 42) {
                  s5 = peg$c69;
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c70);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseDivisionExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c71(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseDivisionExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseUnaryExpression();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 47) {
                s5 = peg$c72;
                peg$currPos++;
              } else {
                s5 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c73);
                }
              }
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseUnaryExpression();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 47) {
                  s5 = peg$c72;
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c73);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseUnaryExpression();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c74(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnaryExpression() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          if (peg$c75.test(input.charAt(peg$currPos))) {
            s1 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c76);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              peg$silentFails++;
              s4 = peg$parseNumber();
              peg$silentFails--;
              if (s4 === peg$FAILED) {
                s3 = void 0;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseConcatExpression();
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c77(s1, s2, s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          if (s0 === peg$FAILED) {
            s0 = peg$parseConcatExpression();
          }
          return s0;
        }
        function peg$parseConcatExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseBaseType();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parse_();
            if (s4 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2) === peg$c78) {
                s5 = peg$c78;
                peg$currPos += 2;
              } else {
                s5 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c79);
                }
              }
              if (s5 !== peg$FAILED) {
                s6 = peg$parse_();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseBaseType();
                  if (s7 !== peg$FAILED) {
                    s4 = [s4, s5, s6, s7];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.substr(peg$currPos, 2) === peg$c78) {
                  s5 = peg$c78;
                  peg$currPos += 2;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c79);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseBaseType();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c80(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseBaseType() {
          var s0;
          s0 = peg$parseSqlPlaceholder();
          if (s0 === peg$FAILED) {
            s0 = peg$parseInterval();
            if (s0 === peg$FAILED) {
              s0 = peg$parseCaseExpression();
              if (s0 === peg$FAILED) {
                s0 = peg$parseFunction();
                if (s0 === peg$FAILED) {
                  s0 = peg$parseSqlValues();
                  if (s0 === peg$FAILED) {
                    s0 = peg$parseSqlLiteral();
                    if (s0 === peg$FAILED) {
                      s0 = peg$parseSqlColumn();
                      if (s0 === peg$FAILED) {
                        s0 = peg$parseSqlRecordMaybe();
                        if (s0 === peg$FAILED) {
                          s0 = peg$parseSqlQueryInParens();
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseCaseExpression() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10;
          s0 = peg$currPos;
          s1 = peg$parseCaseToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseOrExpression();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSqlWhenThenPart();
                if (s4 !== peg$FAILED) {
                  s5 = [];
                  s6 = peg$currPos;
                  s7 = peg$parse_();
                  if (s7 !== peg$FAILED) {
                    s8 = peg$parseSqlWhenThenPart();
                    if (s8 !== peg$FAILED) {
                      s7 = [s7, s8];
                      s6 = s7;
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                  while (s6 !== peg$FAILED) {
                    s5.push(s6);
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseSqlWhenThenPart();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseElseToken();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parse_();
                        if (s9 !== peg$FAILED) {
                          s10 = peg$parseOrExpression();
                          if (s10 !== peg$FAILED) {
                            s7 = [s7, s8, s9, s10];
                            s6 = s7;
                          } else {
                            peg$currPos = s6;
                            s6 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s6;
                          s6 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse_();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parseEndToken();
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c81(s1, s2, s3, s4, s5, s6, s7, s8);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlWhenThenPart() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$parseWhenToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOrExpression();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseOrExpression();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseOrExpression();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseThenToken();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse_();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parseOrExpression();
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c82(s1, s2, s3, s4, s5, s6, s7, s8);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseInterval() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11;
          s0 = peg$currPos;
          s1 = peg$parseIntervalToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseBaseType();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$currPos;
                  s7 = peg$parseTimeUnit();
                  if (s7 !== peg$FAILED) {
                    s8 = peg$parse_();
                    if (s8 !== peg$FAILED) {
                      s9 = peg$parseToToken();
                      if (s9 !== peg$FAILED) {
                        s10 = peg$parse_();
                        if (s10 !== peg$FAILED) {
                          s11 = peg$parseTimeUnit();
                          if (s11 !== peg$FAILED) {
                            s7 = [s7, s8, s9, s10, s11];
                            s6 = s7;
                          } else {
                            peg$currPos = s6;
                            s6 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s6;
                          s6 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                  if (s6 !== peg$FAILED) {
                    s5 = input.substring(s5, peg$currPos);
                  } else {
                    s5 = s6;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = peg$currPos;
                    s6 = peg$currPos;
                    s7 = peg$parseTimeUnit();
                    if (s7 !== peg$FAILED) {
                      if (input.charCodeAt(peg$currPos) === 95) {
                        s8 = peg$c83;
                        peg$currPos++;
                      } else {
                        s8 = peg$FAILED;
                        if (peg$silentFails === 0) {
                          peg$fail(peg$c84);
                        }
                      }
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseTimeUnit();
                        if (s9 !== peg$FAILED) {
                          s7 = [s7, s8, s9];
                          s6 = s7;
                        } else {
                          peg$currPos = s6;
                          s6 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 !== peg$FAILED) {
                      s5 = input.substring(s5, peg$currPos);
                    } else {
                      s5 = s6;
                    }
                    if (s5 === peg$FAILED) {
                      s5 = peg$parseTimeUnit();
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c85(s1, s2, s3, s4, s5);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseTimeUnitLiteral() {
          var s0, s1;
          s0 = peg$currPos;
          s1 = peg$parseTimeUnit();
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c86(s1);
          }
          s0 = s1;
          return s0;
        }
        function peg$parseTimeUnit() {
          var s0;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c87) {
            s0 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c88);
            }
          }
          if (s0 === peg$FAILED) {
            if (input.substr(peg$currPos, 6).toLowerCase() === peg$c89) {
              s0 = input.substr(peg$currPos, 6);
              peg$currPos += 6;
            } else {
              s0 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c90);
              }
            }
            if (s0 === peg$FAILED) {
              if (input.substr(peg$currPos, 4).toLowerCase() === peg$c91) {
                s0 = input.substr(peg$currPos, 4);
                peg$currPos += 4;
              } else {
                s0 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c92);
                }
              }
              if (s0 === peg$FAILED) {
                if (input.substr(peg$currPos, 3).toLowerCase() === peg$c93) {
                  s0 = input.substr(peg$currPos, 3);
                  peg$currPos += 3;
                } else {
                  s0 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c94);
                  }
                }
                if (s0 === peg$FAILED) {
                  if (input.substr(peg$currPos, 4).toLowerCase() === peg$c95) {
                    s0 = input.substr(peg$currPos, 4);
                    peg$currPos += 4;
                  } else {
                    s0 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c96);
                    }
                  }
                  if (s0 === peg$FAILED) {
                    if (input.substr(peg$currPos, 5).toLowerCase() === peg$c97) {
                      s0 = input.substr(peg$currPos, 5);
                      peg$currPos += 5;
                    } else {
                      s0 = peg$FAILED;
                      if (peg$silentFails === 0) {
                        peg$fail(peg$c98);
                      }
                    }
                    if (s0 === peg$FAILED) {
                      if (input.substr(peg$currPos, 7).toLowerCase() === peg$c99) {
                        s0 = input.substr(peg$currPos, 7);
                        peg$currPos += 7;
                      } else {
                        s0 = peg$FAILED;
                        if (peg$silentFails === 0) {
                          peg$fail(peg$c100);
                        }
                      }
                      if (s0 === peg$FAILED) {
                        if (input.substr(peg$currPos, 4).toLowerCase() === peg$c101) {
                          s0 = input.substr(peg$currPos, 4);
                          peg$currPos += 4;
                        } else {
                          s0 = peg$FAILED;
                          if (peg$silentFails === 0) {
                            peg$fail(peg$c102);
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseTimeUnitExtra() {
          var s0;
          s0 = peg$parseTimeUnit();
          if (s0 === peg$FAILED) {
            if (input.substr(peg$currPos, 5).toLowerCase() === peg$c103) {
              s0 = input.substr(peg$currPos, 5);
              peg$currPos += 5;
            } else {
              s0 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c104);
              }
            }
            if (s0 === peg$FAILED) {
              if (input.substr(peg$currPos, 11).toLowerCase() === peg$c105) {
                s0 = input.substr(peg$currPos, 11);
                peg$currPos += 11;
              } else {
                s0 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c106);
                }
              }
              if (s0 === peg$FAILED) {
                if (input.substr(peg$currPos, 11).toLowerCase() === peg$c107) {
                  s0 = input.substr(peg$currPos, 11);
                  peg$currPos += 11;
                } else {
                  s0 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c108);
                  }
                }
                if (s0 === peg$FAILED) {
                  if (input.substr(peg$currPos, 3).toLowerCase() === peg$c109) {
                    s0 = input.substr(peg$currPos, 3);
                    peg$currPos += 3;
                  } else {
                    s0 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c110);
                    }
                  }
                  if (s0 === peg$FAILED) {
                    if (input.substr(peg$currPos, 6).toLowerCase() === peg$c111) {
                      s0 = input.substr(peg$currPos, 6);
                      peg$currPos += 6;
                    } else {
                      s0 = peg$FAILED;
                      if (peg$silentFails === 0) {
                        peg$fail(peg$c112);
                      }
                    }
                    if (s0 === peg$FAILED) {
                      if (input.substr(peg$currPos, 3).toLowerCase() === peg$c113) {
                        s0 = input.substr(peg$currPos, 3);
                        peg$currPos += 3;
                      } else {
                        s0 = peg$FAILED;
                        if (peg$silentFails === 0) {
                          peg$fail(peg$c114);
                        }
                      }
                      if (s0 === peg$FAILED) {
                        if (input.substr(peg$currPos, 7).toLowerCase() === peg$c115) {
                          s0 = input.substr(peg$currPos, 7);
                          peg$currPos += 7;
                        } else {
                          s0 = peg$FAILED;
                          if (peg$silentFails === 0) {
                            peg$fail(peg$c116);
                          }
                        }
                        if (s0 === peg$FAILED) {
                          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c117) {
                            s0 = input.substr(peg$currPos, 6);
                            peg$currPos += 6;
                          } else {
                            s0 = peg$FAILED;
                            if (peg$silentFails === 0) {
                              peg$fail(peg$c118);
                            }
                          }
                          if (s0 === peg$FAILED) {
                            if (input.substr(peg$currPos, 7).toLowerCase() === peg$c119) {
                              s0 = input.substr(peg$currPos, 7);
                              peg$currPos += 7;
                            } else {
                              s0 = peg$FAILED;
                              if (peg$silentFails === 0) {
                                peg$fail(peg$c120);
                              }
                            }
                            if (s0 === peg$FAILED) {
                              if (input.substr(peg$currPos, 10).toLowerCase() === peg$c121) {
                                s0 = input.substr(peg$currPos, 10);
                                peg$currPos += 10;
                              } else {
                                s0 = peg$FAILED;
                                if (peg$silentFails === 0) {
                                  peg$fail(peg$c122);
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseFunction() {
          var s0;
          s0 = peg$parseCastFunction();
          if (s0 === peg$FAILED) {
            s0 = peg$parseGenericFunction();
            if (s0 === peg$FAILED) {
              s0 = peg$parseCountStarFunction();
              if (s0 === peg$FAILED) {
                s0 = peg$parseExtractFunction();
                if (s0 === peg$FAILED) {
                  s0 = peg$parseTrimFunction();
                  if (s0 === peg$FAILED) {
                    s0 = peg$parseFloorCeilFunction();
                    if (s0 === peg$FAILED) {
                      s0 = peg$parseTimestampAddDiffFunction();
                      if (s0 === peg$FAILED) {
                        s0 = peg$parsePositionFunction();
                        if (s0 === peg$FAILED) {
                          s0 = peg$parseArrayFunction();
                          if (s0 === peg$FAILED) {
                            s0 = peg$parseNakedFunction();
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          return s0;
        }
        function peg$parseGenericFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12, s13, s14, s15, s16;
          s0 = peg$currPos;
          s1 = peg$parseRefNameFunction();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 46) {
                s4 = peg$c123;
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c124);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseRefNameFunction();
                  if (s6 !== peg$FAILED) {
                    s3 = [s3, s4, s5, s6];
                    s2 = s3;
                  } else {
                    peg$currPos = s2;
                    s2 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                s4 = peg$parseOpenParen();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parse_();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parseFunctionDecorator();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parse_();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseGeneralFunctionArgument();
                      if (s7 === peg$FAILED) {
                        s7 = null;
                      }
                      if (s7 !== peg$FAILED) {
                        s8 = [];
                        s9 = peg$currPos;
                        s10 = peg$parseCommaSeparator();
                        if (s10 !== peg$FAILED) {
                          s11 = peg$parseGeneralFunctionArgument();
                          if (s11 !== peg$FAILED) {
                            s10 = [s10, s11];
                            s9 = s10;
                          } else {
                            peg$currPos = s9;
                            s9 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s9;
                          s9 = peg$FAILED;
                        }
                        while (s9 !== peg$FAILED) {
                          s8.push(s9);
                          s9 = peg$currPos;
                          s10 = peg$parseCommaSeparator();
                          if (s10 !== peg$FAILED) {
                            s11 = peg$parseGeneralFunctionArgument();
                            if (s11 !== peg$FAILED) {
                              s10 = [s10, s11];
                              s9 = s10;
                            } else {
                              peg$currPos = s9;
                              s9 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s9;
                            s9 = peg$FAILED;
                          }
                        }
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parse_();
                          if (s9 !== peg$FAILED) {
                            s10 = peg$parseCloseParen();
                            if (s10 !== peg$FAILED) {
                              s11 = peg$currPos;
                              s12 = peg$parse_();
                              if (s12 !== peg$FAILED) {
                                s13 = peg$parseFunctionFilter();
                                if (s13 !== peg$FAILED) {
                                  s12 = [s12, s13];
                                  s11 = s12;
                                } else {
                                  peg$currPos = s11;
                                  s11 = peg$FAILED;
                                }
                              } else {
                                peg$currPos = s11;
                                s11 = peg$FAILED;
                              }
                              if (s11 === peg$FAILED) {
                                s11 = null;
                              }
                              if (s11 !== peg$FAILED) {
                                s12 = peg$currPos;
                                s13 = peg$parse_();
                                if (s13 !== peg$FAILED) {
                                  s14 = peg$parseOverToken();
                                  if (s14 !== peg$FAILED) {
                                    s15 = peg$parse_();
                                    if (s15 !== peg$FAILED) {
                                      s16 = peg$parseWindowSpec();
                                      if (s16 !== peg$FAILED) {
                                        s13 = [s13, s14, s15, s16];
                                        s12 = s13;
                                      } else {
                                        peg$currPos = s12;
                                        s12 = peg$FAILED;
                                      }
                                    } else {
                                      peg$currPos = s12;
                                      s12 = peg$FAILED;
                                    }
                                  } else {
                                    peg$currPos = s12;
                                    s12 = peg$FAILED;
                                  }
                                } else {
                                  peg$currPos = s12;
                                  s12 = peg$FAILED;
                                }
                                if (s12 === peg$FAILED) {
                                  s12 = null;
                                }
                                if (s12 !== peg$FAILED) {
                                  s13 = peg$currPos;
                                  s14 = peg$parse_();
                                  if (s14 !== peg$FAILED) {
                                    s15 = peg$parseSqlExtendClause();
                                    if (s15 !== peg$FAILED) {
                                      s14 = [s14, s15];
                                      s13 = s14;
                                    } else {
                                      peg$currPos = s13;
                                      s13 = peg$FAILED;
                                    }
                                  } else {
                                    peg$currPos = s13;
                                    s13 = peg$FAILED;
                                  }
                                  if (s13 === peg$FAILED) {
                                    s13 = null;
                                  }
                                  if (s13 !== peg$FAILED) {
                                    peg$savedPos = s0;
                                    s1 = peg$c125(s1, s2, s3, s5, s6, s7, s8, s9, s11, s12, s13);
                                    s0 = s1;
                                  } else {
                                    peg$currPos = s0;
                                    s0 = peg$FAILED;
                                  }
                                } else {
                                  peg$currPos = s0;
                                  s0 = peg$FAILED;
                                }
                              } else {
                                peg$currPos = s0;
                                s0 = peg$FAILED;
                              }
                            } else {
                              peg$currPos = s0;
                              s0 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseNakedFunction() {
          var s0, s1, s2;
          s0 = peg$currPos;
          s1 = peg$parseUnquotedRefNameFree();
          if (s1 !== peg$FAILED) {
            peg$savedPos = peg$currPos;
            s2 = peg$c126(s1);
            if (s2) {
              s2 = void 0;
            } else {
              s2 = peg$FAILED;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c127(s1);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseCountStarFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10;
          s0 = peg$currPos;
          s1 = peg$parseCountToken();
          if (s1 === peg$FAILED) {
            s1 = peg$currPos;
            if (input.charCodeAt(peg$currPos) === 34) {
              s2 = peg$c128;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c129);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseCountToken();
              if (s3 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 34) {
                  s4 = peg$c128;
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c129);
                  }
                }
                if (s4 !== peg$FAILED) {
                  s2 = [s2, s3, s4];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  if (input.charCodeAt(peg$currPos) === 42) {
                    s5 = peg$c69;
                    peg$currPos++;
                  } else {
                    s5 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c70);
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parse_();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseCloseParen();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$currPos;
                        s9 = peg$parse_();
                        if (s9 !== peg$FAILED) {
                          s10 = peg$parseFunctionFilter();
                          if (s10 !== peg$FAILED) {
                            s9 = [s9, s10];
                            s8 = s9;
                          } else {
                            peg$currPos = s8;
                            s8 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s8;
                          s8 = peg$FAILED;
                        }
                        if (s8 === peg$FAILED) {
                          s8 = null;
                        }
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c130(s1, s2, s4, s6, s8);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseCastFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseCastToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseOrExpression();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseAsSeparator();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseSqlType();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parse_();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parseCloseParen();
                          if (s9 !== peg$FAILED) {
                            peg$savedPos = s0;
                            s1 = peg$c131(s1, s2, s4, s5, s6, s7, s8);
                            s0 = s1;
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseExtractFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseExtractToken();
          if (s1 === peg$FAILED) {
            s1 = peg$currPos;
            if (input.charCodeAt(peg$currPos) === 34) {
              s2 = peg$c128;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c129);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseExtractToken();
              if (s3 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 34) {
                  s4 = peg$c128;
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c129);
                  }
                }
                if (s4 !== peg$FAILED) {
                  s2 = [s2, s3, s4];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseTimeUnitExtra();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseFromSeparator();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseOrExpression();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parse_();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parseCloseParen();
                          if (s9 !== peg$FAILED) {
                            peg$savedPos = s0;
                            s1 = peg$c132(s1, s2, s4, s5, s6, s7, s8);
                            s0 = s1;
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseTrimFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11;
          s0 = peg$currPos;
          s1 = peg$parseTrimToken();
          if (s1 === peg$FAILED) {
            s1 = peg$currPos;
            if (input.charCodeAt(peg$currPos) === 34) {
              s2 = peg$c128;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c129);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseTrimToken();
              if (s3 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 34) {
                  s4 = peg$c128;
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c129);
                  }
                }
                if (s4 !== peg$FAILED) {
                  s2 = [s2, s3, s4];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$currPos;
                  s7 = peg$currPos;
                  s8 = peg$parseLeadingToken();
                  if (s8 === peg$FAILED) {
                    s8 = peg$parseBothToken();
                    if (s8 === peg$FAILED) {
                      s8 = peg$parseTrailingToken();
                    }
                  }
                  if (s8 !== peg$FAILED) {
                    s9 = peg$currPos;
                    s10 = peg$parse_();
                    if (s10 !== peg$FAILED) {
                      s11 = peg$parseFromToken();
                      if (s11 !== peg$FAILED) {
                        s10 = [s10, s11];
                        s9 = s10;
                      } else {
                        peg$currPos = s9;
                        s9 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s9;
                      s9 = peg$FAILED;
                    }
                    if (s9 === peg$FAILED) {
                      s9 = null;
                    }
                    if (s9 !== peg$FAILED) {
                      s8 = [s8, s9];
                      s7 = s8;
                    } else {
                      peg$currPos = s7;
                      s7 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s7;
                    s7 = peg$FAILED;
                  }
                  if (s7 !== peg$FAILED) {
                    s6 = input.substring(s6, peg$currPos);
                  } else {
                    s6 = s7;
                  }
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseOrExpression();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$currPos;
                      s8 = peg$parseFromSeparator();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOrExpression();
                        if (s9 !== peg$FAILED) {
                          s8 = [s8, s9];
                          s7 = s8;
                        } else {
                          peg$currPos = s7;
                          s7 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                      if (s7 === peg$FAILED) {
                        s7 = null;
                      }
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parse_();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parseCloseParen();
                          if (s9 !== peg$FAILED) {
                            peg$savedPos = s0;
                            s1 = peg$c133(s1, s2, s4, s5, s6, s7, s8);
                            s0 = s1;
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseFloorCeilFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseFloorToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseCeilToken();
          }
          if (s1 === peg$FAILED) {
            s1 = peg$currPos;
            if (input.charCodeAt(peg$currPos) === 34) {
              s2 = peg$c128;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c129);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseFloorToken();
              if (s3 === peg$FAILED) {
                s3 = peg$parseCeilToken();
              }
              if (s3 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 34) {
                  s4 = peg$c128;
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c129);
                  }
                }
                if (s4 !== peg$FAILED) {
                  s2 = [s2, s3, s4];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseOrExpression();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseToSeparator();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseTimeUnitLiteral();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parse_();
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parseCloseParen();
                          if (s9 !== peg$FAILED) {
                            peg$savedPos = s0;
                            s1 = peg$c134(s1, s2, s4, s5, s6, s7, s8);
                            s0 = s1;
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseTimestampAddDiffFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseTimestampaddToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseTimestampdiffToken();
          }
          if (s1 === peg$FAILED) {
            s1 = peg$currPos;
            if (input.charCodeAt(peg$currPos) === 34) {
              s2 = peg$c128;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c129);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parseTimestampaddToken();
              if (s3 === peg$FAILED) {
                s3 = peg$parseTimestampdiffToken();
              }
              if (s3 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 34) {
                  s4 = peg$c128;
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c129);
                  }
                }
                if (s4 !== peg$FAILED) {
                  s2 = [s2, s3, s4];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseTimeUnitLiteral();
                  if (s5 !== peg$FAILED) {
                    s6 = [];
                    s7 = peg$currPos;
                    s8 = peg$parseCommaSeparator();
                    if (s8 !== peg$FAILED) {
                      s9 = peg$parseOrExpression();
                      if (s9 !== peg$FAILED) {
                        s8 = [s8, s9];
                        s7 = s8;
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s7;
                      s7 = peg$FAILED;
                    }
                    while (s7 !== peg$FAILED) {
                      s6.push(s7);
                      s7 = peg$currPos;
                      s8 = peg$parseCommaSeparator();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOrExpression();
                        if (s9 !== peg$FAILED) {
                          s8 = [s8, s9];
                          s7 = s8;
                        } else {
                          peg$currPos = s7;
                          s7 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse_();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$parseCloseParen();
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c135(s1, s2, s4, s5, s6, s7);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parsePositionFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10;
          s0 = peg$currPos;
          s1 = peg$parsePositionToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseOrExpression();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parseInSeparator();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseOrExpression();
                      if (s7 !== peg$FAILED) {
                        s8 = peg$currPos;
                        s9 = peg$parseFromSeparator();
                        if (s9 !== peg$FAILED) {
                          s10 = peg$parseOrExpression();
                          if (s10 !== peg$FAILED) {
                            s9 = [s9, s10];
                            s8 = s9;
                          } else {
                            peg$currPos = s8;
                            s8 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s8;
                          s8 = peg$FAILED;
                        }
                        if (s8 === peg$FAILED) {
                          s8 = null;
                        }
                        if (s8 !== peg$FAILED) {
                          s9 = peg$parse_();
                          if (s9 !== peg$FAILED) {
                            s10 = peg$parseCloseParen();
                            if (s10 !== peg$FAILED) {
                              peg$savedPos = s0;
                              s1 = peg$c136(s1, s2, s4, s5, s6, s7, s8, s9);
                              s0 = s1;
                            } else {
                              peg$currPos = s0;
                              s0 = peg$FAILED;
                            }
                          } else {
                            peg$currPos = s0;
                            s0 = peg$FAILED;
                          }
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseArrayFunction() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseArrayToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 91) {
                s3 = peg$c137;
                peg$currPos++;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c138);
                }
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseOrExpression();
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = [];
                    s7 = peg$currPos;
                    s8 = peg$parseCommaSeparator();
                    if (s8 !== peg$FAILED) {
                      s9 = peg$parseOrExpression();
                      if (s9 !== peg$FAILED) {
                        s8 = [s8, s9];
                        s7 = s8;
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s7;
                      s7 = peg$FAILED;
                    }
                    while (s7 !== peg$FAILED) {
                      s6.push(s7);
                      s7 = peg$currPos;
                      s8 = peg$parseCommaSeparator();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOrExpression();
                        if (s9 !== peg$FAILED) {
                          s8 = [s8, s9];
                          s7 = s8;
                        } else {
                          peg$currPos = s7;
                          s7 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parse_();
                      if (s7 !== peg$FAILED) {
                        if (input.charCodeAt(peg$currPos) === 93) {
                          s8 = peg$c139;
                          peg$currPos++;
                        } else {
                          s8 = peg$FAILED;
                          if (peg$silentFails === 0) {
                            peg$fail(peg$c140);
                          }
                        }
                        if (s8 !== peg$FAILED) {
                          peg$savedPos = s0;
                          s1 = peg$c141(s1, s2, s4, s5, s6, s7);
                          s0 = s1;
                        } else {
                          peg$currPos = s0;
                          s0 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseFunctionDecorator() {
          var s0;
          s0 = peg$parseDistinctToken();
          if (s0 === peg$FAILED) {
            s0 = peg$parseAllToken();
          }
          return s0;
        }
        function peg$parseFunctionFilter() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseFilterToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseOpenParen();
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseSqlWhereClause();
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parse_();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseCloseParen();
                      if (s7 !== peg$FAILED) {
                        peg$savedPos = s0;
                        s1 = peg$c142(s1, s2, s4, s5, s6);
                        s0 = s1;
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseWindowSpec() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$parseOpenParen();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parseRefName();
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$currPos;
                s5 = peg$parseOrderByClause();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s5 = [s5, s6];
                    s4 = s5;
                  } else {
                    peg$currPos = s4;
                    s4 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
                if (s4 === peg$FAILED) {
                  s4 = null;
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseCloseParen();
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c143(s2, s3, s4);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseCommaSeparator() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 !== peg$FAILED) {
            if (input.charCodeAt(peg$currPos) === 44) {
              s2 = peg$c144;
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c145);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c146(s1, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseAsSeparator() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 !== peg$FAILED) {
            s2 = peg$parseAsToken();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c147(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseFromSeparator() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 !== peg$FAILED) {
            s2 = peg$parseFromToken();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c147(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseToSeparator() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 !== peg$FAILED) {
            s2 = peg$parseToToken();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c147(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseInSeparator() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parse_();
          if (s1 !== peg$FAILED) {
            s2 = peg$parseInToken();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c147(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlRecordMaybe() {
          var s0, s1;
          s0 = peg$currPos;
          s1 = peg$parseSqlRecord();
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c148(s1);
          }
          s0 = s1;
          return s0;
        }
        function peg$parseSqlRecord() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseRowToken();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseOpenParen();
            if (s2 !== peg$FAILED) {
              s3 = peg$parse_();
              if (s3 !== peg$FAILED) {
                s4 = peg$parseOrExpression();
                if (s4 !== peg$FAILED) {
                  s5 = [];
                  s6 = peg$currPos;
                  s7 = peg$parseCommaSeparator();
                  if (s7 !== peg$FAILED) {
                    s8 = peg$parseOrExpression();
                    if (s8 !== peg$FAILED) {
                      s7 = [s7, s8];
                      s6 = s7;
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s6;
                    s6 = peg$FAILED;
                  }
                  while (s6 !== peg$FAILED) {
                    s5.push(s6);
                    s6 = peg$currPos;
                    s7 = peg$parseCommaSeparator();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseOrExpression();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$parse_();
                    if (s6 !== peg$FAILED) {
                      s7 = peg$parseCloseParen();
                      if (s7 !== peg$FAILED) {
                        peg$savedPos = s0;
                        s1 = peg$c149(s1, s3, s4, s5, s6);
                        s0 = s1;
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlQueryInParens() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseOpenParen();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlQueryInParens();
              if (s3 === peg$FAILED) {
                s3 = peg$parseSqlQuery();
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parse_();
                if (s4 !== peg$FAILED) {
                  s5 = peg$parseCloseParen();
                  if (s5 !== peg$FAILED) {
                    peg$savedPos = s0;
                    s1 = peg$c150(s2, s3, s4);
                    s0 = s1;
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlValues() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8, s9;
          s0 = peg$currPos;
          s1 = peg$parseValuesToken();
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSqlRecord();
              if (s3 !== peg$FAILED) {
                s4 = [];
                s5 = peg$currPos;
                s6 = peg$parseCommaSeparator();
                if (s6 !== peg$FAILED) {
                  s7 = peg$parseSqlRecord();
                  if (s7 !== peg$FAILED) {
                    s6 = [s6, s7];
                    s5 = s6;
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                while (s5 !== peg$FAILED) {
                  s4.push(s5);
                  s5 = peg$currPos;
                  s6 = peg$parseCommaSeparator();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseSqlRecord();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                }
                if (s4 !== peg$FAILED) {
                  s5 = peg$currPos;
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseOrderByClause();
                    if (s7 !== peg$FAILED) {
                      s6 = [s6, s7];
                      s5 = s6;
                    } else {
                      peg$currPos = s5;
                      s5 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s5;
                    s5 = peg$FAILED;
                  }
                  if (s5 === peg$FAILED) {
                    s5 = null;
                  }
                  if (s5 !== peg$FAILED) {
                    s6 = peg$currPos;
                    s7 = peg$parse_();
                    if (s7 !== peg$FAILED) {
                      s8 = peg$parseLimitClause();
                      if (s8 !== peg$FAILED) {
                        s7 = [s7, s8];
                        s6 = s7;
                      } else {
                        peg$currPos = s6;
                        s6 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s6;
                      s6 = peg$FAILED;
                    }
                    if (s6 === peg$FAILED) {
                      s6 = null;
                    }
                    if (s6 !== peg$FAILED) {
                      s7 = peg$currPos;
                      s8 = peg$parse_();
                      if (s8 !== peg$FAILED) {
                        s9 = peg$parseOffsetClause();
                        if (s9 !== peg$FAILED) {
                          s8 = [s8, s9];
                          s7 = s8;
                        } else {
                          peg$currPos = s7;
                          s7 = peg$FAILED;
                        }
                      } else {
                        peg$currPos = s7;
                        s7 = peg$FAILED;
                      }
                      if (s7 === peg$FAILED) {
                        s7 = null;
                      }
                      if (s7 !== peg$FAILED) {
                        peg$savedPos = s0;
                        s1 = peg$c151(s1, s2, s3, s4, s5, s6, s7);
                        s0 = s1;
                      } else {
                        peg$currPos = s0;
                        s0 = peg$FAILED;
                      }
                    } else {
                      peg$currPos = s0;
                      s0 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s0;
                    s0 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlPlaceholder() {
          var s0, s1;
          s0 = peg$currPos;
          if (input.charCodeAt(peg$currPos) === 63) {
            s1 = peg$c152;
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c153);
            }
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c154();
          }
          s0 = s1;
          return s0;
        }
        function peg$parseSqlLiteral() {
          var s0, s1;
          s0 = peg$currPos;
          s1 = peg$parseNullToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseTrueToken();
            if (s1 === peg$FAILED) {
              s1 = peg$parseFalseToken();
              if (s1 === peg$FAILED) {
                s1 = peg$parseNumber();
                if (s1 === peg$FAILED) {
                  s1 = peg$parseSingleQuotedString();
                  if (s1 === peg$FAILED) {
                    s1 = peg$parseUnicodeString();
                    if (s1 === peg$FAILED) {
                      s1 = peg$parseCharsetString();
                      if (s1 === peg$FAILED) {
                        s1 = peg$parseBinaryString();
                        if (s1 === peg$FAILED) {
                          s1 = peg$parseTimestamp();
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c155(s1);
          }
          s0 = s1;
          return s0;
        }
        function peg$parseNullLiteral() {
          var s0, s1;
          s0 = peg$currPos;
          s1 = peg$parseNullToken();
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c156(s1);
          }
          s0 = s1;
          return s0;
        }
        function peg$parseBooleanLiteral() {
          var s0, s1;
          s0 = peg$currPos;
          s1 = peg$parseTrueToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseFalseToken();
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c156(s1);
          }
          s0 = s1;
          return s0;
        }
        function peg$parseSqlType() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseUnquotedRefNameFree();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parseOpenParen();
            if (s3 !== peg$FAILED) {
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseSingleQuotedString();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseCloseParen();
                    if (s7 !== peg$FAILED) {
                      s3 = [s3, s4, s5, s6, s7];
                      s2 = s3;
                    } else {
                      peg$currPos = s2;
                      s2 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s2;
                    s2 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c157();
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseNumber() {
          var s0, s1, s2, s3, s4, s5, s6;
          peg$silentFails++;
          s0 = peg$currPos;
          if (peg$c75.test(input.charAt(peg$currPos))) {
            s1 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c76);
            }
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parseDigits();
            if (s3 !== peg$FAILED) {
              s4 = peg$parseFraction();
              if (s4 === peg$FAILED) {
                s4 = null;
              }
              if (s4 !== peg$FAILED) {
                s3 = [s3, s4];
                s2 = s3;
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = peg$parseFraction();
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              if (input.substr(peg$currPos, 1).toLowerCase() === peg$c159) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c160);
                }
              }
              if (s4 !== peg$FAILED) {
                if (peg$c75.test(input.charAt(peg$currPos))) {
                  s5 = input.charAt(peg$currPos);
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c76);
                  }
                }
                if (s5 === peg$FAILED) {
                  s5 = null;
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseDigits();
                  if (s6 !== peg$FAILED) {
                    s4 = [s4, s5, s6];
                    s3 = s4;
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c161();
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c158);
            }
          }
          return s0;
        }
        function peg$parseFraction() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.charCodeAt(peg$currPos) === 46) {
            s2 = peg$c123;
            peg$currPos++;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c124);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parseDigits();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseDigits() {
          var s0, s1, s2;
          s0 = peg$currPos;
          s1 = [];
          s2 = peg$parseDigit();
          if (s2 !== peg$FAILED) {
            while (s2 !== peg$FAILED) {
              s1.push(s2);
              s2 = peg$parseDigit();
            }
          } else {
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseDigit() {
          var s0;
          if (peg$c162.test(input.charAt(peg$currPos))) {
            s0 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c163);
            }
          }
          return s0;
        }
        function peg$parseSingleQuotedString() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          if (input.charCodeAt(peg$currPos) === 39) {
            s1 = peg$c164;
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c165);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = [];
            if (input.substr(peg$currPos, 2) === peg$c166) {
              s4 = peg$c166;
              peg$currPos += 2;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c167);
              }
            }
            if (s4 === peg$FAILED) {
              if (peg$c168.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c169);
                }
              }
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              if (input.substr(peg$currPos, 2) === peg$c166) {
                s4 = peg$c166;
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c167);
                }
              }
              if (s4 === peg$FAILED) {
                if (peg$c168.test(input.charAt(peg$currPos))) {
                  s4 = input.charAt(peg$currPos);
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c169);
                  }
                }
              }
            }
            if (s3 !== peg$FAILED) {
              s2 = input.substring(s2, peg$currPos);
            } else {
              s2 = s3;
            }
            if (s2 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 39) {
                s3 = peg$c164;
                peg$currPos++;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c165);
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c170(s2);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnicodeString() {
          var s0, s1, s2;
          s0 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c171) {
            s1 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c172);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseSingleQuotedString();
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c173(s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseCharsetString() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          if (input.charCodeAt(peg$currPos) === 95) {
            s1 = peg$c83;
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c84);
            }
          }
          if (s1 !== peg$FAILED) {
            if (peg$c174.test(input.charAt(peg$currPos))) {
              s2 = input.charAt(peg$currPos);
              peg$currPos++;
            } else {
              s2 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c175);
              }
            }
            if (s2 !== peg$FAILED) {
              s3 = [];
              if (peg$c176.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c177);
                }
              }
              while (s4 !== peg$FAILED) {
                s3.push(s4);
                if (peg$c176.test(input.charAt(peg$currPos))) {
                  s4 = input.charAt(peg$currPos);
                  peg$currPos++;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c177);
                  }
                }
              }
              if (s3 !== peg$FAILED) {
                s4 = peg$parseSingleQuotedString();
                if (s4 !== peg$FAILED) {
                  peg$savedPos = s0;
                  s1 = peg$c178(s4);
                  s0 = s1;
                } else {
                  peg$currPos = s0;
                  s0 = peg$FAILED;
                }
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseBinaryString() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c179) {
            s1 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c180);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = [];
            if (peg$c181.test(input.charAt(peg$currPos))) {
              s4 = input.charAt(peg$currPos);
              peg$currPos++;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c182);
              }
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              if (peg$c181.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c182);
                }
              }
            }
            if (s3 !== peg$FAILED) {
              s2 = input.substring(s2, peg$currPos);
            } else {
              s2 = s3;
            }
            if (s2 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 39) {
                s3 = peg$c164;
                peg$currPos++;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c165);
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c183(s2);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseTimestamp() {
          var s0, s1, s2, s3;
          s0 = peg$currPos;
          s1 = peg$parseTimestampToken();
          if (s1 === peg$FAILED) {
            s1 = peg$parseDateToken();
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parse_();
            if (s2 !== peg$FAILED) {
              s3 = peg$parseSingleQuotedString();
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c184(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseArrayEntries() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$parseArrayEntry();
          if (s1 !== peg$FAILED) {
            s2 = [];
            s3 = peg$currPos;
            s4 = peg$parseCommaSeparator();
            if (s4 !== peg$FAILED) {
              s5 = peg$parseArrayEntry();
              if (s5 !== peg$FAILED) {
                s4 = [s4, s5];
                s3 = s4;
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            while (s3 !== peg$FAILED) {
              s2.push(s3);
              s3 = peg$currPos;
              s4 = peg$parseCommaSeparator();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseArrayEntry();
                if (s5 !== peg$FAILED) {
                  s4 = [s4, s5];
                  s3 = s4;
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c185(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseArrayEntry() {
          var s0;
          s0 = peg$parseNumber();
          if (s0 === peg$FAILED) {
            s0 = peg$parseSingleQuotedString();
            if (s0 === peg$FAILED) {
              s0 = peg$parseUnicodeString();
              if (s0 === peg$FAILED) {
                s0 = peg$parseBinaryString();
              }
            }
          }
          return s0;
        }
        function peg$parseSqlColumn() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          s0 = peg$currPos;
          s1 = peg$parseRefName();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 46) {
                s4 = peg$c123;
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c124);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseRefName();
                  if (s6 !== peg$FAILED) {
                    s3 = [s3, s4, s5, s6];
                    s2 = s3;
                  } else {
                    peg$currPos = s2;
                    s2 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              s3 = peg$currPos;
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 46) {
                  s5 = peg$c123;
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c124);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s7 = peg$parseRefName();
                    if (s7 !== peg$FAILED) {
                      s4 = [s4, s5, s6, s7];
                      s3 = s4;
                    } else {
                      peg$currPos = s3;
                      s3 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s3;
                    s3 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s3;
                  s3 = peg$FAILED;
                }
              } else {
                peg$currPos = s3;
                s3 = peg$FAILED;
              }
              if (s3 === peg$FAILED) {
                s3 = null;
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c186(s1, s2, s3);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseSqlTable() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$parseRefName();
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 46) {
                s4 = peg$c123;
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c124);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s6 = peg$parseRefName();
                  if (s6 !== peg$FAILED) {
                    s3 = [s3, s4, s5, s6];
                    s2 = s3;
                  } else {
                    peg$currPos = s2;
                    s2 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c187(s1, s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseRefName() {
          var s0;
          s0 = peg$parseQuotedRefName();
          if (s0 === peg$FAILED) {
            s0 = peg$parseUnicodeRefName();
            if (s0 === peg$FAILED) {
              s0 = peg$parseUnquotedRefName();
            }
          }
          return s0;
        }
        function peg$parseRefNameAlias() {
          var s0;
          s0 = peg$parseQuotedRefName();
          if (s0 === peg$FAILED) {
            s0 = peg$parseUnquotedRefNameAlias();
          }
          return s0;
        }
        function peg$parseRefNameFunction() {
          var s0;
          s0 = peg$parseQuotedRefName();
          if (s0 === peg$FAILED) {
            s0 = peg$parseUnquotedRefNameFunction();
          }
          return s0;
        }
        function peg$parseQuotedRefName() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          if (input.charCodeAt(peg$currPos) === 34) {
            s1 = peg$c128;
            peg$currPos++;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c129);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = [];
            if (input.substr(peg$currPos, 2) === peg$c188) {
              s4 = peg$c188;
              peg$currPos += 2;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c189);
              }
            }
            if (s4 === peg$FAILED) {
              if (peg$c190.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c191);
                }
              }
            }
            if (s4 !== peg$FAILED) {
              while (s4 !== peg$FAILED) {
                s3.push(s4);
                if (input.substr(peg$currPos, 2) === peg$c188) {
                  s4 = peg$c188;
                  peg$currPos += 2;
                } else {
                  s4 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c189);
                  }
                }
                if (s4 === peg$FAILED) {
                  if (peg$c190.test(input.charAt(peg$currPos))) {
                    s4 = input.charAt(peg$currPos);
                    peg$currPos++;
                  } else {
                    s4 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c191);
                    }
                  }
                }
              }
            } else {
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = input.substring(s2, peg$currPos);
            } else {
              s2 = s3;
            }
            if (s2 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 34) {
                s3 = peg$c128;
                peg$currPos++;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c129);
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c192(s2);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnicodeRefName() {
          var s0, s1, s2;
          s0 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c171) {
            s1 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c172);
            }
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$parseQuotedRefName();
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c193(s2);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnquotedRefName() {
          var s0, s1, s2;
          s0 = peg$currPos;
          s1 = peg$parseUnquotedRefNameFree();
          if (s1 !== peg$FAILED) {
            peg$savedPos = peg$currPos;
            s2 = peg$c194(s1);
            if (s2) {
              s2 = void 0;
            } else {
              s2 = peg$FAILED;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c195(s1);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnquotedRefNameAlias() {
          var s0, s1, s2;
          s0 = peg$currPos;
          s1 = peg$parseUnquotedRefNameFree();
          if (s1 !== peg$FAILED) {
            peg$savedPos = peg$currPos;
            s2 = peg$c196(s1);
            if (s2) {
              s2 = void 0;
            } else {
              s2 = peg$FAILED;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c195(s1);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnquotedRefNameFunction() {
          var s0, s1, s2;
          s0 = peg$currPos;
          s1 = peg$parseUnquotedRefNameFree();
          if (s1 !== peg$FAILED) {
            peg$savedPos = peg$currPos;
            s2 = peg$c197(s1);
            if (s2) {
              s2 = void 0;
            } else {
              s2 = peg$FAILED;
            }
            if (s2 !== peg$FAILED) {
              peg$savedPos = s0;
              s1 = peg$c195(s1);
              s0 = s1;
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseUnquotedRefNameFree() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (peg$c198.test(input.charAt(peg$currPos))) {
            s2 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c199);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = [];
            if (peg$c200.test(input.charAt(peg$currPos))) {
              s4 = input.charAt(peg$currPos);
              peg$currPos++;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c201);
              }
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              if (peg$c200.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c201);
                }
              }
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSqlStar() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseRefName();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 46) {
                s4 = peg$c123;
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c124);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse_();
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 === peg$FAILED) {
            s1 = null;
          }
          if (s1 !== peg$FAILED) {
            s2 = peg$currPos;
            s3 = peg$parseRefName();
            if (s3 !== peg$FAILED) {
              s4 = peg$parse_();
              if (s4 !== peg$FAILED) {
                if (input.charCodeAt(peg$currPos) === 46) {
                  s5 = peg$c123;
                  peg$currPos++;
                } else {
                  s5 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c124);
                  }
                }
                if (s5 !== peg$FAILED) {
                  s6 = peg$parse_();
                  if (s6 !== peg$FAILED) {
                    s3 = [s3, s4, s5, s6];
                    s2 = s3;
                  } else {
                    peg$currPos = s2;
                    s2 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s2;
                  s2 = peg$FAILED;
                }
              } else {
                peg$currPos = s2;
                s2 = peg$FAILED;
              }
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
            if (s2 === peg$FAILED) {
              s2 = null;
            }
            if (s2 !== peg$FAILED) {
              if (input.charCodeAt(peg$currPos) === 42) {
                s3 = peg$c69;
                peg$currPos++;
              } else {
                s3 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c70);
                }
              }
              if (s3 !== peg$FAILED) {
                peg$savedPos = s0;
                s1 = peg$c202(s1, s2);
                s0 = s1;
              } else {
                peg$currPos = s0;
                s0 = peg$FAILED;
              }
            } else {
              peg$currPos = s0;
              s0 = peg$FAILED;
            }
          } else {
            peg$currPos = s0;
            s0 = peg$FAILED;
          }
          return s0;
        }
        function peg$parseIdentifierPart() {
          var s0;
          if (peg$c198.test(input.charAt(peg$currPos))) {
            s0 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c199);
            }
          }
          return s0;
        }
        function peg$parse_() {
          var s0, s1, s2, s3, s4, s5, s6, s7;
          peg$silentFails++;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = [];
          s3 = peg$parseSpace();
          while (s3 !== peg$FAILED) {
            s2.push(s3);
            s3 = peg$parseSpace();
          }
          if (s2 !== peg$FAILED) {
            s3 = [];
            s4 = peg$currPos;
            s5 = peg$parseSingleLineComment();
            if (s5 === peg$FAILED) {
              s5 = peg$parseMultiLineComment();
            }
            if (s5 !== peg$FAILED) {
              s6 = [];
              s7 = peg$parseSpace();
              while (s7 !== peg$FAILED) {
                s6.push(s7);
                s7 = peg$parseSpace();
              }
              if (s6 !== peg$FAILED) {
                s5 = [s5, s6];
                s4 = s5;
              } else {
                peg$currPos = s4;
                s4 = peg$FAILED;
              }
            } else {
              peg$currPos = s4;
              s4 = peg$FAILED;
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              s4 = peg$currPos;
              s5 = peg$parseSingleLineComment();
              if (s5 === peg$FAILED) {
                s5 = peg$parseMultiLineComment();
              }
              if (s5 !== peg$FAILED) {
                s6 = [];
                s7 = peg$parseSpace();
                while (s7 !== peg$FAILED) {
                  s6.push(s7);
                  s7 = peg$parseSpace();
                }
                if (s6 !== peg$FAILED) {
                  s5 = [s5, s6];
                  s4 = s5;
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
              } else {
                peg$currPos = s4;
                s4 = peg$FAILED;
              }
            }
            if (s3 !== peg$FAILED) {
              s4 = peg$parseFinalSingleLineComment();
              if (s4 === peg$FAILED) {
                s4 = null;
              }
              if (s4 !== peg$FAILED) {
                s2 = [s2, s3, s4];
                s1 = s2;
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c203);
            }
          }
          return s0;
        }
        function peg$parse__() {
          var s0, s1, s2, s3;
          peg$silentFails++;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$parseSpace();
          if (s2 !== peg$FAILED) {
            s3 = peg$parse_();
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c204);
            }
          }
          return s0;
        }
        function peg$parse___() {
          var s0, s1, s2;
          peg$silentFails++;
          s0 = peg$currPos;
          s1 = [];
          s2 = peg$parseSpace();
          while (s2 !== peg$FAILED) {
            s1.push(s2);
            s2 = peg$parseSpace();
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c205);
            }
          }
          return s0;
        }
        function peg$parseSingleLineComment() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2) === peg$c206) {
            s2 = peg$c206;
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c207);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = [];
            if (peg$c208.test(input.charAt(peg$currPos))) {
              s4 = input.charAt(peg$currPos);
              peg$currPos++;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c209);
              }
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              if (peg$c208.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c209);
                }
              }
            }
            if (s3 !== peg$FAILED) {
              if (peg$c210.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c211);
                }
              }
              if (s4 !== peg$FAILED) {
                s2 = [s2, s3, s4];
                s1 = s2;
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseFinalSingleLineComment() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2) === peg$c206) {
            s2 = peg$c206;
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c207);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = [];
            if (peg$c208.test(input.charAt(peg$currPos))) {
              s4 = input.charAt(peg$currPos);
              peg$currPos++;
            } else {
              s4 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c209);
              }
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              if (peg$c208.test(input.charAt(peg$currPos))) {
                s4 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c209);
                }
              }
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseMultiLineComment() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2) === peg$c212) {
            s2 = peg$c212;
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c213);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = [];
            s4 = peg$currPos;
            s5 = peg$currPos;
            peg$silentFails++;
            if (input.substr(peg$currPos, 2) === peg$c214) {
              s6 = peg$c214;
              peg$currPos += 2;
            } else {
              s6 = peg$FAILED;
              if (peg$silentFails === 0) {
                peg$fail(peg$c215);
              }
            }
            peg$silentFails--;
            if (s6 === peg$FAILED) {
              s5 = void 0;
            } else {
              peg$currPos = s5;
              s5 = peg$FAILED;
            }
            if (s5 !== peg$FAILED) {
              if (input.length > peg$currPos) {
                s6 = input.charAt(peg$currPos);
                peg$currPos++;
              } else {
                s6 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c216);
                }
              }
              if (s6 !== peg$FAILED) {
                s5 = [s5, s6];
                s4 = s5;
              } else {
                peg$currPos = s4;
                s4 = peg$FAILED;
              }
            } else {
              peg$currPos = s4;
              s4 = peg$FAILED;
            }
            while (s4 !== peg$FAILED) {
              s3.push(s4);
              s4 = peg$currPos;
              s5 = peg$currPos;
              peg$silentFails++;
              if (input.substr(peg$currPos, 2) === peg$c214) {
                s6 = peg$c214;
                peg$currPos += 2;
              } else {
                s6 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c215);
                }
              }
              peg$silentFails--;
              if (s6 === peg$FAILED) {
                s5 = void 0;
              } else {
                peg$currPos = s5;
                s5 = peg$FAILED;
              }
              if (s5 !== peg$FAILED) {
                if (input.length > peg$currPos) {
                  s6 = input.charAt(peg$currPos);
                  peg$currPos++;
                } else {
                  s6 = peg$FAILED;
                  if (peg$silentFails === 0) {
                    peg$fail(peg$c216);
                  }
                }
                if (s6 !== peg$FAILED) {
                  s5 = [s5, s6];
                  s4 = s5;
                } else {
                  peg$currPos = s4;
                  s4 = peg$FAILED;
                }
              } else {
                peg$currPos = s4;
                s4 = peg$FAILED;
              }
            }
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2) === peg$c214) {
                s4 = peg$c214;
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c215);
                }
              }
              if (s4 !== peg$FAILED) {
                s2 = [s2, s3, s4];
                s1 = s2;
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSpace() {
          var s0;
          if (peg$c217.test(input.charAt(peg$currPos))) {
            s0 = input.charAt(peg$currPos);
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c218);
            }
          }
          return s0;
        }
        function peg$parseOpenParen() {
          var s0, s1;
          peg$silentFails++;
          if (input.charCodeAt(peg$currPos) === 40) {
            s0 = peg$c220;
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c221);
            }
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c219);
            }
          }
          return s0;
        }
        function peg$parseCloseParen() {
          var s0, s1;
          peg$silentFails++;
          if (input.charCodeAt(peg$currPos) === 41) {
            s0 = peg$c223;
            peg$currPos++;
          } else {
            s0 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c224);
            }
          }
          peg$silentFails--;
          if (s0 === peg$FAILED) {
            s1 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c222);
            }
          }
          return s0;
        }
        function peg$parseAllToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c225) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c226);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseAndToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c227) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c228);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseAnyToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c229) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c230);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseArrayToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c231) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c232);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseAsToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c233) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c234);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseAscToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c235) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c236);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseBetweenToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c237) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c238);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseBothToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c239) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c240);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseCaseToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c241) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c242);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseCastToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c243) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c244);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseCeilToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c245) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c246);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseClusteredByToken() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 9).toLowerCase() === peg$c247) {
            s2 = input.substr(peg$currPos, 9);
            peg$currPos += 9;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c248);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2).toLowerCase() === peg$c249) {
                s4 = input.substr(peg$currPos, 2);
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c250);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                peg$silentFails++;
                s6 = peg$parseIdentifierPart();
                peg$silentFails--;
                if (s6 === peg$FAILED) {
                  s5 = void 0;
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseCountToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c251) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c252);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseCubeToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c253) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c254);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseDateToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c255) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c256);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseDescToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c257) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c258);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseDistinctToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 8).toLowerCase() === peg$c259) {
            s2 = input.substr(peg$currPos, 8);
            peg$currPos += 8;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c260);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseElseToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c261) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c262);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseEndToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c263) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c264);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseEscapeToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c265) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c266);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseExplainPlanForToken() {
          var s0, s1, s2, s3, s4, s5, s6, s7, s8;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c267) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c268);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 4).toLowerCase() === peg$c269) {
                s4 = input.substr(peg$currPos, 4);
                peg$currPos += 4;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c270);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$parse__();
                if (s5 !== peg$FAILED) {
                  if (input.substr(peg$currPos, 3).toLowerCase() === peg$c271) {
                    s6 = input.substr(peg$currPos, 3);
                    peg$currPos += 3;
                  } else {
                    s6 = peg$FAILED;
                    if (peg$silentFails === 0) {
                      peg$fail(peg$c272);
                    }
                  }
                  if (s6 !== peg$FAILED) {
                    s7 = peg$currPos;
                    peg$silentFails++;
                    s8 = peg$parseIdentifierPart();
                    peg$silentFails--;
                    if (s8 === peg$FAILED) {
                      s7 = void 0;
                    } else {
                      peg$currPos = s7;
                      s7 = peg$FAILED;
                    }
                    if (s7 !== peg$FAILED) {
                      s2 = [s2, s3, s4, s5, s6, s7];
                      s1 = s2;
                    } else {
                      peg$currPos = s1;
                      s1 = peg$FAILED;
                    }
                  } else {
                    peg$currPos = s1;
                    s1 = peg$FAILED;
                  }
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseExtendToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c273) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c274);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseExtractToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c275) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c276);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseFalseToken() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c277) {
            s3 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s3 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c278);
            }
          }
          if (s3 !== peg$FAILED) {
            s4 = peg$currPos;
            peg$silentFails++;
            s5 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s5 === peg$FAILED) {
              s4 = void 0;
            } else {
              peg$currPos = s4;
              s4 = peg$FAILED;
            }
            if (s4 !== peg$FAILED) {
              s3 = [s3, s4];
              s2 = s3;
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
          } else {
            peg$currPos = s2;
            s2 = peg$FAILED;
          }
          if (s2 !== peg$FAILED) {
            s1 = input.substring(s1, peg$currPos);
          } else {
            s1 = s2;
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c279();
          }
          s0 = s1;
          return s0;
        }
        function peg$parseFilterToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c280) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c281);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseFloorToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c282) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c283);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseFromToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c284) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c285);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseGroupByToken() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c286) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c287);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2).toLowerCase() === peg$c249) {
                s4 = input.substr(peg$currPos, 2);
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c250);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                peg$silentFails++;
                s6 = peg$parseIdentifierPart();
                peg$silentFails--;
                if (s6 === peg$FAILED) {
                  s5 = void 0;
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseGroupingSetsToken() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 8).toLowerCase() === peg$c288) {
            s2 = input.substr(peg$currPos, 8);
            peg$currPos += 8;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c289);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 4).toLowerCase() === peg$c290) {
                s4 = input.substr(peg$currPos, 4);
                peg$currPos += 4;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c291);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                peg$silentFails++;
                s6 = peg$parseIdentifierPart();
                peg$silentFails--;
                if (s6 === peg$FAILED) {
                  s5 = void 0;
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseHavingToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c292) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c293);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseInToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c294) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c295);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseInsertToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c296) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c297);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseIntervalToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 8).toLowerCase() === peg$c298) {
            s2 = input.substr(peg$currPos, 8);
            peg$currPos += 8;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c299);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseIntoToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c300) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c301);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseIsToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c302) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c303);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseJoinToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c304) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c305);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseLeadingToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c306) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c307);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseLikeToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c308) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c309);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseLimitToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c310) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c311);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseNotToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c312) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c313);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseNullToken() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c314) {
            s3 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s3 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c315);
            }
          }
          if (s3 !== peg$FAILED) {
            s4 = peg$currPos;
            peg$silentFails++;
            s5 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s5 === peg$FAILED) {
              s4 = void 0;
            } else {
              peg$currPos = s4;
              s4 = peg$FAILED;
            }
            if (s4 !== peg$FAILED) {
              s3 = [s3, s4];
              s2 = s3;
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
          } else {
            peg$currPos = s2;
            s2 = peg$FAILED;
          }
          if (s2 !== peg$FAILED) {
            s1 = input.substring(s1, peg$currPos);
          } else {
            s1 = s2;
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c316();
          }
          s0 = s1;
          return s0;
        }
        function peg$parseOffsetToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c317) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c318);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOnToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c319) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c320);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOrToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c321) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c322);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOrderByToken() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c323) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c324);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2).toLowerCase() === peg$c249) {
                s4 = input.substr(peg$currPos, 2);
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c250);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                peg$silentFails++;
                s6 = peg$parseIdentifierPart();
                peg$silentFails--;
                if (s6 === peg$FAILED) {
                  s5 = void 0;
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOuterToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c325) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c326);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOverToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c327) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c328);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseOverwriteToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 9).toLowerCase() === peg$c329) {
            s2 = input.substr(peg$currPos, 9);
            peg$currPos += 9;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c330);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parsePartitionedByToken() {
          var s0, s1, s2, s3, s4, s5, s6;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 11).toLowerCase() === peg$c331) {
            s2 = input.substr(peg$currPos, 11);
            peg$currPos += 11;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c332);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$parse__();
            if (s3 !== peg$FAILED) {
              if (input.substr(peg$currPos, 2).toLowerCase() === peg$c249) {
                s4 = input.substr(peg$currPos, 2);
                peg$currPos += 2;
              } else {
                s4 = peg$FAILED;
                if (peg$silentFails === 0) {
                  peg$fail(peg$c250);
                }
              }
              if (s4 !== peg$FAILED) {
                s5 = peg$currPos;
                peg$silentFails++;
                s6 = peg$parseIdentifierPart();
                peg$silentFails--;
                if (s6 === peg$FAILED) {
                  s5 = void 0;
                } else {
                  peg$currPos = s5;
                  s5 = peg$FAILED;
                }
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parsePositionToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 8).toLowerCase() === peg$c333) {
            s2 = input.substr(peg$currPos, 8);
            peg$currPos += 8;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c334);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseReplaceToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c335) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c336);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseRollupToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c337) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c338);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseRowToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 3).toLowerCase() === peg$c339) {
            s2 = input.substr(peg$currPos, 3);
            peg$currPos += 3;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c340);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSelectToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c341) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c342);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSimilarToToken() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 7).toLowerCase() === peg$c343) {
            s2 = input.substr(peg$currPos, 7);
            peg$currPos += 7;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c344);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s4 = peg$parse__();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseToToken();
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSomeToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c345) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c346);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseSymmetricToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 9).toLowerCase() === peg$c347) {
            s2 = input.substr(peg$currPos, 9);
            peg$currPos += 9;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c348);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTableToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c349) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c350);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseThenToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c351) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c352);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTimeToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c353) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c354);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTimestampToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 9).toLowerCase() === peg$c355) {
            s2 = input.substr(peg$currPos, 9);
            peg$currPos += 9;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c356);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTimestampaddToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 12).toLowerCase() === peg$c357) {
            s2 = input.substr(peg$currPos, 12);
            peg$currPos += 12;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c358);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTimestampdiffToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 13).toLowerCase() === peg$c359) {
            s2 = input.substr(peg$currPos, 13);
            peg$currPos += 13;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c360);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseToToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 2).toLowerCase() === peg$c361) {
            s2 = input.substr(peg$currPos, 2);
            peg$currPos += 2;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c362);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTrailingToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 8).toLowerCase() === peg$c363) {
            s2 = input.substr(peg$currPos, 8);
            peg$currPos += 8;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c364);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTrimToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c365) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c366);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseTrueToken() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          s2 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c367) {
            s3 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s3 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c368);
            }
          }
          if (s3 !== peg$FAILED) {
            s4 = peg$currPos;
            peg$silentFails++;
            s5 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s5 === peg$FAILED) {
              s4 = void 0;
            } else {
              peg$currPos = s4;
              s4 = peg$FAILED;
            }
            if (s4 !== peg$FAILED) {
              s3 = [s3, s4];
              s2 = s3;
            } else {
              peg$currPos = s2;
              s2 = peg$FAILED;
            }
          } else {
            peg$currPos = s2;
            s2 = peg$FAILED;
          }
          if (s2 !== peg$FAILED) {
            s1 = input.substring(s1, peg$currPos);
          } else {
            s1 = s2;
          }
          if (s1 !== peg$FAILED) {
            peg$savedPos = s0;
            s1 = peg$c369();
          }
          s0 = s1;
          return s0;
        }
        function peg$parseUnionToken() {
          var s0, s1, s2, s3, s4, s5;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c370) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c371);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s4 = peg$parse__();
              if (s4 !== peg$FAILED) {
                s5 = peg$parseAllToken();
                if (s5 !== peg$FAILED) {
                  s2 = [s2, s3, s4, s5];
                  s1 = s2;
                } else {
                  peg$currPos = s1;
                  s1 = peg$FAILED;
                }
              } else {
                peg$currPos = s1;
                s1 = peg$FAILED;
              }
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseValuesToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 6).toLowerCase() === peg$c372) {
            s2 = input.substr(peg$currPos, 6);
            peg$currPos += 6;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c373);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseWhenToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c374) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c375);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseWhereToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 5).toLowerCase() === peg$c376) {
            s2 = input.substr(peg$currPos, 5);
            peg$currPos += 5;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c377);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function peg$parseWithToken() {
          var s0, s1, s2, s3, s4;
          s0 = peg$currPos;
          s1 = peg$currPos;
          if (input.substr(peg$currPos, 4).toLowerCase() === peg$c378) {
            s2 = input.substr(peg$currPos, 4);
            peg$currPos += 4;
          } else {
            s2 = peg$FAILED;
            if (peg$silentFails === 0) {
              peg$fail(peg$c379);
            }
          }
          if (s2 !== peg$FAILED) {
            s3 = peg$currPos;
            peg$silentFails++;
            s4 = peg$parseIdentifierPart();
            peg$silentFails--;
            if (s4 === peg$FAILED) {
              s3 = void 0;
            } else {
              peg$currPos = s3;
              s3 = peg$FAILED;
            }
            if (s3 !== peg$FAILED) {
              s2 = [s2, s3];
              s1 = s2;
            } else {
              peg$currPos = s1;
              s1 = peg$FAILED;
            }
          } else {
            peg$currPos = s1;
            s1 = peg$FAILED;
          }
          if (s1 !== peg$FAILED) {
            s0 = input.substring(s0, peg$currPos);
          } else {
            s0 = s1;
          }
          return s0;
        }
        function makeListMap(tail, index, head) {
          if (head === void 0) {
            head = [];
          } else {
            head = [head];
          }
          return head.concat(tail.map(function(t) {
            return t[index];
          }));
        }
        function maybeMakeMulti(op, head, tail) {
          if (!tail.length)
            return head;
          return new sql.SqlMulti({
            op,
            args: new sql.SeparatedArray(makeListMap(tail, 3, head), makeSeparatorsList(tail))
          });
        }
        function makeSeparatorsList(tail) {
          return tail.map(function(t) {
            return new sql.Separator({
              left: t[0],
              separator: t[1],
              right: t[2]
            });
          });
        }
        function makeSeparatedArray(head, tail) {
          return new sql.SeparatedArray(makeListMap(tail, 1, head), makeListMap(tail, 0));
        }
        function makeFunctionName(f) {
          return Array.isArray(f) ? sql.RefName.functionName(f[1], true) : sql.RefName.functionName(f);
        }
        peg$result = peg$startRuleFunction();
        if (peg$result !== peg$FAILED && peg$currPos === input.length) {
          return peg$result;
        } else {
          if (peg$result !== peg$FAILED && peg$currPos < input.length) {
            peg$fail(peg$endExpectation());
          }
          throw peg$buildStructuredError(peg$maxFailExpected, peg$maxFailPos < input.length ? input.charAt(peg$maxFailPos) : null, peg$maxFailPos < input.length ? peg$computeLocation(peg$maxFailPos, peg$maxFailPos + 1) : peg$computeLocation(peg$maxFailPos, peg$maxFailPos));
        }
      }
      return {
        SyntaxError: peg$SyntaxError,
        parse: peg$parse
      };
    }();
    exports.parseSql = function parseSql4(input) {
      return p.parse(input);
    };
  }
});

// src/sql/sql-base.ts
function pseudoRandomCapitalization(str) {
  const upper = str.toUpperCase();
  const lower = str.toLowerCase();
  let seed = 0;
  return upper.split("").map((u, i) => {
    seed += u.charCodeAt(0);
    seed = (457 * seed + 1279) % 3631;
    return seed % 5 > 1 ? u : lower[i];
  }).join("");
}
function normalizeKeywordSpace(keyword) {
  return keyword.replace(/\/\*.*\*\//g, " ").replace(/--[^\n]*\n/gm, " ").replace(/\s+/gm, " ");
}
var import_parser, _SqlBase, SqlBase;
var init_sql_base = __esm({
  "src/sql/sql-base.ts"() {
    "use strict";
    init_utils();
    init_sql();
    import_parser = __toESM(require_parser());
    _SqlBase = class {
      static parseSql(input) {
        if (typeof input === "string") {
          return (0, import_parser.parseSql)(input);
        } else if (input instanceof _SqlBase) {
          return input;
        } else {
          throw new Error("unknown input");
        }
      }
      static setCapitalization(capitalization) {
        switch (capitalization) {
          case "upper":
            _SqlBase.capitalize = (k) => k;
            break;
          case "lower":
            _SqlBase.capitalize = (k) => k.toLowerCase();
            break;
          case "title":
            _SqlBase.capitalize = (k) => k.slice(0, 1) + k.slice(1).toLowerCase();
            break;
          case "random":
            _SqlBase.capitalize = pseudoRandomCapitalization;
            break;
          default:
            throw new Error(`unknown capitalization '${capitalization}'`);
        }
      }
      static walkSeparatedArray(a, stack, fn, postorder) {
        let stop = false;
        let changed = false;
        const newA = a.map((v) => {
          if (stop)
            return v;
          const ret = v._walkHelper(stack, fn, postorder);
          if (!ret) {
            stop = true;
            return v;
          }
          if (ret !== v) {
            changed = true;
          }
          return ret;
        });
        if (stop)
          return void 0;
        return changed ? newA : a;
      }
      static register(ex) {
        _SqlBase.classMap[ex.type] = ex;
      }
      static getConstructorFor(type) {
        const ClassFn = _SqlBase.classMap[type];
        if (!ClassFn)
          throw new Error(`unsupported expression type '${type}'`);
        return ClassFn;
      }
      static fromValue(parameters) {
        const { type } = parameters;
        if (!type)
          throw new Error(`must set 'type' to use fromValue`);
        const ClassFn = _SqlBase.getConstructorFor(type);
        return new ClassFn(parameters);
      }
      constructor(options, typeOverride) {
        const type = typeOverride || options.type;
        if (!type)
          throw new Error(`could not determine type`);
        this.type = type;
        this.spacing = cleanObject(options.spacing || {});
        this.keywords = cleanObject(options.keywords || {});
        this.parens = options.parens;
      }
      valueOf() {
        const value = {
          type: this.type,
          spacing: this.spacing,
          keywords: this.keywords
        };
        if (this.parens)
          value.parens = this.parens;
        return value;
      }
      toJSON() {
        return this.toString();
      }
      equals(other) {
        if (!other)
          return false;
        if (this === other)
          return true;
        if (this.type !== other.type)
          return false;
        return this.toString() === other.toString();
      }
      logicalEquals(other) {
        if (!other)
          return false;
        if (this === other)
          return true;
        if (this.type !== other.type)
          return false;
        return this.prettify().toString() === other.prettify().toString();
      }
      getParens() {
        return this.parens || [];
      }
      changeParens(parens) {
        const newParens = parens.length ? parens : void 0;
        if (this.parens === newParens)
          return this;
        const value = this.valueOf();
        value.parens = newParens;
        return _SqlBase.fromValue(value);
      }
      addParens(leftSpacing, rightSpacing) {
        return this.changeParens(this.getParens().concat({
          leftSpacing: leftSpacing || "",
          rightSpacing: rightSpacing || ""
        }));
      }
      ensureParens(leftSpacing, rightSpacing) {
        if (this.parens)
          return this;
        return this.addParens(leftSpacing, rightSpacing);
      }
      removeOwnParenSpaces() {
        if (!this.parens)
          return this;
        return this.changeParens(this.parens.map(() => ({
          leftSpacing: "",
          rightSpacing: ""
        })));
      }
      resetOwnSpacing() {
        if (Object.keys(this.spacing).length === 0)
          return this;
        const value = this.valueOf();
        value.spacing = {};
        return _SqlBase.fromValue(value);
      }
      resetOwnKeywords() {
        if (Object.keys(this.keywords).length === 0)
          return this;
        const value = this.valueOf();
        value.keywords = {};
        return _SqlBase.fromValue(value);
      }
      normalizeOwnKeywordSpaces() {
        if (Object.keys(this.keywords).length === 0)
          return this;
        const value = this.valueOf();
        value.keywords = objectMap(value.keywords || {}, normalizeKeywordSpace);
        return _SqlBase.fromValue(value);
      }
      clearOwnSeparators() {
        return this;
      }
      getSpace(name, defaultSpace = " ") {
        const s = this.spacing[name];
        return typeof s === "string" ? s : defaultSpace;
      }
      changeSpace(name, space) {
        return this.changeSpaces({ [name]: space });
      }
      changeSpaces(newSpacing) {
        const value = this.valueOf();
        value.spacing = __spreadValues(__spreadValues({}, value.spacing), newSpacing);
        return _SqlBase.fromValue(value);
      }
      getSpacingWithout(...toRemove) {
        const ret = __spreadValues({}, this.spacing);
        for (const thing of toRemove) {
          delete ret[thing];
        }
        return ret;
      }
      getKeyword(name, defaultKeyword, followSpace, defaultFollowSpace) {
        let keyword = this.keywords[name];
        if (typeof keyword === "undefined")
          keyword = _SqlBase.capitalize(defaultKeyword);
        if (keyword && followSpace)
          keyword += this.getSpace(followSpace, defaultFollowSpace);
        return keyword;
      }
      getKeywordsWithout(...toRemove) {
        const ret = __spreadValues({}, this.keywords);
        for (const thing of toRemove) {
          delete ret[thing];
        }
        return ret;
      }
      toString() {
        const { parens } = this;
        let str = this._toRawString();
        if (parens) {
          for (const paren of parens) {
            str = `(${paren.leftSpacing}${str}${paren.rightSpacing})`;
          }
        }
        return this.getSpace("initial", "") + str + this.getSpace("final", "");
      }
      walk(fn) {
        const ret = this._walkHelper([], fn, false);
        if (!ret)
          return this;
        return ret;
      }
      walkPostorder(fn) {
        const ret = this._walkHelper([], fn, true);
        if (!ret)
          return this;
        return ret;
      }
      _walkHelper(stack, fn, postorder) {
        let ret = this;
        if (!postorder) {
          ret = fn(ret, stack);
          if (!ret)
            return;
          if (ret !== this)
            return ret;
        }
        ret = ret._walkInner([ret].concat(stack), fn, postorder);
        if (!ret)
          return;
        if (postorder) {
          ret = fn(ret, stack);
          if (!ret)
            return;
        }
        return ret;
      }
      _walkInner(_nextStack, _fn, _postorder) {
        return this;
      }
      isPart() {
        return this.type.endsWith("Part");
      }
      some(fn) {
        let some = false;
        this.walk((b, s) => {
          some = some || fn(b, s);
          return some ? void 0 : b;
        });
        return some;
      }
      every(fn) {
        let every = true;
        this.walk((b, s) => {
          every = every && fn(b, s);
          return every ? b : void 0;
        });
        return every;
      }
      collect(fn) {
        const collected = [];
        this.walk((b, s) => {
          if (fn(b, s)) {
            collected.push(b);
          }
          return b;
        });
        return collected;
      }
      getColumns() {
        return this.collect((b) => b instanceof SqlColumn);
      }
      getUsedColumnNames() {
        return dedupe(this.getColumns().map((x) => x.getName())).sort();
      }
      contains(thing) {
        return this.some((b) => b.equals(thing));
      }
      containsColumnName(columnName) {
        return this.some((b) => b instanceof SqlColumn && b.getName() === columnName);
      }
      getFirstColumnName() {
        const column = this.getColumns()[0];
        if (!column)
          return;
        return column.getName();
      }
      prettify(options = {}) {
        const { keywordCasing } = options;
        return this.walkPostorder((ex) => {
          const resetSpaceEx = ex.resetOwnSpacing().clearOwnSeparators().removeOwnParenSpaces();
          return keywordCasing === "preserve" ? resetSpaceEx.normalizeOwnKeywordSpaces() : resetSpaceEx.resetOwnKeywords();
        });
      }
      prettyTrim(maxLength) {
        return this.walk((ex) => {
          if (ex instanceof SqlLiteral || ex instanceof SqlColumn) {
            return ex.prettyTrim(maxLength);
          }
          return ex;
        });
      }
      apply(fn) {
        return fn(this);
      }
      applyIf(condition, fn) {
        if (!condition)
          return this;
        return fn(this);
      }
      applyForEach(things, fn) {
        return things.reduce(fn, this);
      }
    };
    SqlBase = _SqlBase;
    SqlBase.type = "base";
    SqlBase.capitalize = (keyword) => {
      return keyword;
    };
    SqlBase.classMap = {};
  }
});

// src/sql/sql-expression.ts
var import_parser2, SqlExpression;
var init_sql_expression = __esm({
  "src/sql/sql-expression.ts"() {
    "use strict";
    init_utils();
    init_sql();
    import_parser2 = __toESM(require_parser());
    init_sql_base();
    init_utils2();
    SqlExpression = class extends SqlBase {
      static parse(input) {
        if (typeof input === "string") {
          const parsed = (0, import_parser2.parseSql)(input);
          if (!(parsed instanceof SqlExpression)) {
            throw new Error("Provided SQL was not an expression");
          }
          return parsed;
        } else if (input instanceof SqlExpression) {
          return input;
        } else {
          throw new Error("unknown input");
        }
      }
      static maybeParse(input) {
        try {
          return SqlExpression.parse(input);
        } catch {
          return;
        }
      }
      static wrap(input) {
        return input instanceof SqlExpression ? input : SqlLiteral.create(input);
      }
      static and(...args) {
        const compactArgs = filterMap(args, (a) => {
          if (!a)
            return;
          if (a instanceof SqlLiteral && a.value === true) {
            return;
          }
          if (a instanceof SqlMulti) {
            return a.ensureParens();
          }
          return a;
        });
        switch (compactArgs.length) {
          case 0:
            return SqlLiteral.TRUE;
          case 1:
            return compactArgs[0];
          default:
            return new SqlMulti({
              op: "AND",
              args: SeparatedArray.fromArray(compactArgs, Separator.symmetricSpace("AND"))
            });
        }
      }
      static or(...args) {
        const compactArgs = filterMap(args, (a) => {
          if (!a)
            return;
          if (a instanceof SqlLiteral && a.value === false) {
            return;
          }
          if (a instanceof SqlMulti) {
            return a.ensureParens();
          }
          return a;
        });
        switch (compactArgs.length) {
          case 0:
            return SqlLiteral.FALSE;
          case 1:
            return compactArgs[0];
          default:
            return new SqlMulti({
              op: "OR",
              args: SeparatedArray.fromArray(compactArgs, Separator.symmetricSpace("OR"))
            });
        }
      }
      static fromTimeExpressionAndInterval(time, interval) {
        if (Array.isArray(interval)) {
          return SqlExpression.or(...interval.map((int) => SqlExpression.fromTimeExpressionAndInterval(time, int)));
        }
        const parts = interval.split("/");
        if (parts.length !== 2)
          throw new Error(`can not convert interval: ${interval}`);
        const start = new Date(parts[0]);
        if (isNaN(start.valueOf()))
          throw new Error(`can not parse the start of interval: ${interval}`);
        const end = new Date(parts[1]);
        if (isNaN(end.valueOf()))
          throw new Error(`can not parse the end of interval: ${interval}`);
        return SqlLiteral.create(start).lessThanOrEqual(time).and(time.lessThan(SqlLiteral.create(end)));
      }
      static arrayOfLiterals(xs) {
        return SqlFunction.array(...xs);
      }
      _walkHelper(stack, fn, postorder) {
        const ret = super._walkHelper(stack, fn, postorder);
        if (!ret)
          return;
        if (ret === this)
          return this;
        if (ret instanceof SqlExpression) {
          return ret;
        } else {
          throw new Error("expression walker must return a SQL expression");
        }
      }
      _walkInner(_nextStack, _fn, _postorder) {
        return this;
      }
      as(alias, forceQuotes) {
        if (!alias)
          return this.getUnderlyingExpression();
        return SqlAlias.create(this, alias, forceQuotes);
      }
      ifUnnamedAliasAs(alias, forceQuotes) {
        return SqlAlias.create(this, alias, forceQuotes);
      }
      convertToTable() {
        return this;
      }
      getAliasName() {
        return;
      }
      getOutputName() {
        return;
      }
      getUnderlyingExpression() {
        return this;
      }
      toOrderByExpression(direction) {
        return SqlOrderByExpression.create(this, direction);
      }
      not() {
        return SqlUnary.not(this);
      }
      negate() {
        return this.not();
      }
      equal(rhs) {
        return SqlComparison.equal(this, rhs);
      }
      unequal(rhs) {
        return SqlComparison.unequal(this, rhs);
      }
      lessThan(rhs) {
        return SqlComparison.lessThan(this, rhs);
      }
      greaterThan(rhs) {
        return SqlComparison.greaterThan(this, rhs);
      }
      lessThanOrEqual(rhs) {
        return SqlComparison.lessThanOrEqual(this, rhs);
      }
      greaterThanOrEqual(rhs) {
        return SqlComparison.greaterThanOrEqual(this, rhs);
      }
      isNull() {
        return SqlComparison.isNull(this);
      }
      isNotNull() {
        return SqlComparison.isNotNull(this);
      }
      in(values) {
        return SqlComparison.in(this, values);
      }
      notIn(values) {
        return SqlComparison.notIn(this, values);
      }
      like(rhs, escape) {
        return SqlComparison.like(this, rhs, escape);
      }
      between(start, end) {
        return SqlComparison.between(this, start, end);
      }
      notBetween(start, end) {
        return SqlComparison.notBetween(this, start, end);
      }
      betweenSymmetric(start, end) {
        return SqlComparison.betweenSymmetric(this, start, end);
      }
      notBetweenSymmetric(start, end) {
        return SqlComparison.notBetweenSymmetric(this, start, end);
      }
      and(expression) {
        return SqlExpression.and(this, expression);
      }
      or(expression) {
        return SqlExpression.or(this, expression);
      }
      decomposeViaAnd(_options) {
        return [this];
      }
      decomposeViaOr(_options) {
        return [this];
      }
      filterAnd(fn) {
        if (!fn(this))
          return;
        return this;
      }
      removeColumnFromAnd(column) {
        return this.filterAnd((ex) => !ex.containsColumnName(column));
      }
      fillPlaceholders(fillWith) {
        let i = 0;
        return this.walk((ex) => {
          if (ex instanceof SqlPlaceholder) {
            if (i === fillWith.length) {
              return ex;
            }
            return SqlExpression.wrap(fillWith[i++]);
          }
          return ex;
        });
      }
      cast(asType) {
        return SqlFunction.cast(this, asType);
      }
      addFilterToAggregations(filter, knownAggregations) {
        return this.walk((x) => {
          if (x instanceof SqlFunction) {
            if (x.isAggregation(knownAggregations))
              return x.addWhereExpression(filter);
            const { args } = x;
            if (!args)
              return x;
            try {
              return x.changeArgs(args.map((a) => {
                return a.addFilterToAggregations(filter, knownAggregations);
              }));
            } catch (e) {
              if (e.message === "column reference outside aggregation") {
                return x.addWhereExpression(filter);
              }
              throw e;
            }
          } else if (x instanceof SqlColumn) {
            throw new Error("column reference outside aggregation");
          }
          return x;
        });
      }
    };
  }
});

// src/sql/sql-type/sql-type.ts
var _SqlType, SqlType;
var init_sql_type = __esm({
  "src/sql/sql-type/sql-type.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    _SqlType = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlType.type);
        this.value = options.value;
      }
      static create(value) {
        if (value instanceof _SqlType)
          return value;
        return new _SqlType({
          value
        });
      }
      static fromNativeType(value) {
        switch (value.toLowerCase()) {
          case "double":
            return _SqlType.DOUBLE;
          case "float":
            return _SqlType.FLOAT;
          case "long":
            return _SqlType.BIGINT;
          case "complex<json>":
            return _SqlType.create(`TYPE('COMPLEX<json>')`);
          default:
            return _SqlType.VARCHAR;
        }
      }
      valueOf() {
        const value = super.valueOf();
        value.value = this.value;
        return value;
      }
      _toRawString() {
        return this.value;
      }
      getEffectiveType() {
        const m = this.value.match(/^TYPE\((.+)\)$/i);
        if (m) {
          return `TYPE(${m[1]})`;
        }
        return this.value.toUpperCase();
      }
      getNativeType() {
        const sqlType = this.getEffectiveType();
        switch (sqlType) {
          case "DOUBLE":
          case "FLOAT":
            return sqlType.toLowerCase();
          case "TIMESTAMP":
          case "BIGINT":
            return "long";
          case `TYPE('COMPLEX<json>')`:
            return "COMPLEX<json>";
          default:
            return "string";
        }
      }
    };
    SqlType = _SqlType;
    SqlType.type = "type";
    SqlBase.register(SqlType);
    SqlType.VARCHAR = SqlType.create("VARCHAR");
    SqlType.DOUBLE = SqlType.create("DOUBLE");
    SqlType.FLOAT = SqlType.create("FLOAT");
    SqlType.BIGINT = SqlType.create("BIGINT");
    SqlType.TIMESTAMP = SqlType.create("TIMESTAMP");
  }
});

// src/sql/sql-column-list/sql-column-list.ts
var _SqlColumnList, SqlColumnList;
var init_sql_column_list = __esm({
  "src/sql/sql-column-list/sql-column-list.ts"() {
    "use strict";
    init_src();
    init_sql_base();
    init_utils2();
    _SqlColumnList = class extends SqlBase {
      constructor(options) {
        super(options, _SqlColumnList.type);
        this.columns = options.columns;
      }
      static create(columns) {
        if (columns instanceof _SqlColumnList)
          return columns;
        return new _SqlColumnList({
          columns: SeparatedArray.fromArray(columns, Separator.COMMA)
        }).ensureParens();
      }
      valueOf() {
        const value = super.valueOf();
        value.columns = this.columns;
        return value;
      }
      _toRawString() {
        return this.columns.toString(Separator.COMMA);
      }
      changeColumns(columns) {
        const value = this.valueOf();
        value.columns = SeparatedArray.fromArray(columns, Separator.COMMA);
        return SqlBase.fromValue(value);
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.columns = this.columns.clearSeparators();
        return SqlBase.fromValue(value);
      }
    };
    SqlColumnList = _SqlColumnList;
    SqlColumnList.type = "columnList";
    SqlBase.register(SqlColumnList);
  }
});

// src/sql/sql-placeholder/sql-placeholder.ts
var _SqlPlaceholder, SqlPlaceholder;
var init_sql_placeholder = __esm({
  "src/sql/sql-placeholder/sql-placeholder.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    _SqlPlaceholder = class extends SqlExpression {
      constructor(options = {}) {
        super(options, _SqlPlaceholder.type);
        this.customPlaceholder = options.customPlaceholder;
      }
      valueOf() {
        const value = super.valueOf();
        value.customPlaceholder = this.customPlaceholder;
        return value;
      }
      _toRawString() {
        return this.customPlaceholder || "?";
      }
      changeCustomPlaceholder(customPlaceholder) {
        const value = this.valueOf();
        value.customPlaceholder = customPlaceholder;
        return new _SqlPlaceholder(value);
      }
    };
    SqlPlaceholder = _SqlPlaceholder;
    SqlPlaceholder.type = "placeholder";
    SqlBase.register(SqlPlaceholder);
    SqlPlaceholder.PLACEHOLDER = new SqlPlaceholder();
  }
});

// src/sql/sql-literal/sql-literal.ts
function isDate(v) {
  return Boolean(v && typeof v.toISOString === "function");
}
function isInteger(v) {
  return isFinite(v) && Math.floor(v) === v;
}
var _SqlLiteral, SqlLiteral;
var init_sql_literal = __esm({
  "src/sql/sql-literal/sql-literal.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_utils2();
    _SqlLiteral = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlLiteral.type);
        this.value = options.value;
        this.stringValue = options.stringValue;
      }
      static create(value) {
        if (value instanceof _SqlLiteral)
          return value;
        switch (typeof value) {
          case "object":
            if (value !== null && !isDate(value)) {
              throw new TypeError("SqlLiteral invalid object input");
            }
            break;
          case "boolean":
          case "number":
          case "bigint":
          case "string":
            break;
          default:
            throw new TypeError(`SqlLiteral invalid input of type ${typeof value}`);
        }
        return new _SqlLiteral({
          value
        });
      }
      static double(value) {
        return new _SqlLiteral({
          value,
          stringValue: isInteger(value) ? value.toFixed(1) : void 0
        });
      }
      static maybe(value) {
        try {
          return _SqlLiteral.create(value);
        } catch {
          return;
        }
      }
      static index(n) {
        return _SqlLiteral.create(n + 1);
      }
      static direct(v) {
        if (v instanceof _SqlLiteral)
          return v;
        return new _SqlLiteral({
          value: v.toUpperCase(),
          stringValue: v
        });
      }
      static escapeLiteralString(str) {
        str = str.replace(/'/g, "''");
        if (needsUnicodeEscape(str)) {
          return `U&'${sqlEscapeUnicode(str)}'`;
        } else {
          return `'${str}'`;
        }
      }
      static dateToTimestampValue(date) {
        return date.toISOString().replace("T", " ").replace("Z", "").replace(/\.000$/, "").replace(/ 00:00:00$/, "");
      }
      static _equalsLiteral(expression, value) {
        return expression instanceof _SqlLiteral && expression.value === value;
      }
      valueOf() {
        const value = super.valueOf();
        value.value = this.value;
        value.stringValue = this.stringValue;
        return value;
      }
      getEffectiveStringValue() {
        const { value, stringValue } = this;
        if (stringValue)
          return stringValue;
        switch (typeof value) {
          case "object":
            if (value === null) {
              return SqlBase.capitalize(_SqlLiteral.DEFAULT_NULL_KEYWORD);
            } else if (isDate(value)) {
              return `'${_SqlLiteral.dateToTimestampValue(value)}'`;
            } else {
              return String(value);
            }
          case "boolean":
            return SqlBase.capitalize(value ? _SqlLiteral.DEFAULT_TRUE_KEYWORD : _SqlLiteral.DEFAULT_FALSE_KEYWORD);
          case "string":
            return _SqlLiteral.escapeLiteralString(value);
          default:
            return String(value);
        }
      }
      _toRawString() {
        const retParts = [];
        if (this.isDate()) {
          retParts.push(this.getKeyword("timestamp", _SqlLiteral.DEFAULT_TIMESTAMP_KEYWORD), this.getSpace("postTimestamp"));
        }
        retParts.push(this.getEffectiveStringValue());
        return retParts.join("");
      }
      resetOwnKeywords() {
        const { value, stringValue } = this;
        if (stringValue && (value === null || typeof value === "boolean")) {
          const v = this.valueOf();
          delete v.stringValue;
          return new _SqlLiteral(v).resetOwnKeywords();
        } else {
          return super.resetOwnKeywords();
        }
      }
      isIndex() {
        return typeof this.value === "number";
      }
      getIndexValue() {
        const { value } = this;
        if (typeof value !== "number")
          return -1;
        return Math.floor(value - 1);
      }
      incrementIndex(amount = 1) {
        if (!this.isIndex())
          return this;
        const value = this.valueOf();
        value.value = Math.floor(this.value) + amount;
        delete value.stringValue;
        return new _SqlLiteral(value);
      }
      prettyTrim(maxLength) {
        if (typeof this.value === "string") {
          return _SqlLiteral.create(trimString(this.value, maxLength));
        }
        return this;
      }
      decomposeViaAnd(_options) {
        return this.value === true ? [] : [this];
      }
      isInteger() {
        const { value, stringValue } = this;
        switch (typeof value) {
          case "number":
            if (typeof stringValue === "string") {
              return !stringValue.includes(".");
            }
            return isInteger(value);
          case "bigint":
            return true;
          default:
            return false;
        }
      }
      isDate() {
        return isDate(this.value);
      }
      getNumberValue() {
        const { value } = this;
        if (typeof value !== "number")
          return;
        return value;
      }
      getNumberOrBigintValue() {
        const { value } = this;
        if (typeof value !== "number" && typeof value !== "bigint")
          return;
        return value;
      }
      getStringValue() {
        const { value } = this;
        if (typeof value !== "string")
          return;
        return value;
      }
      getDateValue() {
        const { value } = this;
        if (!isDate(value))
          return;
        return value;
      }
    };
    SqlLiteral = _SqlLiteral;
    SqlLiteral.type = "literal";
    SqlLiteral.DEFAULT_TIMESTAMP_KEYWORD = "TIMESTAMP";
    SqlLiteral.DEFAULT_NULL_KEYWORD = "NULL";
    SqlLiteral.DEFAULT_FALSE_KEYWORD = "FALSE";
    SqlLiteral.DEFAULT_TRUE_KEYWORD = "TRUE";
    SqlBase.register(SqlLiteral);
    SqlLiteral.NULL = SqlLiteral.create(null);
    SqlLiteral.FALSE = SqlLiteral.create(false);
    SqlLiteral.TRUE = SqlLiteral.create(true);
    SqlLiteral.ZERO = SqlLiteral.create(0);
  }
});

// src/sql/sql-interval/sql-interval.ts
var _SqlInterval, SqlInterval;
var init_sql_interval = __esm({
  "src/sql/sql-interval/sql-interval.ts"() {
    "use strict";
    init_sql();
    init_sql_base();
    init_sql_expression();
    _SqlInterval = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlInterval.type);
        this.intervalValue = options.intervalValue;
        this.unit = options.unit;
      }
      static create(unit, intervalValue) {
        return new _SqlInterval({
          intervalValue: SqlLiteral.create(String(intervalValue)),
          unit
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.intervalValue = this.intervalValue;
        value.unit = this.unit;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("interval", _SqlInterval.DEFAULT_INTERVAL_KEYWORD),
          this.getSpace("postInterval"),
          this.intervalValue,
          this.getSpace("postIntervalValue"),
          this.unit
        ].join("");
      }
    };
    SqlInterval = _SqlInterval;
    SqlInterval.type = "interval";
    SqlInterval.DEFAULT_INTERVAL_KEYWORD = "INTERVAL";
    SqlBase.register(SqlInterval);
  }
});

// src/sql/special-functions.ts
var SPECIAL_FUNCTIONS;
var init_special_functions = __esm({
  "src/sql/special-functions.ts"() {
    "use strict";
    SPECIAL_FUNCTIONS = [
      "CURRENT_CATALOG",
      "CURRENT_DATE",
      "CURRENT_DEFAULT_TRANSFORM_GROUP",
      "CURRENT_PATH",
      "CURRENT_ROLE",
      "CURRENT_ROW",
      "CURRENT_SCHEMA",
      "CURRENT_TIME",
      "CURRENT_TIMESTAMP",
      "CURRENT_TRANSFORM_GROUP_FOR_TYPE",
      "CURRENT_USER",
      "PI"
    ];
  }
});

// src/sql/sql-clause/sql-clause.ts
var SqlClause;
var init_sql_clause = __esm({
  "src/sql/sql-clause/sql-clause.ts"() {
    "use strict";
    init_sql_base();
    SqlClause = class extends SqlBase {
      _walkHelper(stack, fn, postorder) {
        const ret = super._walkHelper(stack, fn, postorder);
        if (!ret)
          return;
        if (ret === this)
          return this;
        if (ret instanceof SqlClause) {
          return ret;
        } else {
          throw new Error("must return a sql clause");
        }
      }
      _walkInner(_nextStack, _fn, _postorder) {
        return this;
      }
    };
  }
});

// src/sql/sql-clause/sql-order-by-expression/sql-order-by-expression.ts
var _SqlOrderByExpression, SqlOrderByExpression;
var init_sql_order_by_expression = __esm({
  "src/sql/sql-clause/sql-order-by-expression/sql-order-by-expression.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    _SqlOrderByExpression = class extends SqlBase {
      constructor(options) {
        super(options, _SqlOrderByExpression.type);
        this.expression = options.expression;
        const direction = options.direction;
        this.direction = direction;
        if (direction) {
          if (direction !== "ASC" && direction !== "DESC") {
            throw new Error(`invalid direction ${direction}`);
          }
        }
      }
      static create(expression, direction) {
        return new _SqlOrderByExpression({
          expression,
          direction
        });
      }
      static index(value, direction) {
        return new _SqlOrderByExpression({
          expression: SqlLiteral.index(value),
          direction
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expression = this.expression;
        value.direction = this.direction;
        return value;
      }
      _toRawString() {
        const rawParts = [this.expression.toString()];
        if (this.direction) {
          rawParts.push(this.getSpace("preDirection"), this.getKeyword("direction", this.direction));
        }
        return rawParts.join("");
      }
      changeExpression(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      changeDirection(direction) {
        const value = this.valueOf();
        if (direction) {
          value.direction = direction;
        } else {
          delete value.direction;
          value.spacing = this.getSpacingWithout("preDirection");
        }
        value.keywords = this.getKeywordsWithout("direction");
        return SqlBase.fromValue(value);
      }
      getEffectiveDirection() {
        const { direction } = this;
        if (!direction)
          return "ASC";
        return direction;
      }
      reverseDirection() {
        return this.changeDirection(this.getEffectiveDirection() === "ASC" ? "DESC" : "ASC");
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expression = this.expression._walkHelper(nextStack, fn, postorder);
        if (!expression)
          return;
        if (expression !== this.expression) {
          ret = ret.changeExpression(expression);
        }
        return ret;
      }
      isIndex() {
        const { expression } = this;
        return expression instanceof SqlLiteral && expression.isIndex();
      }
      getIndexValue() {
        const { expression } = this;
        return expression instanceof SqlLiteral ? expression.getIndexValue() : -1;
      }
      incrementIndex(amount = 1) {
        const { expression } = this;
        if (expression instanceof SqlLiteral) {
          return this.changeExpression(expression.incrementIndex(amount));
        } else {
          return this;
        }
      }
    };
    SqlOrderByExpression = _SqlOrderByExpression;
    SqlOrderByExpression.type = "orderByExpression";
    SqlBase.register(SqlOrderByExpression);
  }
});

// src/sql/sql-clause/sql-order-by-clause/sql-order-by-clause.ts
var _SqlOrderByClause, SqlOrderByClause;
var init_sql_order_by_clause = __esm({
  "src/sql/sql-clause/sql-order-by-clause/sql-order-by-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    init_utils2();
    init_sql_clause();
    init_sql_order_by_expression();
    _SqlOrderByClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlOrderByClause.type);
        this.expressions = options.expressions;
      }
      static create(expressions) {
        if (expressions instanceof SqlOrderByExpression) {
          return new _SqlOrderByClause({
            expressions: SeparatedArray.fromSingleValue(expressions)
          });
        } else {
          return new _SqlOrderByClause({
            expressions: SeparatedArray.fromArray(expressions)
          });
        }
      }
      valueOf() {
        const value = super.valueOf();
        value.expressions = this.expressions;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("orderBy", _SqlOrderByClause.DEFAULT_ORDER_BY_KEYWORD),
          this.getSpace("postOrderBy"),
          this.expressions.toString()
        ].join("");
      }
      changeExpressions(expressions) {
        const value = this.valueOf();
        value.expressions = SeparatedArray.fromArray(expressions);
        return SqlBase.fromValue(value);
      }
      addExpression(expression, where = "end") {
        const { expressions } = this;
        return this.changeExpressions(expressions.insert(where === "start" ? 0 : Infinity, expression));
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expressions = SqlBase.walkSeparatedArray(this.expressions, nextStack, fn, postorder);
        if (!expressions)
          return;
        if (expressions !== this.expressions) {
          ret = ret.changeExpressions(expressions);
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.expressions = this.expressions.clearSeparators();
        return SqlBase.fromValue(value);
      }
      toArray() {
        return this.expressions.values;
      }
      addFirst(orderBy) {
        return this.changeExpressions(this.expressions.prepend(orderBy));
      }
      removeExpression(selectExpression, selectIndex) {
        if (!this.expressions)
          return this;
        const newExpression = this.expressions.filterMap((orderByExpression) => {
          const { expression } = orderByExpression;
          if (expression instanceof SqlLiteral && expression.isIndex()) {
            const expressionIndex = expression.getIndexValue();
            if (expressionIndex > selectIndex) {
              orderByExpression = orderByExpression.changeExpression(expression.incrementIndex(-1));
            } else if (expressionIndex === selectIndex) {
              return;
            }
          }
          if (expression.equals(selectExpression.getUnderlyingExpression()))
            return;
          return orderByExpression;
        });
        if (!newExpression)
          return;
        return this.changeExpressions(newExpression);
      }
      shiftIndexes(aboveIndex) {
        if (!this.expressions)
          return this;
        return this.changeExpressions(this.expressions.map((orderExpression) => {
          if (orderExpression.isIndex()) {
            const orderExpressionIndex = orderExpression.getIndexValue();
            if (aboveIndex <= orderExpressionIndex) {
              return orderExpression.incrementIndex();
            }
          }
          return orderExpression;
        }));
      }
    };
    SqlOrderByClause = _SqlOrderByClause;
    SqlOrderByClause.type = "orderByClause";
    SqlOrderByClause.DEFAULT_ORDER_BY_KEYWORD = "ORDER BY";
    SqlBase.register(SqlOrderByClause);
  }
});

// src/sql/sql-clause/sql-from-clause/sql-join-part.ts
var _SqlJoinPart, SqlJoinPart;
var init_sql_join_part = __esm({
  "src/sql/sql-clause/sql-from-clause/sql-join-part.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    _SqlJoinPart = class extends SqlBase {
      constructor(options) {
        super(options, _SqlJoinPart.type);
        this.joinType = options.joinType;
        this.table = options.table;
        this.onExpression = options.onExpression;
      }
      static create(joinType, table, onExpression) {
        if (Array.isArray(onExpression)) {
          onExpression = SqlExpression.and(...onExpression);
        }
        return new _SqlJoinPart({
          joinType,
          table: table.convertToTable(),
          onExpression
        });
      }
      static cross(table) {
        return new _SqlJoinPart({
          joinType: "CROSS",
          table: table.convertToTable()
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.joinType = this.joinType;
        value.table = this.table;
        value.onExpression = this.onExpression;
        return value;
      }
      _toRawString() {
        const rawParts = [];
        if (this.joinType) {
          rawParts.push(this.getKeyword("joinType", this.joinType), this.getSpace("postJoinType"));
        }
        rawParts.push(this.getKeyword("join", _SqlJoinPart.DEFAULT_JOIN_KEYWORD), this.getSpace("postJoin"), this.table.toString());
        if (this.onExpression) {
          rawParts.push(this.getSpace("preOn"), this.getKeyword("on", _SqlJoinPart.DEFAULT_ON_KEYWORD), this.getSpace("postOn"), this.onExpression.toString());
        }
        return rawParts.join("");
      }
      changeJoinTable(table) {
        const value = this.valueOf();
        value.table = table;
        return SqlBase.fromValue(value);
      }
      changeOnExpression(onExpression) {
        const value = this.valueOf();
        value.onExpression = onExpression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const table = this.table._walkHelper(nextStack, fn, postorder);
        if (!table)
          return;
        if (table !== this.table) {
          ret = ret.changeJoinTable(table);
        }
        if (this.onExpression) {
          const onExpression = this.onExpression._walkHelper(nextStack, fn, postorder);
          if (!onExpression)
            return;
          if (onExpression !== this.onExpression) {
            ret = ret.changeOnExpression(onExpression);
          }
        }
        return ret;
      }
    };
    SqlJoinPart = _SqlJoinPart;
    SqlJoinPart.type = "joinPart";
    SqlJoinPart.DEFAULT_JOIN_KEYWORD = "JOIN";
    SqlJoinPart.DEFAULT_ON_KEYWORD = "ON";
    SqlBase.register(SqlJoinPart);
  }
});

// src/sql/sql-column/sql-column.ts
var _SqlColumn, SqlColumn;
var init_sql_column = __esm({
  "src/sql/sql-column/sql-column.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_sql_table();
    init_utils2();
    _SqlColumn = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlColumn.type);
        this.refName = options.refName;
        this.table = options.table;
      }
      static create(name, table) {
        if (name instanceof _SqlColumn) {
          return table ? name.changeTable(SqlTable.create(table)) : name;
        }
        return new _SqlColumn({
          refName: RefName.create(name),
          table: table ? SqlTable.create(table) : void 0
        });
      }
      static optionalQuotes(name, table) {
        if (name instanceof _SqlColumn) {
          return table ? name.changeTable(SqlTable.create(table)) : name;
        }
        return new _SqlColumn({
          refName: RefName.create(name, false),
          table: table ? SqlTable.optionalQuotes(table) : void 0
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.refName = this.refName;
        value.table = this.table;
        return value;
      }
      _toRawString() {
        const { table, refName } = this;
        const rawParts = [];
        if (table) {
          rawParts.push(table.toString(), this.getSpace("postTable", ""), ".", this.getSpace("postDot", ""));
        }
        rawParts.push(refName.toString());
        return rawParts.join("");
      }
      changeRefName(refName) {
        const value = this.valueOf();
        value.refName = refName;
        return SqlBase.fromValue(value);
      }
      getName() {
        return this.refName.name;
      }
      changeName(name) {
        const { refName } = this;
        const value = this.valueOf();
        value.refName = refName.changeName(name);
        return SqlBase.fromValue(value);
      }
      changeTable(table) {
        const value = this.valueOf();
        if (table) {
          value.table = table;
        } else {
          delete value.table;
          value.spacing = this.getSpacingWithout("postTable", "postDot");
        }
        return SqlBase.fromValue(value);
      }
      getTableName() {
        var _a;
        return (_a = this.table) == null ? void 0 : _a.getName();
      }
      changeTableName(table) {
        return this.changeTable(table ? this.table ? this.table.changeName(table) : SqlTable.create(table) : void 0);
      }
      getNamespaceName() {
        var _a;
        return (_a = this.table) == null ? void 0 : _a.getNamespaceName();
      }
      prettyTrim(maxLength) {
        const { refName, table } = this;
        let ret = this.changeRefName(refName.prettyTrim(maxLength));
        if (table) {
          ret = ret.changeTable(table.prettyTrim(maxLength));
        }
        return ret;
      }
      convertToTable() {
        const { refName, table, spacing } = this;
        return new SqlTable({
          refName,
          namespace: table == null ? void 0 : table.convertToNamespace(),
          spacing: {
            postNamespace: spacing["postTable"],
            postDot: spacing["postDot"]
          }
        });
      }
      getOutputName() {
        return this.getName();
      }
    };
    SqlColumn = _SqlColumn;
    SqlColumn.type = "column";
    SqlBase.register(SqlColumn);
  }
});

// src/sql/sql-star/sql-star.ts
var _SqlStar, SqlStar;
var init_sql_star = __esm({
  "src/sql/sql-star/sql-star.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_sql_table();
    _SqlStar = class extends SqlExpression {
      constructor(options = {}) {
        super(options, _SqlStar.type);
        this.table = options.table;
      }
      static create(table) {
        return new _SqlStar({
          table
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.table = this.table;
        return value;
      }
      _toRawString() {
        const { table } = this;
        const rawParts = [];
        if (table) {
          rawParts.push(table.toString(), this.getSpace("postTable", ""), ".", this.getSpace("postDot", ""));
        }
        rawParts.push("*");
        return rawParts.join("");
      }
      changeTable(table) {
        const value = this.valueOf();
        if (table) {
          value.table = table;
        } else {
          delete value.table;
          value.spacing = this.getSpacingWithout("postTable", "postDot");
        }
        return SqlBase.fromValue(value);
      }
      getTableName() {
        var _a;
        return (_a = this.table) == null ? void 0 : _a.getName();
      }
      changeTableName(table) {
        return this.changeTable(table ? this.table ? this.table.changeName(table) : SqlTable.create(table) : void 0);
      }
      prettyTrim(maxLength) {
        const { table } = this;
        if (!table)
          return this;
        return this.changeTable(table.prettyTrim(maxLength));
      }
    };
    SqlStar = _SqlStar;
    SqlStar.type = "star";
    SqlBase.register(SqlStar);
    SqlStar.PLAIN = SqlStar.create();
  }
});

// src/sql/sql-table/sql-table.ts
var _SqlTable, SqlTable;
var init_sql_table = __esm({
  "src/sql/sql-table/sql-table.ts"() {
    "use strict";
    init_sql_base();
    init_sql_column();
    init_sql_expression();
    init_sql_namespace();
    init_sql_star();
    init_utils2();
    _SqlTable = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlTable.type);
        this.refName = options.refName;
        this.namespace = options.namespace;
      }
      static create(name, namespace) {
        if (name instanceof _SqlTable) {
          return namespace ? name.changeNamespace(SqlNamespace.create(namespace)) : name;
        }
        return new _SqlTable({
          refName: RefName.create(name),
          namespace: namespace ? SqlNamespace.create(namespace) : void 0
        });
      }
      static optionalQuotes(name, namespace) {
        if (name instanceof _SqlTable) {
          return namespace ? name.changeNamespace(SqlNamespace.create(namespace)) : name;
        }
        return new _SqlTable({
          refName: RefName.create(name, false),
          namespace: namespace ? SqlNamespace.optionalQuotes(namespace) : void 0
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.refName = this.refName;
        value.namespace = this.namespace;
        return value;
      }
      _toRawString() {
        const { namespace, refName } = this;
        const rawParts = [];
        if (namespace) {
          rawParts.push(namespace.toString(), this.getSpace("postNamespace", ""), ".", this.getSpace("postDot", ""));
        }
        rawParts.push(refName.toString());
        return rawParts.join("");
      }
      changeRefName(refName) {
        const value = this.valueOf();
        value.refName = refName;
        return SqlBase.fromValue(value);
      }
      getName() {
        return this.refName.name;
      }
      changeName(name) {
        const { refName } = this;
        const value = this.valueOf();
        value.refName = refName.changeName(name);
        return SqlBase.fromValue(value);
      }
      changeNamespace(namespace) {
        const value = this.valueOf();
        if (namespace) {
          value.namespace = namespace;
        } else {
          delete value.namespace;
          value.spacing = this.getSpacingWithout("postNamespace", "postDot");
        }
        return SqlBase.fromValue(value);
      }
      getNamespaceName() {
        var _a;
        return (_a = this.namespace) == null ? void 0 : _a.getName();
      }
      changeNamespaceName(namespace) {
        return this.changeNamespace(namespace ? this.namespace ? this.namespace.changeName(namespace) : SqlNamespace.create(namespace) : void 0);
      }
      prettyTrim(maxLength) {
        const { refName, namespace } = this;
        let ret = this.changeRefName(refName.prettyTrim(maxLength));
        if (namespace) {
          ret = ret.changeNamespace(namespace.prettyTrim(maxLength));
        }
        return ret;
      }
      convertToNamespace() {
        if (this.namespace) {
          throw new Error("can not convert");
        }
        return new SqlNamespace({
          refName: this.refName
        });
      }
      column(name) {
        return SqlColumn.create(name, this);
      }
      columnWithOptionalQuotes(name) {
        return SqlColumn.optionalQuotes(name, this);
      }
      star() {
        return SqlStar.create(this);
      }
    };
    SqlTable = _SqlTable;
    SqlTable.type = "table";
    SqlBase.register(SqlTable);
  }
});

// src/sql/sql-clause/sql-where-clause/sql-where-clause.ts
var _SqlWhereClause, SqlWhereClause;
var init_sql_where_clause = __esm({
  "src/sql/sql-clause/sql-where-clause/sql-where-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_clause();
    _SqlWhereClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlWhereClause.type);
        this.expression = options.expression;
      }
      static create(expression) {
        if (expression instanceof _SqlWhereClause)
          return expression;
        return new _SqlWhereClause({
          expression
        });
      }
      static createForFunction(expression) {
        return _SqlWhereClause.create(expression).addParens();
      }
      valueOf() {
        const value = super.valueOf();
        value.expression = this.expression;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("where", _SqlWhereClause.DEFAULT_WHERE_KEYWORD),
          this.getSpace("postWhere"),
          this.expression.toString()
        ].join("");
      }
      changeExpression(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expression = this.expression._walkHelper(nextStack, fn, postorder);
        if (!expression)
          return;
        if (expression !== this.expression) {
          ret = ret.changeExpression(expression);
        }
        return ret;
      }
      removeColumnFromAnd(column) {
        const newExpression = this.expression.removeColumnFromAnd(column);
        if (!newExpression)
          return;
        return this.changeExpression(newExpression);
      }
    };
    SqlWhereClause = _SqlWhereClause;
    SqlWhereClause.type = "whereClause";
    SqlWhereClause.DEFAULT_WHERE_KEYWORD = "WHERE";
    SqlBase.register(SqlWhereClause);
  }
});

// src/sql/sql-clause/sql-replace-clause/sql-replace-clause.ts
var _SqlReplaceClause, SqlReplaceClause;
var init_sql_replace_clause = __esm({
  "src/sql/sql-clause/sql-replace-clause/sql-replace-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_table();
    init_sql_clause();
    init_sql_where_clause();
    _SqlReplaceClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlReplaceClause.type);
        this.table = options.table;
        this.columns = options.columns;
        this.whereClause = options.whereClause;
      }
      static create(table, where) {
        if (table instanceof _SqlReplaceClause)
          return table;
        if (typeof table === "string") {
          table = SqlTable.create(table);
        }
        let whereClause;
        if (where) {
          whereClause = where instanceof SqlWhereClause ? where : SqlWhereClause.create(where);
        }
        return new _SqlReplaceClause({
          table,
          whereClause
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.table = this.table;
        value.columns = this.columns;
        value.whereClause = this.whereClause;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("replace", _SqlReplaceClause.DEFAULT_REPLACE_KEYWORD),
          this.getSpace("postReplace"),
          this.getKeyword("into", _SqlReplaceClause.DEFAULT_INTO_KEYWORD),
          this.getSpace("postInto"),
          this.table.toString()
        ];
        if (this.columns) {
          rawParts.push(this.getSpace("preColumns"), this.columns.toString());
        }
        rawParts.push(this.getSpace("preOverwrite"), this.getKeyword("overwrite", _SqlReplaceClause.DEFAULT_OVERWRITE_KEYWORD), this.getSpace("postOverwrite"));
        if (this.whereClause) {
          rawParts.push(this.whereClause.toString());
        } else {
          rawParts.push(this.getKeyword("all", _SqlReplaceClause.DEFAULT_ALL_KEYWORD));
        }
        return rawParts.join("");
      }
      changeTable(table) {
        const value = this.valueOf();
        if (typeof table === "string") {
          table = SqlTable.create(table);
        }
        value.table = table;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const table = this.table._walkHelper(nextStack, fn, postorder);
        if (!table)
          return;
        if (!(table instanceof SqlTable)) {
          throw new Error("must return table");
        }
        if (table !== this.table) {
          ret = ret.changeTable(table);
        }
        if (this.whereClause) {
          const whereClause = this.whereClause._walkHelper(nextStack, fn, postorder);
          if (!whereClause)
            return;
          if (whereClause !== this.whereClause) {
            ret = ret.changeWhereClause(whereClause);
          }
        }
        return ret;
      }
      changeWhereClause(whereClause) {
        const value = this.valueOf();
        if (whereClause) {
          value.whereClause = whereClause;
        } else {
          delete value.whereClause;
          value.keywords = this.getKeywordsWithout("filter");
        }
        return SqlBase.fromValue(value);
      }
    };
    SqlReplaceClause = _SqlReplaceClause;
    SqlReplaceClause.type = "replaceClause";
    SqlReplaceClause.DEFAULT_REPLACE_KEYWORD = "REPLACE";
    SqlReplaceClause.DEFAULT_INTO_KEYWORD = "INTO";
    SqlReplaceClause.DEFAULT_OVERWRITE_KEYWORD = "OVERWRITE";
    SqlReplaceClause.DEFAULT_ALL_KEYWORD = "ALL";
    SqlBase.register(SqlReplaceClause);
  }
});

// src/sql/sql-query/sql-query.ts
var import_parser3, INDENT_SPACE, _SqlQuery, SqlQuery;
var init_sql_query = __esm({
  "src/sql/sql-query/sql-query.ts"() {
    "use strict";
    init_utils();
    import_parser3 = __toESM(require_parser());
    init_sql_base();
    init_sql_clause2();
    init_sql_replace_clause();
    init_sql_column();
    init_sql_expression();
    init_sql_literal();
    init_sql_star();
    init_sql_table();
    init_utils2();
    INDENT_SPACE = "\n  ";
    _SqlQuery = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlQuery.type);
        this.explain = options.explain;
        this.insertClause = options.insertClause;
        this.replaceClause = options.replaceClause;
        if (this.insertClause && this.replaceClause) {
          throw new Error("SqlQuery can not have both an insertClause and a replaceClause");
        }
        this.withClause = options.withClause;
        this.decorator = options.decorator;
        this.selectExpressions = options.selectExpressions;
        this.fromClause = options.fromClause;
        this.whereClause = options.whereClause;
        this.groupByClause = options.groupByClause;
        this.havingClause = options.havingClause;
        this.orderByClause = options.orderByClause;
        this.limitClause = options.limitClause;
        this.offsetClause = options.offsetClause;
        this.partitionedByClause = options.partitionedByClause;
        this.clusteredByClause = options.clusteredByClause;
        this.unionQuery = options.unionQuery;
      }
      static create(from) {
        return new _SqlQuery({
          selectExpressions: SeparatedArray.fromSingleValue(SqlStar.PLAIN),
          fromClause: from instanceof SqlFromClause ? from : SqlFromClause.create(SeparatedArray.fromSingleValue(typeof from === "string" ? SqlTable.create(from) : from.convertToTable()))
        });
      }
      static from(from) {
        return new _SqlQuery({
          fromClause: from instanceof SqlFromClause ? from : SqlFromClause.create(SeparatedArray.fromSingleValue(typeof from === "string" ? SqlTable.create(from) : from.convertToTable()))
        });
      }
      static parse(input) {
        if (typeof input === "string") {
          const parsed = (0, import_parser3.parseSql)(input);
          if (!(parsed instanceof _SqlQuery)) {
            throw new Error("Provided SQL was not a query");
          }
          return parsed;
        } else if (input instanceof _SqlQuery) {
          return input;
        } else {
          throw new Error("unknown input");
        }
      }
      static maybeParse(input) {
        try {
          return _SqlQuery.parse(input);
        } catch {
          return;
        }
      }
      static getSelectExpressionOutput(selectExpression, i) {
        return selectExpression.getOutputName() || `EXPR$${i}`;
      }
      static isPhonyOutputName(name) {
        return /^EXPR\$(?:\d|[1-9]\d*)$/.test(name);
      }
      valueOf() {
        const value = super.valueOf();
        value.explain = this.explain;
        value.insertClause = this.insertClause;
        value.replaceClause = this.replaceClause;
        value.withClause = this.withClause;
        value.decorator = this.decorator;
        value.selectExpressions = this.selectExpressions;
        value.fromClause = this.fromClause;
        value.whereClause = this.whereClause;
        value.groupByClause = this.groupByClause;
        value.havingClause = this.havingClause;
        value.orderByClause = this.orderByClause;
        value.limitClause = this.limitClause;
        value.offsetClause = this.offsetClause;
        value.partitionedByClause = this.partitionedByClause;
        value.clusteredByClause = this.clusteredByClause;
        value.unionQuery = this.unionQuery;
        return value;
      }
      _toRawString() {
        const {
          explain,
          insertClause,
          replaceClause,
          withClause,
          decorator,
          selectExpressions,
          fromClause,
          whereClause,
          groupByClause,
          havingClause,
          orderByClause,
          limitClause,
          offsetClause,
          partitionedByClause,
          clusteredByClause,
          unionQuery
        } = this;
        const rawParts = [];
        if (explain) {
          rawParts.push(this.getKeyword("explainPlanFor", _SqlQuery.DEFAULT_EXPLAIN_PLAN_FOR_KEYWORD), this.getSpace("postExplainPlanFor", "\n"));
        }
        if (insertClause) {
          rawParts.push(insertClause.toString(), this.getSpace("postInsertClause", "\n"));
        } else if (replaceClause) {
          rawParts.push(replaceClause.toString(), this.getSpace("postReplaceClause", "\n"));
        }
        if (withClause) {
          rawParts.push(withClause.toString(), this.getSpace("postWithClause", "\n"));
        }
        const indentSpace = selectExpressions && selectExpressions.length() > 1 ? INDENT_SPACE : " ";
        rawParts.push(this.getKeyword("select", _SqlQuery.DEFAULT_SELECT_KEYWORD), this.getSpace("postSelect", decorator ? " " : indentSpace));
        if (this.decorator) {
          rawParts.push(this.getKeyword("decorator", this.decorator), this.getSpace("postDecorator", indentSpace));
        }
        rawParts.push(selectExpressions ? selectExpressions.toString(new Separator({ separator: ",", right: indentSpace })) : "...");
        if (fromClause) {
          rawParts.push(this.getSpace("preFromClause", "\n"), fromClause.toString());
        }
        if (whereClause) {
          rawParts.push(this.getSpace("preWhereClause", "\n"), whereClause.toString());
        }
        if (unionQuery) {
          rawParts.push(this.getSpace("preUnion", "\n"), this.getKeyword("union", _SqlQuery.DEFAULT_UNION_KEYWORD), this.getSpace("postUnion"), unionQuery.toString());
        }
        if (groupByClause) {
          rawParts.push(this.getSpace("preGroupByClause", "\n"), groupByClause.toString());
        }
        if (havingClause) {
          rawParts.push(this.getSpace("preHavingClause", "\n"), havingClause.toString());
        }
        if (orderByClause) {
          rawParts.push(this.getSpace("preOrderByClause", "\n"), orderByClause.toString());
        }
        if (limitClause) {
          rawParts.push(this.getSpace("preLimitClause", "\n"), limitClause.toString());
        }
        if (offsetClause) {
          rawParts.push(this.getSpace("preOffsetClause", "\n"), offsetClause.toString());
        }
        if (partitionedByClause) {
          rawParts.push(this.getSpace("prePartitionedByClause", "\n"), partitionedByClause.toString());
        }
        if (clusteredByClause) {
          rawParts.push(this.getSpace("preClusteredByClause", "\n"), clusteredByClause.toString());
        }
        return rawParts.join("");
      }
      changeExplain(explain) {
        if (this.explain === explain)
          return this;
        const value = this.valueOf();
        if (explain) {
          value.explain = true;
        } else {
          delete value.explain;
          value.spacing = this.getSpacingWithout("postExplainPlanFor");
        }
        return SqlBase.fromValue(value);
      }
      changeInsertClause(insertClause) {
        if (this.insertClause === insertClause)
          return this;
        const value = this.valueOf();
        if (insertClause) {
          value.insertClause = insertClause;
        } else {
          delete value.insertClause;
          value.spacing = this.getSpacingWithout("postInsertClause");
        }
        return SqlBase.fromValue(value);
      }
      changeReplaceClause(replaceClause) {
        if (this.replaceClause === replaceClause)
          return this;
        const value = this.valueOf();
        if (replaceClause) {
          value.replaceClause = replaceClause;
        } else {
          delete value.replaceClause;
          value.spacing = this.getSpacingWithout("postReplaceClause");
        }
        return SqlBase.fromValue(value);
      }
      changeWithClause(withClause) {
        if (this.withClause === withClause)
          return this;
        const value = this.valueOf();
        if (withClause) {
          value.withClause = withClause;
        } else {
          delete value.withClause;
          value.spacing = this.getSpacingWithout("postWithClause");
        }
        return SqlBase.fromValue(value);
      }
      changeSelectExpressions(selectExpressions) {
        if (this.selectExpressions === selectExpressions)
          return this;
        const value = this.valueOf();
        if (selectExpressions) {
          value.selectExpressions = SeparatedArray.fromArray(selectExpressions);
        } else {
          delete value.selectExpressions;
        }
        return SqlBase.fromValue(value);
      }
      changeFromClause(fromClause) {
        if (this.fromClause === fromClause)
          return this;
        const value = this.valueOf();
        if (fromClause) {
          value.fromClause = fromClause;
        } else {
          delete value.fromClause;
          value.spacing = this.getSpacingWithout("preFromClause");
        }
        return SqlBase.fromValue(value);
      }
      changeFromExpressions(fromExpressions) {
        if (!fromExpressions)
          return this.changeFromClause(void 0);
        return this.changeFromClause(this.fromClause ? this.fromClause.changeExpressions(fromExpressions) : SqlFromClause.create(fromExpressions));
      }
      changeWhereClause(whereClause) {
        if (this.whereClause === whereClause)
          return this;
        const value = this.valueOf();
        if (whereClause) {
          value.whereClause = whereClause;
        } else {
          delete value.whereClause;
          value.spacing = this.getSpacingWithout("preWhereClause");
        }
        return SqlBase.fromValue(value);
      }
      changeWhereExpression(whereExpression) {
        if (!whereExpression)
          return this.changeWhereClause(void 0);
        return this.changeWhereClause(this.whereClause ? this.whereClause.changeExpression(whereExpression) : SqlWhereClause.create(whereExpression));
      }
      changeGroupByClause(groupByClause) {
        if (this.groupByClause === groupByClause)
          return this;
        const value = this.valueOf();
        if (groupByClause) {
          value.groupByClause = groupByClause;
        } else {
          delete value.groupByClause;
          value.spacing = this.getSpacingWithout("preGroupByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeGroupByExpressions(groupByExpressions) {
        if (typeof groupByExpressions === "undefined")
          return this.changeGroupByClause(void 0);
        return this.changeGroupByClause(this.groupByClause ? this.groupByClause.changeExpressions(groupByExpressions) : SqlGroupByClause.create(groupByExpressions));
      }
      changeHavingClause(havingClause) {
        if (this.havingClause === havingClause)
          return this;
        const value = this.valueOf();
        if (havingClause) {
          value.havingClause = havingClause;
        } else {
          delete value.havingClause;
          value.spacing = this.getSpacingWithout("preHavingClause");
        }
        return SqlBase.fromValue(value);
      }
      changeHavingExpression(havingExpression) {
        if (!havingExpression)
          return this.changeHavingClause(void 0);
        return this.changeHavingClause(this.havingClause ? this.havingClause.changeExpression(havingExpression) : SqlHavingClause.create(havingExpression));
      }
      changeOrderByClause(orderByClause) {
        if (this.orderByClause === orderByClause)
          return this;
        const value = this.valueOf();
        if (orderByClause) {
          value.orderByClause = orderByClause;
        } else {
          delete value.orderByClause;
          value.spacing = this.getSpacingWithout("preOrderByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOrderByExpressions(orderByExpressions) {
        if (!orderByExpressions || isEmptyArray(orderByExpressions)) {
          return this.changeOrderByClause(void 0);
        } else {
          return this.changeOrderByClause(this.orderByClause ? this.orderByClause.changeExpressions(orderByExpressions) : SqlOrderByClause.create(orderByExpressions));
        }
      }
      changeOrderByExpression(orderByExpression) {
        if (!orderByExpression)
          return this.changeOrderByClause(void 0);
        return this.changeOrderByExpressions([orderByExpression]);
      }
      changeLimitClause(limitClause) {
        if (this.limitClause === limitClause)
          return this;
        const value = this.valueOf();
        if (limitClause) {
          value.limitClause = limitClause;
        } else {
          delete value.limitClause;
          value.spacing = this.getSpacingWithout("preLimitClause");
        }
        return SqlBase.fromValue(value);
      }
      changeLimitValue(limitValue) {
        if (typeof limitValue === "undefined")
          return this.changeLimitClause(void 0);
        return this.changeLimitClause(this.limitClause ? this.limitClause.changeLimit(limitValue) : SqlLimitClause.create(limitValue));
      }
      changeOffsetClause(offsetClause) {
        if (this.offsetClause === offsetClause)
          return this;
        const value = this.valueOf();
        if (offsetClause) {
          value.offsetClause = offsetClause;
        } else {
          delete value.offsetClause;
          value.spacing = this.getSpacingWithout("preOffsetClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOffsetValue(offsetValue) {
        if (typeof offsetValue === "undefined")
          return this.changeOffsetClause(void 0);
        return this.changeOffsetClause(this.offsetClause ? this.offsetClause.changeOffset(offsetValue) : SqlOffsetClause.create(offsetValue));
      }
      changePartitionedByClause(partitionedByClause) {
        if (this.partitionedByClause === partitionedByClause)
          return this;
        const value = this.valueOf();
        if (partitionedByClause) {
          value.partitionedByClause = partitionedByClause;
        } else {
          delete value.partitionedByClause;
          value.spacing = this.getSpacingWithout("prePartitionedByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeClusteredByClause(clusteredByClause) {
        if (this.clusteredByClause === clusteredByClause)
          return this;
        const value = this.valueOf();
        if (clusteredByClause) {
          value.clusteredByClause = clusteredByClause;
        } else {
          delete value.clusteredByClause;
          value.spacing = this.getSpacingWithout("preClusteredByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeClusteredByExpressions(clusteredByExpressions) {
        if (!clusteredByExpressions || isEmptyArray(clusteredByExpressions)) {
          return this.changeClusteredByClause(void 0);
        } else {
          return this.changeClusteredByClause(this.clusteredByClause ? this.clusteredByClause.changeExpressions(clusteredByExpressions) : SqlClusteredByClause.create(clusteredByExpressions));
        }
      }
      changeUnionQuery(unionQuery) {
        const value = this.valueOf();
        if (typeof unionQuery === "undefined") {
          delete value.unionQuery;
          value.spacing = this.getSpacingWithout("preUnion", "postUnion");
          value.keywords = this.getKeywordsWithout("union");
        } else {
          value.unionQuery = unionQuery;
        }
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.insertClause) {
          const insertClause = this.insertClause._walkHelper(nextStack, fn, postorder);
          if (!insertClause)
            return;
          if (insertClause !== this.insertClause) {
            ret = ret.changeInsertClause(insertClause);
          }
        } else if (this.replaceClause) {
          const replaceClause = this.replaceClause._walkHelper(nextStack, fn, postorder);
          if (!replaceClause)
            return;
          if (replaceClause !== this.replaceClause) {
            ret = ret.changeReplaceClause(replaceClause);
          }
        }
        if (this.withClause) {
          const withClause = this.withClause._walkHelper(nextStack, fn, postorder);
          if (!withClause)
            return;
          if (withClause !== this.withClause) {
            ret = ret.changeWithClause(withClause);
          }
        }
        if (this.selectExpressions) {
          const selectExpressions = SqlBase.walkSeparatedArray(this.selectExpressions, nextStack, fn, postorder);
          if (!selectExpressions)
            return;
          if (selectExpressions !== this.selectExpressions) {
            ret = ret.changeSelectExpressions(selectExpressions);
          }
        }
        if (this.fromClause) {
          const fromClause = this.fromClause._walkHelper(nextStack, fn, postorder);
          if (!fromClause)
            return;
          if (fromClause !== this.fromClause) {
            ret = ret.changeFromClause(fromClause);
          }
        }
        if (this.whereClause) {
          const whereClause = this.whereClause._walkHelper(nextStack, fn, postorder);
          if (!whereClause)
            return;
          if (whereClause !== this.whereClause) {
            ret = ret.changeWhereClause(whereClause);
          }
        }
        if (this.groupByClause) {
          const groupByClause = this.groupByClause._walkHelper(nextStack, fn, postorder);
          if (!groupByClause)
            return;
          if (groupByClause !== this.groupByClause) {
            ret = ret.changeGroupByClause(groupByClause);
          }
        }
        if (this.havingClause) {
          const havingClause = this.havingClause._walkHelper(nextStack, fn, postorder);
          if (!havingClause)
            return;
          if (havingClause !== this.havingClause) {
            ret = ret.changeHavingClause(havingClause);
          }
        }
        if (this.orderByClause) {
          const orderByClause = this.orderByClause._walkHelper(nextStack, fn, postorder);
          if (!orderByClause)
            return;
          if (orderByClause !== this.orderByClause) {
            ret = ret.changeOrderByClause(orderByClause);
          }
        }
        if (this.limitClause) {
          const limitClause = this.limitClause._walkHelper(nextStack, fn, postorder);
          if (!limitClause)
            return;
          if (limitClause !== this.limitClause) {
            ret = ret.changeLimitClause(limitClause);
          }
        }
        if (this.offsetClause) {
          const offsetClause = this.offsetClause._walkHelper(nextStack, fn, postorder);
          if (!offsetClause)
            return;
          if (offsetClause !== this.offsetClause) {
            ret = ret.changeOffsetClause(offsetClause);
          }
        }
        if (this.partitionedByClause) {
          const partitionedByClause = this.partitionedByClause._walkHelper(nextStack, fn, postorder);
          if (!partitionedByClause)
            return;
          if (partitionedByClause !== this.partitionedByClause) {
            ret = ret.changePartitionedByClause(partitionedByClause);
          }
        }
        if (this.clusteredByClause) {
          const clusteredByClause = this.clusteredByClause._walkHelper(nextStack, fn, postorder);
          if (!clusteredByClause)
            return;
          if (clusteredByClause !== this.clusteredByClause) {
            ret = ret.changeClusteredByClause(clusteredByClause);
          }
        }
        if (this.unionQuery) {
          const unionQuery = this.unionQuery._walkHelper(nextStack, fn, postorder);
          if (!unionQuery)
            return;
          if (unionQuery !== this.unionQuery) {
            ret = ret.changeUnionQuery(unionQuery);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        var _a;
        const value = this.valueOf();
        value.selectExpressions = (_a = this.selectExpressions) == null ? void 0 : _a.clearSeparators();
        return SqlBase.fromValue(value);
      }
      makeExplain() {
        return this.changeExplain(true);
      }
      getInsertIntoTable() {
        var _a;
        return (_a = this.insertClause) == null ? void 0 : _a.table;
      }
      changeInsertIntoTable(table) {
        return this.changeInsertClause(table ? this.insertClause ? this.insertClause.changeTable(table) : SqlInsertClause.create(table) : void 0);
      }
      getReplaceIntoTable() {
        var _a;
        return (_a = this.replaceClause) == null ? void 0 : _a.table;
      }
      changeReplaceIntoTable(table) {
        return this.changeReplaceClause(table ? this.replaceClause ? this.replaceClause.changeTable(table) : SqlReplaceClause.create(table) : void 0);
      }
      getIngestTable() {
        return this.getInsertIntoTable() || this.getReplaceIntoTable();
      }
      getWithParts() {
        const { withClause } = this;
        if (!withClause)
          return [];
        return withClause.withParts.values;
      }
      changeWithParts(withParts) {
        if (!withParts || isEmptyArray(withParts)) {
          return this.changeWithClause(void 0);
        } else {
          return this.changeWithClause(this.withClause ? this.withClause.changeWithParts(withParts) : SqlWithClause.create(withParts));
        }
      }
      prependWith(name, query) {
        return this.changeWithParts([SqlWithPart.simple(name, query.ensureParens())].concat(this.getWithParts()));
      }
      flattenWith() {
        return this;
      }
      isValidSelectIndex(selectIndex) {
        const { selectExpressions } = this;
        if (!selectExpressions)
          return false;
        return 0 <= selectIndex && selectIndex < selectExpressions.length();
      }
      getSelectExpressionsArray() {
        var _a;
        return ((_a = this.selectExpressions) == null ? void 0 : _a.values) || [];
      }
      getSelectExpressionForIndex(selectIndex) {
        var _a;
        return (_a = this.selectExpressions) == null ? void 0 : _a.get(selectIndex);
      }
      hasStarInSelect() {
        return this.getSelectExpressionsArray().some((v) => v instanceof SqlStar);
      }
      getOutputColumns() {
        return this.getSelectExpressionsArray().map(_SqlQuery.getSelectExpressionOutput);
      }
      getSelectIndexesForColumn(column) {
        return filterMap(this.getSelectExpressionsArray(), (selectExpression, i) => {
          return selectExpression.containsColumnName(column) ? i : void 0;
        });
      }
      getGroupedSelectIndexesForColumn(column) {
        return this.getSelectIndexesForColumn(column).filter((selectIndex) => this.isGroupedSelectIndex(selectIndex));
      }
      getSelectIndexForOutputColumn(outputColumn) {
        return this.getSelectExpressionsArray().findIndex((selectExpression, i) => {
          return _SqlQuery.getSelectExpressionOutput(selectExpression, i) === outputColumn;
        });
      }
      expressionRefersToSelectIndex(ex, selectIndex, allowAliasReferences) {
        const selectExpression = this.getSelectExpressionForIndex(selectIndex);
        if (!selectExpression)
          return false;
        if (ex instanceof SqlLiteral && ex.isIndex()) {
          return selectIndex === ex.getIndexValue();
        }
        if (allowAliasReferences && ex instanceof SqlColumn) {
          return selectExpression.getOutputName() === ex.getName();
        }
        return ex.equals(selectExpression.getUnderlyingExpression());
      }
      getSelectIndexForExpression(ex, allowAliasReferences) {
        const selectExpressionsArray = this.getSelectExpressionsArray();
        if (ex instanceof SqlLiteral && ex.isIndex()) {
          const idx = ex.getIndexValue();
          return this.isValidSelectIndex(idx) ? idx : -1;
        }
        if (allowAliasReferences && ex instanceof SqlColumn) {
          const refIdx = selectExpressionsArray.findIndex((selectExpression, i) => {
            return _SqlQuery.getSelectExpressionOutput(selectExpression, i) === ex.getName();
          });
          if (refIdx !== -1) {
            return refIdx;
          }
        }
        return selectExpressionsArray.findIndex((selectExpression) => {
          return ex.equals(selectExpression.getUnderlyingExpression());
        });
      }
      isRealOutputColumnAtSelectIndex(selectIndex) {
        const selectExpression = this.getSelectExpressionForIndex(selectIndex);
        if (!selectExpression)
          return false;
        return Boolean(selectExpression.getOutputName());
      }
      isGroupedSelectIndex(selectIndex) {
        const { groupByClause } = this;
        if (!groupByClause || !this.isValidSelectIndex(selectIndex))
          return false;
        return groupByClause.toArray().some((ex) => this.expressionRefersToSelectIndex(ex, selectIndex, false));
      }
      isGroupedOutputColumn(outputColumn) {
        return this.isGroupedSelectIndex(this.getSelectIndexForOutputColumn(outputColumn));
      }
      getGroupedSelectExpressions() {
        if (!this.hasGroupBy())
          return [];
        return this.getSelectExpressionsArray().filter((_selectExpression, i) => {
          return this.isGroupedSelectIndex(i);
        });
      }
      getGroupedOutputColumns() {
        if (!this.hasGroupBy())
          return [];
        return filterMap(this.getSelectExpressionsArray(), (selectExpression, i) => {
          if (!this.isGroupedSelectIndex(i))
            return;
          return _SqlQuery.getSelectExpressionOutput(selectExpression, i);
        });
      }
      isAggregateSelectIndex(selectIndex) {
        const { groupByClause } = this;
        if (!groupByClause || !this.isValidSelectIndex(selectIndex))
          return false;
        return !this.isGroupedSelectIndex(selectIndex);
      }
      isAggregateOutputColumn(outputColumn) {
        return this.isAggregateSelectIndex(this.getSelectIndexForOutputColumn(outputColumn));
      }
      getAggregateSelectExpressions() {
        if (!this.groupByClause)
          return [];
        return this.getSelectExpressionsArray().filter((_selectExpression, i) => {
          return this.isAggregateSelectIndex(i);
        });
      }
      getAggregateOutputColumns() {
        if (!this.groupByClause)
          return [];
        return filterMap(this.getSelectExpressionsArray(), (selectExpression, i) => {
          if (!this.isAggregateSelectIndex(i))
            return;
          return _SqlQuery.getSelectExpressionOutput(selectExpression, i);
        });
      }
      decodeInsertIndex(insertIndex = "last") {
        const selectExpressionsArray = this.getSelectExpressionsArray();
        if (insertIndex === "last-grouping") {
          return selectExpressionsArray.reduce((maxSeenGrouping, _, i) => {
            return this.isGroupedSelectIndex(i) ? i + 1 : maxSeenGrouping;
          }, 0);
        } else if (insertIndex === "last") {
          return selectExpressionsArray.length;
        } else if (typeof insertIndex === "number") {
          const n = selectExpressionsArray.length;
          return clampIndex(normalizeIndex(insertIndex, n), 0, n);
        } else {
          throw new Error(`unsupported insert index (${insertIndex})`);
        }
      }
      addSelect(ex, options = {}) {
        var _a, _b, _c;
        const { insertIndex, addToGroupBy, addToOrderBy, orderByExpression, direction } = options;
        const idx = this.decodeInsertIndex(insertIndex);
        const selectExpressions = ((_a = this.selectExpressions) == null ? void 0 : _a.insert(idx, ex)) || SeparatedArray.fromSingleValue(ex);
        let groupByClause = (_b = this.groupByClause) == null ? void 0 : _b.shiftIndexes(idx);
        if (addToGroupBy) {
          const indexLiteral = SqlLiteral.index(idx);
          groupByClause = groupByClause ? groupByClause.addExpression(indexLiteral, addToGroupBy) : SqlGroupByClause.create([indexLiteral]);
        }
        let orderByClause = (_c = this.orderByClause) == null ? void 0 : _c.shiftIndexes(idx);
        if (addToOrderBy) {
          const indexOrderBy = orderByExpression ? orderByExpression.changeExpression(SqlLiteral.index(idx)) : SqlOrderByExpression.index(idx, direction);
          orderByClause = orderByClause ? orderByClause.addExpression(indexOrderBy, addToOrderBy) : SqlOrderByClause.create([indexOrderBy]);
        }
        return this.changeSelectExpressions(selectExpressions).changeGroupByClause(groupByClause).changeOrderByClause(orderByClause);
      }
      changeSelect(selectIndex, ex) {
        if (!this.selectExpressions)
          return this;
        return this.changeSelectExpressions(this.selectExpressions.change(selectIndex, ex));
      }
      removeSelectIndex(selectIndex) {
        if (!this.selectExpressions)
          return this;
        const selectExpression = this.getSelectExpressionForIndex(selectIndex);
        if (!selectExpression)
          return this;
        const newSelectExpressions = this.selectExpressions.remove(selectIndex);
        let ret = this;
        if (this.groupByClause) {
          ret = ret.changeGroupByClause(this.groupByClause.removeExpression(selectExpression, selectIndex));
        }
        if (this.orderByClause) {
          ret = ret.changeOrderByClause(this.orderByClause.removeExpression(selectExpression, selectIndex));
        }
        return ret.changeSelectExpressions(newSelectExpressions);
      }
      removeSelectIndexes(selectIndexes) {
        return this.applyForEach(selectIndexes.slice().sort((a, b) => b - a), (a, idx) => a.removeSelectIndex(idx));
      }
      removeOutputColumn(outputColumn) {
        return this.removeSelectIndex(this.getSelectIndexForOutputColumn(outputColumn));
      }
      hasFrom() {
        return Boolean(this.fromClause);
      }
      getFromExpressions() {
        if (!this.fromClause)
          return [];
        return this.fromClause.expressions.values;
      }
      getFirstFromExpression() {
        if (!this.fromClause)
          return;
        return this.fromClause.expressions.first();
      }
      getFirstTableName() {
        if (!this.fromClause)
          return;
        return this.fromClause.getFirstTableName();
      }
      getFirstSchema() {
        if (!this.fromClause)
          return;
        return this.fromClause.getFirstSchema();
      }
      hasJoin() {
        if (!this.fromClause)
          return false;
        return this.fromClause.hasJoin();
      }
      getJoins() {
        if (!this.fromClause)
          return [];
        return this.fromClause.getJoins();
      }
      addJoin(join) {
        if (!this.fromClause)
          return this;
        return this.changeFromClause(this.fromClause.addJoin(join));
      }
      addLeftJoin(table, onExpression) {
        return this.addJoin(SqlJoinPart.create("LEFT", table, onExpression));
      }
      addRightJoin(table, onExpression) {
        return this.addJoin(SqlJoinPart.create("RIGHT", table, onExpression));
      }
      addInnerJoin(table, onExpression) {
        return this.addJoin(SqlJoinPart.create("INNER", table, onExpression));
      }
      addFullJoin(table, onExpression) {
        return this.addJoin(SqlJoinPart.create("FULL", table, onExpression));
      }
      addCrossJoin(table) {
        return this.addJoin(SqlJoinPart.cross(table));
      }
      removeAllJoins() {
        if (!this.fromClause)
          return this;
        return this.changeFromClause(this.fromClause.changeJoinParts(void 0));
      }
      hasWhere() {
        return Boolean(this.whereClause);
      }
      getWhereExpression() {
        if (!this.whereClause)
          return;
        return this.whereClause.expression;
      }
      getEffectiveWhereExpression() {
        if (!this.whereClause)
          return SqlLiteral.TRUE;
        return this.whereClause.expression;
      }
      addWhere(ex) {
        return this.changeWhereExpression(SqlExpression.and(this.getWhereExpression(), ex));
      }
      removeColumnFromWhere(column) {
        if (!this.whereClause)
          return this;
        return this.changeWhereClause(this.whereClause.removeColumnFromAnd(column));
      }
      hasGroupBy() {
        return Boolean(this.groupByClause);
      }
      getGroupingExpressionInfos() {
        const { groupByClause } = this;
        if (!groupByClause)
          return [];
        return groupByClause.toArray().map((ex) => {
          const selectIndex = this.getSelectIndexForExpression(ex, true);
          if (selectIndex === -1) {
            return {
              expression: ex,
              selectIndex,
              orderByExpression: this.getOrderByForExpression(ex)
            };
          } else {
            const selectExpression = this.getSelectExpressionForIndex(selectIndex);
            return {
              expression: selectExpression.getUnderlyingExpression(),
              selectIndex,
              outputColumn: _SqlQuery.getSelectExpressionOutput(selectExpression, selectIndex),
              orderByExpression: this.getOrderByForSelectIndex(selectIndex)
            };
          }
        });
      }
      getGroupingExpressions() {
        return this.getGroupingExpressionInfos().map((info) => info.expression);
      }
      addGroupBy(ex) {
        return this.changeGroupByClause(this.groupByClause ? this.groupByClause.addExpression(ex) : SqlGroupByClause.create([ex]));
      }
      hasHaving() {
        return Boolean(this.havingClause);
      }
      getHavingExpression() {
        if (!this.havingClause)
          return;
        return this.havingClause.expression;
      }
      getEffectiveHavingExpression() {
        if (!this.havingClause)
          return SqlLiteral.TRUE;
        return this.havingClause.expression;
      }
      addHaving(ex) {
        return this.changeHavingExpression(SqlExpression.and(this.getHavingExpression(), ex));
      }
      removeFromHaving(column) {
        if (!this.havingClause)
          return this;
        return this.changeHavingClause(this.havingClause.removeColumnFromAnd(column));
      }
      hasOrderBy() {
        return Boolean(this.orderByClause);
      }
      getOrderByExpressions() {
        const { orderByClause } = this;
        if (!orderByClause)
          return [];
        return orderByClause.expressions.values;
      }
      getOrderByForSelectIndex(selectIndex) {
        if (!this.orderByClause || !this.isValidSelectIndex(selectIndex))
          return;
        return this.orderByClause.toArray().find((orderByExpression) => this.expressionRefersToSelectIndex(orderByExpression.expression, selectIndex, true));
      }
      getOrderByForExpression(ex) {
        if (!this.orderByClause)
          return;
        return this.orderByClause.toArray().find((orderByExpression) => {
          return orderByExpression.expression.equals(ex);
        });
      }
      getOrderByForOutputColumn(outputColumn) {
        return this.getOrderByForSelectIndex(this.getSelectIndexForOutputColumn(outputColumn));
      }
      getOrderedSelectExpressions() {
        if (!this.orderByClause)
          return [];
        return this.getSelectExpressionsArray().filter((_selectExpression, i) => {
          return this.getOrderByForSelectIndex(i);
        });
      }
      getOrderedOutputColumns() {
        if (!this.orderByClause)
          return [];
        return filterMap(this.getSelectExpressionsArray(), (selectExpression, i) => {
          if (!this.getOrderByForSelectIndex(i))
            return;
          return _SqlQuery.getSelectExpressionOutput(selectExpression, i);
        });
      }
      removeOrderByForSelectIndex(selectIndex) {
        if (!this.orderByClause || !this.isValidSelectIndex(selectIndex))
          return this;
        return this.changeOrderByExpressions(this.orderByClause.expressions.filter((orderByExpression) => this.expressionRefersToSelectIndex(orderByExpression.expression, selectIndex, true)));
      }
      removeOrderByForOutputColumn(outputColumn) {
        return this.removeOrderByForSelectIndex(this.getSelectIndexForOutputColumn(outputColumn));
      }
      addOrderBy(orderBy) {
        return this.changeOrderByClause(this.orderByClause ? this.orderByClause.addFirst(orderBy) : SqlOrderByClause.create(orderBy));
      }
      hasLimit() {
        return Boolean(this.limitClause);
      }
      combineWithLimitClause(otherLimitClause) {
        if (this.limitClause) {
          return this.changeLimitValue(Math.min(this.limitClause.getLimitValue(), otherLimitClause.getLimitValue()));
        } else {
          return this.changeLimitClause(otherLimitClause);
        }
      }
      hasOffset() {
        return Boolean(this.offsetClause);
      }
      combineWithOffsetClause(otherOffsetClause) {
        if (this.offsetClause) {
          return this.changeOffsetValue(this.offsetClause.getOffsetValue() + otherOffsetClause.getOffsetValue());
        } else {
          return this.changeOffsetClause(otherOffsetClause);
        }
      }
    };
    SqlQuery = _SqlQuery;
    SqlQuery.type = "query";
    SqlQuery.DEFAULT_EXPLAIN_PLAN_FOR_KEYWORD = "EXPLAIN PLAN FOR";
    SqlQuery.DEFAULT_SELECT_KEYWORD = "SELECT";
    SqlQuery.DEFAULT_UNION_KEYWORD = "UNION ALL";
    SqlBase.register(SqlQuery);
  }
});

// src/sql/sql-with-query/sql-with-query.ts
var _SqlWithQuery, SqlWithQuery;
var init_sql_with_query = __esm({
  "src/sql/sql-with-query/sql-with-query.ts"() {
    "use strict";
    init_utils();
    init_sql_base();
    init_sql_clause2();
    init_sql_expression();
    init_sql_query();
    _SqlWithQuery = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlWithQuery.type);
        this.explain = options.explain;
        this.insertClause = options.insertClause;
        this.replaceClause = options.replaceClause;
        if (this.insertClause && this.replaceClause) {
          throw new Error("SqlWithQuery can not have both an insertClause and a replaceClause");
        }
        this.withClause = options.withClause;
        this.query = options.query;
        this.orderByClause = options.orderByClause;
        this.limitClause = options.limitClause;
        this.offsetClause = options.offsetClause;
        this.partitionedByClause = options.partitionedByClause;
        this.clusteredByClause = options.clusteredByClause;
      }
      valueOf() {
        const value = super.valueOf();
        value.explain = this.explain;
        value.insertClause = this.insertClause;
        value.replaceClause = this.replaceClause;
        value.withClause = this.withClause;
        value.query = this.query;
        value.orderByClause = this.orderByClause;
        value.limitClause = this.limitClause;
        value.offsetClause = this.offsetClause;
        value.partitionedByClause = this.partitionedByClause;
        value.clusteredByClause = this.clusteredByClause;
        return value;
      }
      _toRawString() {
        const {
          explain,
          insertClause,
          replaceClause,
          withClause,
          query,
          orderByClause,
          limitClause,
          offsetClause,
          partitionedByClause,
          clusteredByClause
        } = this;
        const rawParts = [];
        if (explain) {
          rawParts.push(this.getKeyword("explainPlanFor", SqlQuery.DEFAULT_EXPLAIN_PLAN_FOR_KEYWORD), this.getSpace("postExplainPlanFor", "\n"));
        }
        if (insertClause) {
          rawParts.push(insertClause.toString(), this.getSpace("postInsertClause", "\n"));
        } else if (replaceClause) {
          rawParts.push(replaceClause.toString(), this.getSpace("postReplaceClause", "\n"));
        }
        rawParts.push(withClause.toString(), this.getSpace("postWithClause", "\n"));
        rawParts.push(query.toString());
        if (orderByClause) {
          rawParts.push(this.getSpace("preOrderByClause", "\n"), orderByClause.toString());
        }
        if (limitClause) {
          rawParts.push(this.getSpace("preLimitClause", "\n"), limitClause.toString());
        }
        if (offsetClause) {
          rawParts.push(this.getSpace("preOffsetClause", "\n"), offsetClause.toString());
        }
        if (partitionedByClause) {
          rawParts.push(this.getSpace("prePartitionedByClause", "\n"), partitionedByClause.toString());
        }
        if (clusteredByClause) {
          rawParts.push(this.getSpace("preClusteredByClause", "\n"), clusteredByClause.toString());
        }
        return rawParts.join("");
      }
      changeExplain(explain) {
        if (this.explain === explain)
          return this;
        const value = this.valueOf();
        if (explain) {
          value.explain = true;
        } else {
          delete value.explain;
          value.spacing = this.getSpacingWithout("postExplainPlanFor");
        }
        return SqlBase.fromValue(value);
      }
      changeInsertClause(insertClause) {
        if (this.insertClause === insertClause)
          return this;
        const value = this.valueOf();
        if (insertClause) {
          value.insertClause = insertClause;
        } else {
          delete value.insertClause;
          value.spacing = this.getSpacingWithout("postInsertClause");
        }
        return SqlBase.fromValue(value);
      }
      changeReplaceClause(replaceClause) {
        if (this.replaceClause === replaceClause)
          return this;
        const value = this.valueOf();
        if (replaceClause) {
          value.replaceClause = replaceClause;
        } else {
          delete value.replaceClause;
          value.spacing = this.getSpacingWithout("postReplaceClause");
        }
        return SqlBase.fromValue(value);
      }
      changeWithClause(withClause) {
        if (this.withClause === withClause)
          return this;
        const value = this.valueOf();
        value.withClause = withClause;
        return SqlBase.fromValue(value);
      }
      changeQuery(query) {
        if (this.query === query)
          return this;
        const value = this.valueOf();
        value.query = query;
        return SqlBase.fromValue(value);
      }
      changeOrderByClause(orderByClause) {
        if (this.orderByClause === orderByClause)
          return this;
        const value = this.valueOf();
        if (orderByClause) {
          value.orderByClause = orderByClause;
        } else {
          delete value.orderByClause;
          value.spacing = this.getSpacingWithout("preOrderByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOrderByExpressions(orderByExpressions) {
        if (!orderByExpressions)
          return this.changeOrderByClause(void 0);
        return this.changeOrderByClause(this.orderByClause ? this.orderByClause.changeExpressions(orderByExpressions) : SqlOrderByClause.create(orderByExpressions));
      }
      changeOrderByExpression(orderByExpression) {
        if (!orderByExpression)
          return this.changeOrderByClause(void 0);
        return this.changeOrderByExpressions([orderByExpression]);
      }
      changeLimitClause(limitClause) {
        if (this.limitClause === limitClause)
          return this;
        const value = this.valueOf();
        if (limitClause) {
          value.limitClause = limitClause;
        } else {
          delete value.limitClause;
          value.spacing = this.getSpacingWithout("preLimitClause");
        }
        return SqlBase.fromValue(value);
      }
      changeLimitValue(limitValue) {
        if (typeof limitValue === "undefined")
          return this.changeLimitClause(void 0);
        return this.changeLimitClause(this.limitClause ? this.limitClause.changeLimit(limitValue) : SqlLimitClause.create(limitValue));
      }
      changeOffsetClause(offsetClause) {
        if (this.offsetClause === offsetClause)
          return this;
        const value = this.valueOf();
        if (offsetClause) {
          value.offsetClause = offsetClause;
        } else {
          delete value.offsetClause;
          value.spacing = this.getSpacingWithout("preOffsetClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOffsetValue(offsetValue) {
        if (typeof offsetValue === "undefined")
          return this.changeOffsetClause(void 0);
        return this.changeOffsetClause(this.offsetClause ? this.offsetClause.changeOffset(offsetValue) : SqlOffsetClause.create(offsetValue));
      }
      changePartitionedByClause(partitionedByClause) {
        if (this.partitionedByClause === partitionedByClause)
          return this;
        const value = this.valueOf();
        if (partitionedByClause) {
          value.partitionedByClause = partitionedByClause;
        } else {
          delete value.partitionedByClause;
          value.spacing = this.getSpacingWithout("prePartitionedByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeClusteredByClause(clusteredByClause) {
        if (this.clusteredByClause === clusteredByClause)
          return this;
        const value = this.valueOf();
        if (clusteredByClause) {
          value.clusteredByClause = clusteredByClause;
        } else {
          delete value.clusteredByClause;
          value.spacing = this.getSpacingWithout("preClusteredByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeClusteredByExpressions(clusteredByExpressions) {
        if (!clusteredByExpressions || isEmptyArray(clusteredByExpressions)) {
          return this.changeClusteredByClause(void 0);
        } else {
          return this.changeClusteredByClause(this.clusteredByClause ? this.clusteredByClause.changeExpressions(clusteredByExpressions) : SqlClusteredByClause.create(clusteredByExpressions));
        }
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.insertClause) {
          const insertClause = this.insertClause._walkHelper(nextStack, fn, postorder);
          if (!insertClause)
            return;
          if (insertClause !== this.insertClause) {
            ret = ret.changeInsertClause(insertClause);
          }
        } else if (this.replaceClause) {
          const replaceClause = this.replaceClause._walkHelper(nextStack, fn, postorder);
          if (!replaceClause)
            return;
          if (replaceClause !== this.replaceClause) {
            ret = ret.changeReplaceClause(replaceClause);
          }
        }
        const withClause = this.withClause._walkHelper(nextStack, fn, postorder);
        if (!withClause)
          return;
        if (withClause !== this.withClause) {
          ret = ret.changeWithClause(withClause);
        }
        if (this.orderByClause) {
          const orderByClause = this.orderByClause._walkHelper(nextStack, fn, postorder);
          if (!orderByClause)
            return;
          if (orderByClause !== this.orderByClause) {
            ret = ret.changeOrderByClause(orderByClause);
          }
        }
        if (this.limitClause) {
          const limitClause = this.limitClause._walkHelper(nextStack, fn, postorder);
          if (!limitClause)
            return;
          if (limitClause !== this.limitClause) {
            ret = ret.changeLimitClause(limitClause);
          }
        }
        if (this.offsetClause) {
          const offsetClause = this.offsetClause._walkHelper(nextStack, fn, postorder);
          if (!offsetClause)
            return;
          if (offsetClause !== this.offsetClause) {
            ret = ret.changeOffsetClause(offsetClause);
          }
        }
        if (this.partitionedByClause) {
          const partitionedByClause = this.partitionedByClause._walkHelper(nextStack, fn, postorder);
          if (!partitionedByClause)
            return;
          if (partitionedByClause !== this.partitionedByClause) {
            ret = ret.changePartitionedByClause(partitionedByClause);
          }
        }
        if (this.clusteredByClause) {
          const clusteredByClause = this.clusteredByClause._walkHelper(nextStack, fn, postorder);
          if (!clusteredByClause)
            return;
          if (clusteredByClause !== this.clusteredByClause) {
            ret = ret.changeClusteredByClause(clusteredByClause);
          }
        }
        return ret;
      }
      makeExplain() {
        return this.changeExplain(true);
      }
      getInsertIntoTable() {
        var _a;
        return (_a = this.insertClause) == null ? void 0 : _a.table;
      }
      changeInsertIntoTable(table) {
        const value = this.valueOf();
        value.insertClause = table ? value.insertClause ? value.insertClause.changeTable(table) : SqlInsertClause.create(table) : void 0;
        return SqlBase.fromValue(value);
      }
      getReplaceIntoTable() {
        var _a;
        return (_a = this.replaceClause) == null ? void 0 : _a.table;
      }
      changeReplaceIntoTable(table) {
        return this.changeReplaceClause(table ? this.replaceClause ? this.replaceClause.changeTable(table) : SqlReplaceClause.create(table) : void 0);
      }
      getIngestTable() {
        return this.getInsertIntoTable() || this.getReplaceIntoTable();
      }
      getWithParts() {
        const { withClause } = this;
        return withClause.withParts.values;
      }
      changeWithParts(withParts) {
        return this.changeWithClause(this.withClause ? this.withClause.changeWithParts(withParts) : SqlWithClause.create(withParts));
      }
      prependWith(name, query) {
        return this.changeWithParts([SqlWithPart.simple(name, query.ensureParens())].concat(this.getWithParts()));
      }
      flattenWith() {
        let flatQuery = this.query.flattenWith().changeParens([]).changeSpaces({ initial: this.spacing["initial"], final: this.spacing["final"] });
        flatQuery = flatQuery.changeWithParts(this.getWithParts().concat(flatQuery.getWithParts()));
        if (this.insertClause) {
          flatQuery = flatQuery.changeInsertClause(this.insertClause);
        } else if (this.replaceClause) {
          flatQuery = flatQuery.changeReplaceClause(this.replaceClause);
        }
        if (this.orderByClause) {
          flatQuery = flatQuery.changeOrderByClause(this.orderByClause);
        }
        if (this.limitClause) {
          flatQuery = flatQuery.combineWithLimitClause(this.limitClause);
        }
        if (this.offsetClause) {
          flatQuery = flatQuery.combineWithOffsetClause(this.offsetClause);
        }
        if (this.partitionedByClause) {
          flatQuery = flatQuery.changePartitionedByClause(this.partitionedByClause);
        }
        if (this.clusteredByClause) {
          flatQuery = flatQuery.changeClusteredByClause(this.clusteredByClause);
        }
        return flatQuery;
      }
      hasOrderBy() {
        return Boolean(this.orderByClause);
      }
      getOrderByExpressions() {
        const { orderByClause } = this;
        if (!orderByClause)
          return [];
        return orderByClause.expressions.values;
      }
      getOrderByForExpression(ex) {
        if (!this.orderByClause)
          return;
        return this.orderByClause.toArray().find((orderByExpression) => {
          return orderByExpression.expression.equals(ex);
        });
      }
      addOrderBy(orderBy) {
        return this.changeOrderByClause(this.orderByClause ? this.orderByClause.addFirst(orderBy) : SqlOrderByClause.create(orderBy));
      }
      hasLimit() {
        return Boolean(this.limitClause);
      }
      hasOffset() {
        return Boolean(this.offsetClause);
      }
    };
    SqlWithQuery = _SqlWithQuery;
    SqlWithQuery.type = "withQuery";
    SqlBase.register(SqlWithQuery);
  }
});

// src/sql/sql-clause/sql-from-clause/sql-from-clause.ts
var _SqlFromClause, SqlFromClause;
var init_sql_from_clause = __esm({
  "src/sql/sql-clause/sql-from-clause/sql-from-clause.ts"() {
    "use strict";
    init_utils();
    init_sql_base();
    init_sql_query();
    init_sql_table();
    init_sql_with_query();
    init_utils2();
    init_sql_clause();
    _SqlFromClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlFromClause.type);
        this.expressions = options.expressions;
        this.joinParts = options.joinParts;
      }
      static create(expressions) {
        return new _SqlFromClause({
          expressions: SeparatedArray.fromArray(expressions).map((ex) => ex instanceof SqlQuery || ex instanceof SqlWithQuery ? ex.ensureParens() : ex)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expressions = this.expressions;
        value.joinParts = this.joinParts;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("from", _SqlFromClause.DEFAULT_FROM_KEYWORD),
          this.getSpace("postFrom")
        ];
        rawParts.push(this.expressions.toString());
        if (this.joinParts) {
          rawParts.push(this.getSpace("preJoin", "\n"), this.joinParts.toString("\n"));
        }
        return rawParts.join("");
      }
      changeExpressions(expressions) {
        const value = this.valueOf();
        value.expressions = SeparatedArray.fromArray(expressions);
        return SqlBase.fromValue(value);
      }
      changeJoinParts(joinParts) {
        const value = this.valueOf();
        if (joinParts) {
          value.joinParts = SeparatedArray.fromArray(joinParts);
        } else {
          value.spacing = this.getSpacingWithout("preJoin");
          delete value.joinParts;
        }
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expressions = SqlBase.walkSeparatedArray(this.expressions, nextStack, fn, postorder);
        if (!expressions)
          return;
        if (expressions !== this.expressions) {
          ret = ret.changeExpressions(expressions);
        }
        if (this.joinParts) {
          const joinParts = SqlBase.walkSeparatedArray(this.joinParts, nextStack, fn, postorder);
          if (!joinParts)
            return;
          if (joinParts !== this.joinParts) {
            ret = ret.changeJoinParts(joinParts);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.expressions = this.expressions.clearSeparators();
        if (this.joinParts) {
          value.joinParts = this.joinParts.clearSeparators();
        }
        return SqlBase.fromValue(value);
      }
      getFirstTableName() {
        return filterMap(this.expressions.values, (table) => {
          table = table.getUnderlyingExpression();
          if (table instanceof SqlTable) {
            return table.getName();
          }
          return;
        })[0];
      }
      getFirstSchema() {
        return filterMap(this.expressions.values, (table) => {
          table = table.getUnderlyingExpression();
          if (table instanceof SqlTable) {
            return table.getNamespaceName();
          }
          return;
        })[0];
      }
      hasJoin() {
        return Boolean(this.joinParts);
      }
      getJoins() {
        if (!this.joinParts)
          return [];
        return this.joinParts.values;
      }
      addJoin(join) {
        return this.changeJoinParts(this.joinParts ? this.joinParts.append(join) : SeparatedArray.fromSingleValue(join));
      }
      removeAllJoins() {
        return this.changeJoinParts(void 0);
      }
    };
    SqlFromClause = _SqlFromClause;
    SqlFromClause.type = "fromClause";
    SqlFromClause.DEFAULT_FROM_KEYWORD = "FROM";
    SqlBase.register(SqlFromClause);
  }
});

// src/sql/sql-clause/sql-group-by-clause/sql-group-by-clause.ts
var _SqlGroupByClause, SqlGroupByClause;
var init_sql_group_by_clause = __esm({
  "src/sql/sql-clause/sql-group-by-clause/sql-group-by-clause.ts"() {
    "use strict";
    init_utils();
    init_sql_base();
    init_sql_literal();
    init_utils2();
    init_sql_clause();
    _SqlGroupByClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlGroupByClause.type);
        this.decorator = options.decorator;
        this.innerParens = Boolean(options.innerParens);
        this.expressions = options.expressions;
      }
      static create(expressions) {
        return new _SqlGroupByClause({
          expressions: !expressions || isEmptyArray(expressions) ? void 0 : SeparatedArray.fromArray(expressions)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.decorator = this.decorator;
        if (this.innerParens)
          value.innerParens = true;
        value.expressions = this.expressions;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("groupBy", _SqlGroupByClause.DEFAULT_GROUP_BY_KEYWORD),
          this.getSpace("postGroupBy")
        ];
        if (this.decorator) {
          rawParts.push(this.getKeyword("decorator", this.decorator), this.getSpace("postDecorator"));
        }
        const effectiveInnerParens = Boolean(this.innerParens || !this.expressions);
        if (effectiveInnerParens) {
          rawParts.push("(", this.getSpace("postLeftParen", ""));
        }
        if (this.expressions) {
          rawParts.push(this.expressions.toString(), this.getSpace("postExpressions", ""));
        }
        if (effectiveInnerParens) {
          rawParts.push(")");
        }
        return rawParts.join("");
      }
      changeExpressions(expressions) {
        const value = this.valueOf();
        if (!expressions || isEmptyArray(expressions)) {
          delete value.expressions;
        } else {
          value.expressions = SeparatedArray.fromArray(expressions);
        }
        return SqlBase.fromValue(value);
      }
      addExpression(expression, where = "end") {
        const { expressions } = this;
        return this.changeExpressions(expressions ? expressions.insert(where === "start" ? 0 : Infinity, expression) : SeparatedArray.fromSingleValue(expression));
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.expressions) {
          const expressions = SqlBase.walkSeparatedArray(this.expressions, nextStack, fn, postorder);
          if (!expressions)
            return;
          if (expressions !== this.expressions) {
            ret = ret.changeExpressions(expressions);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        if (!this.expressions)
          return this;
        const value = this.valueOf();
        value.expressions = this.expressions.clearSeparators();
        return SqlBase.fromValue(value);
      }
      toArray() {
        return this.expressions ? this.expressions.values : [];
      }
      removeExpression(selectExpression, selectIndex) {
        if (!this.expressions)
          return this;
        return this.changeExpressions(this.expressions.filterMap((expression) => {
          if (expression instanceof SqlLiteral && expression.isIndex()) {
            const expressionIndex = expression.getIndexValue();
            if (selectIndex < expressionIndex) {
              return expression.incrementIndex(-1);
            } else if (expressionIndex === selectIndex) {
              return;
            }
          }
          if (expression.equals(selectExpression.getUnderlyingExpression()))
            return;
          return expression;
        }));
      }
      shiftIndexes(aboveIndex) {
        if (!this.expressions)
          return this;
        return this.changeExpressions(this.expressions.map((expression) => {
          if (expression instanceof SqlLiteral && expression.isIndex()) {
            const expressionIndex = expression.getIndexValue();
            if (aboveIndex <= expressionIndex) {
              return expression.incrementIndex();
            }
          }
          return expression;
        }));
      }
    };
    SqlGroupByClause = _SqlGroupByClause;
    SqlGroupByClause.type = "groupByClause";
    SqlGroupByClause.DEFAULT_GROUP_BY_KEYWORD = "GROUP BY";
    SqlBase.register(SqlGroupByClause);
  }
});

// src/sql/sql-clause/sql-having-clause/sql-having-clause.ts
var _SqlHavingClause, SqlHavingClause;
var init_sql_having_clause = __esm({
  "src/sql/sql-clause/sql-having-clause/sql-having-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_clause();
    _SqlHavingClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlHavingClause.type);
        this.expression = options.expression;
      }
      static create(expression) {
        return new _SqlHavingClause({
          expression
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expression = this.expression;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("having", _SqlHavingClause.DEFAULT_HAVING_KEYWORD),
          this.getSpace("postHaving"),
          this.expression.toString()
        ].join("");
      }
      changeExpression(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expression = this.expression._walkHelper(nextStack, fn, postorder);
        if (!expression)
          return;
        if (expression !== this.expression) {
          ret = ret.changeExpression(expression);
        }
        return ret;
      }
      removeColumnFromAnd(column) {
        const newExpression = this.expression.removeColumnFromAnd(column);
        if (!newExpression)
          return;
        return this.changeExpression(newExpression);
      }
    };
    SqlHavingClause = _SqlHavingClause;
    SqlHavingClause.type = "havingClause";
    SqlHavingClause.DEFAULT_HAVING_KEYWORD = "HAVING";
    SqlBase.register(SqlHavingClause);
  }
});

// src/sql/sql-clause/sql-limit-clause/sql-limit-clause.ts
var _SqlLimitClause, SqlLimitClause;
var init_sql_limit_clause = __esm({
  "src/sql/sql-clause/sql-limit-clause/sql-limit-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    init_sql_clause();
    _SqlLimitClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlLimitClause.type);
        this.limit = options.limit;
      }
      static create(limit) {
        return new _SqlLimitClause({
          limit: SqlLiteral.create(limit)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.limit = this.limit;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("limit", _SqlLimitClause.DEFAULT_LIMIT_KEYWORD),
          this.getSpace("postLimit"),
          this.limit.toString()
        ].join("");
      }
      changeLimit(limit) {
        const value = this.valueOf();
        value.limit = SqlLiteral.create(limit);
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const limit = this.limit._walkHelper(nextStack, fn, postorder);
        if (!limit)
          return;
        if (limit !== this.limit) {
          ret = ret.changeLimit(limit);
        }
        return ret;
      }
      getLimitValue() {
        return this.limit.getNumberValue();
      }
    };
    SqlLimitClause = _SqlLimitClause;
    SqlLimitClause.type = "limitClause";
    SqlLimitClause.DEFAULT_LIMIT_KEYWORD = "LIMIT";
    SqlBase.register(SqlLimitClause);
  }
});

// src/sql/sql-clause/sql-offset-clause/sql-offset-clause.ts
var _SqlOffsetClause, SqlOffsetClause;
var init_sql_offset_clause = __esm({
  "src/sql/sql-clause/sql-offset-clause/sql-offset-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    init_sql_clause();
    _SqlOffsetClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlOffsetClause.type);
        this.offset = options.offset;
      }
      static create(offset) {
        return new _SqlOffsetClause({
          offset: SqlLiteral.create(offset)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.offset = this.offset;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("offset", _SqlOffsetClause.DEFAULT_OFFSET_KEYWORD),
          this.getSpace("postOffset"),
          this.offset.toString()
        ].join("");
      }
      changeOffset(offset) {
        const value = this.valueOf();
        value.offset = SqlLiteral.create(offset);
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const offset = this.offset._walkHelper(nextStack, fn, postorder);
        if (!offset)
          return;
        if (offset !== this.offset) {
          ret = ret.changeOffset(offset);
        }
        return ret;
      }
      getOffsetValue() {
        return this.offset.getNumberValue();
      }
    };
    SqlOffsetClause = _SqlOffsetClause;
    SqlOffsetClause.type = "offsetClause";
    SqlOffsetClause.DEFAULT_OFFSET_KEYWORD = "OFFSET";
    SqlBase.register(SqlOffsetClause);
  }
});

// src/sql/sql-clause/sql-partitioned-by-clause/sql-partitioned-by-clause.ts
var _SqlPartitionedByClause, SqlPartitionedByClause3;
var init_sql_partitioned_by_clause = __esm({
  "src/sql/sql-clause/sql-partitioned-by-clause/sql-partitioned-by-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_clause();
    _SqlPartitionedByClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlPartitionedByClause.type);
        this.expression = options.expression;
      }
      static create(unit) {
        return new _SqlPartitionedByClause({ expression: unit });
      }
      valueOf() {
        const value = super.valueOf();
        value.expression = this.expression;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("partitionedBy", _SqlPartitionedByClause.DEFAULT_PARTITIONED_BY_KEYWORD),
          this.getSpace("postPartitionedBy")
        ];
        if (this.expression) {
          rawParts.push(this.expression.toString());
        } else {
          rawParts.push(this.getKeyword("all", _SqlPartitionedByClause.DEFAULT_ALL_KEYWORD));
        }
        return rawParts.join("");
      }
      changePartitionedBy(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.expression) {
          const expression = this.expression._walkHelper(nextStack, fn, postorder);
          if (!expression)
            return;
          if (expression !== this.expression) {
            ret = ret.changePartitionedBy(expression);
          }
        }
        return ret;
      }
    };
    SqlPartitionedByClause3 = _SqlPartitionedByClause;
    SqlPartitionedByClause3.type = "partitionedByClause";
    SqlPartitionedByClause3.DEFAULT_PARTITIONED_BY_KEYWORD = "PARTITIONED BY";
    SqlPartitionedByClause3.DEFAULT_ALL_KEYWORD = "ALL";
    SqlBase.register(SqlPartitionedByClause3);
  }
});

// src/sql/sql-clause/sql-clustered-by-clause/sql-clustered-by-clause.ts
var _SqlClusteredByClause, SqlClusteredByClause;
var init_sql_clustered_by_clause = __esm({
  "src/sql/sql-clause/sql-clustered-by-clause/sql-clustered-by-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    init_utils2();
    init_sql_clause();
    _SqlClusteredByClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlClusteredByClause.type);
        this.expressions = options.expressions;
      }
      static create(expressions) {
        return new _SqlClusteredByClause({
          expressions: SeparatedArray.fromArray(expressions)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expressions = this.expressions;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("clusteredBy", _SqlClusteredByClause.DEFAULT_CLUSTERED_BY_KEYWORD),
          this.getSpace("postClusteredBy"),
          this.expressions.toString()
        ].join("");
      }
      changeExpressions(expressions) {
        const value = this.valueOf();
        value.expressions = SeparatedArray.fromArray(expressions);
        return SqlBase.fromValue(value);
      }
      addExpression(expression, where = "end") {
        const { expressions } = this;
        return this.changeExpressions(expressions ? expressions.insert(where === "start" ? 0 : Infinity, expression) : SeparatedArray.fromSingleValue(expression));
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expressions = SqlBase.walkSeparatedArray(this.expressions, nextStack, fn, postorder);
        if (!expressions)
          return;
        if (expressions !== this.expressions) {
          ret = ret.changeExpressions(expressions);
        }
        return ret;
      }
      clearOwnSeparators() {
        if (!this.expressions)
          return this;
        const value = this.valueOf();
        value.expressions = this.expressions.clearSeparators();
        return SqlBase.fromValue(value);
      }
      toArray() {
        return this.expressions.values;
      }
      shiftIndexes(aboveIndex) {
        if (!this.expressions)
          return this;
        return this.changeExpressions(this.expressions.map((expression) => {
          if (expression instanceof SqlLiteral && expression.isIndex()) {
            const expressionIndex = expression.getIndexValue();
            if (aboveIndex <= expressionIndex) {
              return expression.incrementIndex();
            }
          }
          return expression;
        }));
      }
    };
    SqlClusteredByClause = _SqlClusteredByClause;
    SqlClusteredByClause.type = "clusteredByClause";
    SqlClusteredByClause.DEFAULT_CLUSTERED_BY_KEYWORD = "CLUSTERED BY";
    SqlBase.register(SqlClusteredByClause);
  }
});

// src/sql/sql-clause/sql-with-clause/sql-with-part.ts
var _SqlWithPart, SqlWithPart;
var init_sql_with_part = __esm({
  "src/sql/sql-clause/sql-with-clause/sql-with-part.ts"() {
    "use strict";
    init_sql_base();
    init_sql_column_list();
    init_utils2();
    _SqlWithPart = class extends SqlBase {
      constructor(options) {
        super(options, _SqlWithPart.type);
        this.table = options.table;
        this.columns = options.columns;
        this.query = options.query;
      }
      static simple(name, query) {
        return new _SqlWithPart({
          table: RefName.create(name),
          query: query.ensureParens()
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.table = this.table;
        value.columns = this.columns;
        value.query = this.query;
        return value;
      }
      _toRawString() {
        const rawParts = [this.table.toString(), this.getSpace("postTable")];
        if (this.columns) {
          rawParts.push(this.columns.toString(), this.getSpace("postColumns"));
        }
        rawParts.push(this.getKeyword("as", _SqlWithPart.DEFAULT_AS_KEYWORD), this.getSpace("postAs"), this.query.toString());
        return rawParts.join("");
      }
      changeTable(table) {
        const value = this.valueOf();
        value.table = table;
        return SqlBase.fromValue(value);
      }
      changeColumns(columns) {
        const value = this.valueOf();
        if (!columns) {
          value.spacing = this.getSpacingWithout("postColumns");
          delete value.columns;
        } else if (columns instanceof SqlColumnList) {
          value.columns = columns;
        } else {
          value.columns = this.columns ? this.columns.changeColumns(columns) : SqlColumnList.create(columns);
        }
        return SqlBase.fromValue(value);
      }
      changeQuery(query) {
        const value = this.valueOf();
        value.query = query;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const query = this.query._walkHelper(nextStack, fn, postorder);
        if (!query)
          return;
        if (query !== this.query) {
          ret = ret.changeQuery(query);
        }
        return ret;
      }
    };
    SqlWithPart = _SqlWithPart;
    SqlWithPart.type = "withPart";
    SqlWithPart.DEFAULT_AS_KEYWORD = "AS";
    SqlBase.register(SqlWithPart);
  }
});

// src/sql/sql-clause/sql-with-clause/sql-with-clause.ts
var _SqlWithClause, SqlWithClause;
var init_sql_with_clause = __esm({
  "src/sql/sql-clause/sql-with-clause/sql-with-clause.ts"() {
    "use strict";
    init_sql_base();
    init_utils2();
    init_sql_clause();
    _SqlWithClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlWithClause.type);
        this.withParts = options.withParts;
      }
      static create(withParts) {
        return new _SqlWithClause({
          withParts: SeparatedArray.fromArray(withParts)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.withParts = this.withParts;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("with", _SqlWithClause.DEFAULT_WITH_KEYWORD),
          this.getSpace("postWith")
        ];
        rawParts.push(this.withParts.toString(",\n"));
        return rawParts.join("");
      }
      changeWithParts(withParts) {
        const value = this.valueOf();
        value.withParts = SeparatedArray.fromArray(withParts);
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const withParts = SqlBase.walkSeparatedArray(this.withParts, nextStack, fn, postorder);
        if (!withParts)
          return;
        if (withParts !== this.withParts) {
          ret = ret.changeWithParts(withParts);
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.withParts = this.withParts.clearSeparators();
        return SqlBase.fromValue(value);
      }
    };
    SqlWithClause = _SqlWithClause;
    SqlWithClause.type = "withClause";
    SqlWithClause.DEFAULT_WITH_KEYWORD = "WITH";
    SqlBase.register(SqlWithClause);
  }
});

// src/sql/sql-clause/sql-insert-clause/sql-insert-clause.ts
var _SqlInsertClause, SqlInsertClause;
var init_sql_insert_clause = __esm({
  "src/sql/sql-clause/sql-insert-clause/sql-insert-clause.ts"() {
    "use strict";
    init_sql_base();
    init_sql_table();
    init_sql_clause();
    _SqlInsertClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlInsertClause.type);
        this.table = options.table;
        this.columns = options.columns;
      }
      static create(table) {
        if (table instanceof _SqlInsertClause)
          return table;
        if (typeof table === "string") {
          table = SqlTable.create(table);
        }
        return new _SqlInsertClause({
          table
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.table = this.table;
        value.columns = this.columns;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("insert", _SqlInsertClause.DEFAULT_INSERT_KEYWORD),
          this.getSpace("postInsert"),
          this.getKeyword("into", _SqlInsertClause.DEFAULT_INTO_KEYWORD),
          this.getSpace("postInto"),
          this.table.toString()
        ];
        if (this.columns) {
          rawParts.push(this.getSpace("preColumns"), this.columns.toString());
        }
        return rawParts.join("");
      }
      changeTable(table) {
        const value = this.valueOf();
        if (typeof table === "string") {
          table = SqlTable.create(table);
        }
        value.table = table;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const table = this.table._walkHelper(nextStack, fn, postorder);
        if (!table)
          return;
        if (!(table instanceof SqlTable)) {
          throw new Error("must return table");
        }
        if (table !== this.table) {
          ret = ret.changeTable(table);
        }
        return ret;
      }
    };
    SqlInsertClause = _SqlInsertClause;
    SqlInsertClause.type = "insertClause";
    SqlInsertClause.DEFAULT_INSERT_KEYWORD = "INSERT";
    SqlInsertClause.DEFAULT_INTO_KEYWORD = "INTO";
    SqlBase.register(SqlInsertClause);
  }
});

// src/sql/sql-clause/sql-extend-clause/sql-column-declaration.ts
var _SqlColumnDeclaration, SqlColumnDeclaration;
var init_sql_column_declaration = __esm({
  "src/sql/sql-clause/sql-extend-clause/sql-column-declaration.ts"() {
    "use strict";
    init_sql_base();
    init_sql_type();
    init_utils2();
    _SqlColumnDeclaration = class extends SqlBase {
      constructor(options) {
        super(options, _SqlColumnDeclaration.type);
        this.column = options.column;
        this.columnType = options.columnType;
      }
      static create(column, columnType) {
        return new _SqlColumnDeclaration({
          column: RefName.create(column),
          columnType: SqlType.create(columnType)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.column = this.column;
        value.columnType = this.columnType;
        return value;
      }
      _toRawString() {
        return [this.column.toString(), this.getSpace("postColumn"), this.columnType].join("");
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.columnType) {
          const columnType = this.columnType._walkHelper(nextStack, fn, postorder);
          if (!columnType)
            return;
          if (columnType !== this.columnType) {
            ret = ret.changeColumnType(columnType);
          }
        }
        return ret;
      }
      getColumnName() {
        return this.column.name;
      }
      changeColumn(column) {
        const value = this.valueOf();
        value.column = RefName.create(column);
        return SqlBase.fromValue(value);
      }
      changeColumnType(columnType) {
        const value = this.valueOf();
        value.columnType = SqlType.create(columnType);
        return SqlBase.fromValue(value);
      }
    };
    SqlColumnDeclaration = _SqlColumnDeclaration;
    SqlColumnDeclaration.type = "columnDeclaration";
    SqlBase.register(SqlColumnDeclaration);
  }
});

// src/sql/sql-clause/sql-extend-clause/sql-extend-clause.ts
var _SqlExtendClause, SqlExtendClause;
var init_sql_extend_clause = __esm({
  "src/sql/sql-clause/sql-extend-clause/sql-extend-clause.ts"() {
    "use strict";
    init_sql_base();
    init_utils2();
    init_sql_clause();
    _SqlExtendClause = class extends SqlClause {
      constructor(options) {
        super(options, _SqlExtendClause.type);
        this.columnDeclarations = options.columnDeclarations;
      }
      static create(columnDeclarations) {
        return new _SqlExtendClause({
          columnDeclarations: SeparatedArray.fromArray(columnDeclarations)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.columnDeclarations = this.columnDeclarations;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("extend", _SqlExtendClause.DEFAULT_EXTEND_KEYWORD, "postExtend"),
          "(",
          this.getSpace("postLeftParen"),
          this.columnDeclarations.toString(),
          this.getSpace("postColumnDeclarations"),
          ")"
        ].join("");
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const columnDeclarations = SqlBase.walkSeparatedArray(this.columnDeclarations, nextStack, fn, postorder);
        if (!columnDeclarations)
          return;
        if (columnDeclarations !== this.columnDeclarations) {
          ret = ret.changeColumnDeclarations(columnDeclarations);
        }
        return ret;
      }
      changeColumnDeclarations(columnDeclarations) {
        const value = this.valueOf();
        value.columnDeclarations = SeparatedArray.fromArray(columnDeclarations);
        return SqlBase.fromValue(value);
      }
    };
    SqlExtendClause = _SqlExtendClause;
    SqlExtendClause.type = "extendClause";
    SqlExtendClause.DEFAULT_EXTEND_KEYWORD = "EXTEND";
    SqlBase.register(SqlExtendClause);
  }
});

// src/sql/sql-clause/index.ts
var init_sql_clause2 = __esm({
  "src/sql/sql-clause/index.ts"() {
    "use strict";
    init_sql_clause();
    init_sql_order_by_expression();
    init_sql_order_by_clause();
    init_sql_join_part();
    init_sql_from_clause();
    init_sql_group_by_clause();
    init_sql_where_clause();
    init_sql_having_clause();
    init_sql_limit_clause();
    init_sql_offset_clause();
    init_sql_partitioned_by_clause();
    init_sql_clustered_by_clause();
    init_sql_with_part();
    init_sql_with_clause();
    init_sql_insert_clause();
    init_sql_replace_clause();
    init_sql_column_declaration();
    init_sql_extend_clause();
  }
});

// src/sql/sql-function/sql-function.ts
var specialFunctionLookup, _SqlFunction, SqlFunction;
var init_sql_function = __esm({
  "src/sql/sql-function/sql-function.ts"() {
    "use strict";
    init_utils();
    init_special_functions();
    init_sql_base();
    init_sql_clause2();
    init_sql_expression();
    init_sql_literal();
    init_sql_namespace();
    init_sql_star();
    init_sql_type();
    init_utils2();
    specialFunctionLookup = {};
    for (const r of SPECIAL_FUNCTIONS) {
      specialFunctionLookup[r] = true;
    }
    _SqlFunction = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlFunction.type);
        this.namespace = options.namespace;
        this.functionName = options.functionName;
        this.specialParen = options.specialParen;
        this.decorator = options.decorator;
        this.args = options.args;
        this.whereClause = options.whereClause;
        this.extendClause = options.extendClause;
        this.windowSpec = options.windowSpec;
      }
      static isValidFunctionName(functionName) {
        return Boolean(!RefName.isReservedFunctionName(functionName));
      }
      static isNakedFunction(functionName) {
        return Boolean(specialFunctionLookup[functionName.toUpperCase()]);
      }
      static simple(functionName, args, filter) {
        return new _SqlFunction({
          functionName: RefName.functionName(functionName),
          args: SeparatedArray.fromArray(Array.isArray(args) ? args.map(SqlExpression.wrap) : args, Separator.COMMA),
          whereClause: filter ? SqlWhereClause.createForFunction(filter) : void 0
        });
      }
      static decorated(functionName, decorator, args, filter) {
        return new _SqlFunction({
          functionName: RefName.functionName(functionName),
          decorator,
          args: SeparatedArray.fromArray(Array.isArray(args) ? args.map(SqlExpression.wrap) : args, Separator.COMMA),
          whereClause: filter ? SqlWhereClause.createForFunction(filter) : void 0
        });
      }
      static count(arg) {
        return _SqlFunction.simple("COUNT", [arg || SqlStar.PLAIN]);
      }
      static countDistinct(arg) {
        return _SqlFunction.decorated("COUNT", "DISTINCT", [arg]);
      }
      static sum(arg) {
        return _SqlFunction.simple("SUM", [arg]);
      }
      static min(arg) {
        return _SqlFunction.simple("MIN", [arg]);
      }
      static max(arg) {
        return _SqlFunction.simple("MAX", [arg]);
      }
      static avg(arg) {
        return _SqlFunction.simple("AVG", [arg]);
      }
      static cast(ex, asType) {
        return new _SqlFunction({
          functionName: RefName.functionName("CAST"),
          args: SeparatedArray.fromTwoValuesWithSeparator(ex, Separator.symmetricSpace("AS"), SqlType.create(asType))
        });
      }
      static floor(ex, timeUnit) {
        return new _SqlFunction({
          functionName: RefName.functionName("FLOOR"),
          args: SeparatedArray.fromTwoValuesWithSeparator(ex, Separator.symmetricSpace("TO"), SqlLiteral.direct(timeUnit))
        });
      }
      static timeFloor(timestampExpr, period, origin, timezone) {
        return _SqlFunction.simple("TIME_FLOOR", compact([timestampExpr, period, origin, timezone]));
      }
      static timeCeil(timestampExpr, period, origin, timezone) {
        return _SqlFunction.simple("TIME_CEIL", compact([timestampExpr, period, origin, timezone]));
      }
      static timeShift(timestampExpr, period, step, timezone) {
        return _SqlFunction.simple("TIME_SHIFT", compact([timestampExpr, period, step, timezone]));
      }
      static array(...exs) {
        if (exs.length === 1 && Array.isArray(exs[0]))
          exs = exs[0];
        return new _SqlFunction({
          functionName: RefName.functionName("ARRAY"),
          specialParen: "square",
          args: SeparatedArray.fromArray(exs.map(SqlExpression.wrap))
        });
      }
      static stringFormat(format, ...args) {
        return _SqlFunction.simple("STRING_FORMAT", [format].concat(args));
      }
      static regexpLike(ex, pattern) {
        return _SqlFunction.simple("REGEXP_LIKE", [ex, pattern]);
      }
      valueOf() {
        const value = super.valueOf();
        value.namespace = this.namespace;
        value.functionName = this.functionName;
        value.specialParen = this.specialParen;
        value.decorator = this.decorator;
        value.args = this.args;
        value.whereClause = this.whereClause;
        value.extendClause = this.extendClause;
        value.windowSpec = this.windowSpec;
        return value;
      }
      _toRawString() {
        const { specialParen } = this;
        const rawParts = [];
        if (this.namespace) {
          rawParts.push(this.namespace.toString(), this.getSpace("postNamespace", ""), ".", this.getSpace("postDot", ""));
        }
        rawParts.push(this.functionName.toString());
        if (specialParen !== "none") {
          rawParts.push(this.getSpace("preLeftParen", ""), specialParen === "square" ? "[" : "(", this.getSpace("postLeftParen", ""));
          if (this.decorator) {
            rawParts.push(this.getKeyword("decorator", this.decorator), this.getSpace("postDecorator"));
          }
          if (this.args) {
            rawParts.push(this.args.toString(Separator.COMMA), this.getSpace("postArguments", ""));
          }
          rawParts.push(specialParen === "square" ? "]" : ")");
          if (this.whereClause) {
            rawParts.push(this.getSpace("preFilter"), this.getKeyword("filter", _SqlFunction.DEFAULT_FILTER_KEYWORD), this.getSpace("postFilter"), this.whereClause.toString());
          } else if (this.extendClause) {
            rawParts.push(this.getSpace("preExtend"), this.extendClause.toString());
          }
          if (this.windowSpec) {
            rawParts.push(this.getSpace("preOver"), this.getKeyword("over", _SqlFunction.DEFAULT_OVER_KEYWORD), this.getSpace("postOver"), this.windowSpec.toString());
          }
        }
        return rawParts.join("");
      }
      changeNamespace(namespace) {
        const value = this.valueOf();
        if (namespace) {
          value.namespace = namespace;
        } else {
          delete value.namespace;
          value.spacing = this.getSpacingWithout("postNamespace", "postDot");
        }
        return SqlBase.fromValue(value);
      }
      getNamespaceName() {
        var _a;
        return (_a = this.namespace) == null ? void 0 : _a.getName();
      }
      changeNamespaceName(namespace) {
        return this.changeNamespace(namespace ? this.namespace ? this.namespace.changeName(namespace) : SqlNamespace.create(namespace) : void 0);
      }
      getEffectiveFunctionName() {
        return this.functionName.name.toUpperCase();
      }
      getEffectiveDecorator() {
        var _a;
        return (_a = this.decorator) == null ? void 0 : _a.toUpperCase();
      }
      changeArgs(args) {
        const value = this.valueOf();
        value.args = args;
        return SqlBase.fromValue(value);
      }
      numArgs() {
        var _a;
        return ((_a = this.args) == null ? void 0 : _a.length()) || 0;
      }
      getArgArray() {
        var _a;
        return ((_a = this.args) == null ? void 0 : _a.values) || [];
      }
      getArg(index) {
        var _a;
        return (_a = this.args) == null ? void 0 : _a.get(index);
      }
      getArgAsString(index) {
        const arg = this.getArg(index);
        if (!(arg instanceof SqlLiteral))
          return;
        return arg.getStringValue();
      }
      getArgAsNumber(index) {
        const arg = this.getArg(index);
        if (!(arg instanceof SqlLiteral))
          return;
        return arg.getNumberValue();
      }
      getArgAsNumberOrBigint(index) {
        const arg = this.getArg(index);
        if (!(arg instanceof SqlLiteral))
          return;
        return arg.getNumberOrBigintValue();
      }
      changeWhereClause(whereClause) {
        const value = this.valueOf();
        if (whereClause) {
          value.whereClause = whereClause;
        } else {
          delete value.whereClause;
          value.keywords = this.getKeywordsWithout("filter");
        }
        return SqlBase.fromValue(value);
      }
      changeWhereExpression(whereExpression) {
        if (!whereExpression)
          return this.changeWhereClause(void 0);
        return this.changeWhereClause(this.whereClause ? this.whereClause.changeExpression(whereExpression) : SqlWhereClause.createForFunction(whereExpression));
      }
      addWhereExpression(whereExpression) {
        const { whereClause } = this;
        return this.changeWhereClause(whereClause ? whereClause.changeExpression(whereClause.expression.and(whereExpression)) : SqlWhereClause.createForFunction(whereExpression));
      }
      getWhereExpression() {
        var _a;
        return (_a = this.whereClause) == null ? void 0 : _a.expression;
      }
      getEffectiveWhereExpression() {
        return this.getWhereExpression() || SqlLiteral.TRUE;
      }
      changeExtendClause(extendClause) {
        const value = this.valueOf();
        if (extendClause) {
          value.extendClause = extendClause;
        } else {
          delete value.extendClause;
          value.spacing = this.getSpacingWithout("preExtend");
        }
        return SqlBase.fromValue(value);
      }
      isAggregation(knownAggregations) {
        return Boolean(this.whereClause || knownAggregations.includes(this.getEffectiveFunctionName()));
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.args) {
          const args = SqlBase.walkSeparatedArray(this.args, nextStack, fn, postorder);
          if (!args)
            return;
          if (args !== this.args) {
            ret = ret.changeArgs(args);
          }
        }
        if (this.whereClause) {
          const whereClause = this.whereClause._walkHelper(nextStack, fn, postorder);
          if (!whereClause)
            return;
          if (whereClause !== this.whereClause) {
            ret = ret.changeWhereClause(whereClause);
          }
        }
        if (this.extendClause) {
          const extendClause = this.extendClause._walkHelper(nextStack, fn, postorder);
          if (!extendClause)
            return;
          if (extendClause !== this.extendClause) {
            ret = ret.changeExtendClause(extendClause);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        if (!this.args)
          return this;
        const value = this.valueOf();
        value.args = this.args.clearSeparatorsMatching(",");
        return SqlBase.fromValue(value);
      }
      resetOwnKeywords() {
        const upperCaseName = this.getEffectiveFunctionName();
        if (Object.keys(this.keywords).length === 0 && upperCaseName === this.functionName.name) {
          return this;
        }
        const value = this.valueOf();
        value.functionName = value.functionName.changeNameAsFunctionName(upperCaseName);
        value.keywords = {};
        return SqlBase.fromValue(value);
      }
      isCountStar() {
        if (this.getEffectiveFunctionName() !== "COUNT")
          return false;
        const args = this.args;
        if (!args || args.length() !== 1)
          return false;
        return args.first() instanceof SqlStar;
      }
      getCastType() {
        var _a;
        if (this.getEffectiveFunctionName() !== "CAST")
          return;
        const arg1 = (_a = this.args) == null ? void 0 : _a.get(1);
        return arg1 instanceof SqlType ? arg1 : void 0;
      }
      getColumnDeclarations() {
        var _a, _b;
        return (_b = (_a = this.extendClause) == null ? void 0 : _a.columnDeclarations) == null ? void 0 : _b.values;
      }
      changeColumnDeclarations(columnDeclarations) {
        if (!columnDeclarations)
          return this.changeExtendClause(void 0);
        return this.changeExtendClause(this.extendClause ? this.extendClause.changeColumnDeclarations(columnDeclarations) : SqlExtendClause.create(columnDeclarations));
      }
    };
    SqlFunction = _SqlFunction;
    SqlFunction.type = "function";
    SqlFunction.DEFAULT_FILTER_KEYWORD = "FILTER";
    SqlFunction.DEFAULT_OVER_KEYWORD = "OVER";
    SqlFunction.SPECIAL_FUNCTIONS = SPECIAL_FUNCTIONS;
    SqlBase.register(SqlFunction);
    SqlFunction.COUNT_STAR = SqlFunction.simple("COUNT", [SqlStar.PLAIN]);
  }
});

// src/sql/sql-namespace/sql-namespace.ts
var _SqlNamespace, SqlNamespace;
var init_sql_namespace = __esm({
  "src/sql/sql-namespace/sql-namespace.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_sql_function();
    init_sql_table();
    init_utils2();
    _SqlNamespace = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlNamespace.type);
        this.refName = options.refName;
      }
      static create(name) {
        if (name instanceof _SqlNamespace)
          return name;
        return new _SqlNamespace({
          refName: RefName.create(name)
        });
      }
      static optionalQuotes(name) {
        if (name instanceof _SqlNamespace)
          return name;
        return new _SqlNamespace({
          refName: RefName.create(name, false)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.refName = this.refName;
        return value;
      }
      _toRawString() {
        const { refName } = this;
        return refName.toString();
      }
      changeRefName(refName) {
        const value = this.valueOf();
        value.refName = refName;
        return SqlBase.fromValue(value);
      }
      getName() {
        return this.refName.name;
      }
      changeName(name) {
        const { refName } = this;
        const value = this.valueOf();
        value.refName = refName.changeName(name);
        return SqlBase.fromValue(value);
      }
      prettyTrim(maxLength) {
        return this.changeRefName(this.refName.prettyTrim(maxLength));
      }
      table(name) {
        return SqlTable.create(name, this);
      }
      tableWithOptionalQuotes(name) {
        return SqlTable.optionalQuotes(name, this);
      }
      F(name, ...args) {
        return SqlFunction.simple(name, args).changeNamespace(this);
      }
    };
    SqlNamespace = _SqlNamespace;
    SqlNamespace.type = "namespace";
    SqlBase.register(SqlNamespace);
  }
});

// src/sql/sql-multi/sql-multi.ts
var _SqlMulti, SqlMulti;
var init_sql_multi = __esm({
  "src/sql/sql-multi/sql-multi.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_utils2();
    _SqlMulti = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlMulti.type);
        this.op = options.op;
        if (!this.op)
          throw new Error(`must have op`);
        this.args = options.args;
        if (!this.args)
          throw new Error(`must have args`);
      }
      valueOf() {
        const value = super.valueOf();
        value.op = this.op;
        value.args = this.args;
        return value;
      }
      _toRawString() {
        return this.args.toString(Separator.symmetricSpace(SqlBase.capitalize(this.op)));
      }
      numArgs() {
        var _a;
        return ((_a = this.args) == null ? void 0 : _a.length()) || 0;
      }
      getArgArray() {
        var _a;
        return ((_a = this.args) == null ? void 0 : _a.values) || [];
      }
      getArg(index) {
        var _a;
        return (_a = this.args) == null ? void 0 : _a.get(index);
      }
      changeArgs(args) {
        const value = this.valueOf();
        value.args = args;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const args = SqlBase.walkSeparatedArray(this.args, nextStack, fn, postorder);
        if (!args)
          return;
        if (args !== this.args) {
          ret = ret.changeArgs(args);
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.args = this.args.clearSeparators();
        return SqlBase.fromValue(value);
      }
      and(expression) {
        if (this.op !== "AND")
          return super.and(expression);
        return this.changeArgs(this.args.append(expression));
      }
      decomposeViaAnd(options = {}) {
        if (this.op !== "AND")
          return super.decomposeViaAnd(options);
        if (options.flatten) {
          return this.args.values.flatMap((v) => v.decomposeViaAnd(options));
        } else {
          return this.args.values.map((v) => v.changeParens([]));
        }
      }
      or(expression) {
        if (this.op !== "OR")
          return super.or(expression);
        return this.changeArgs(this.args.append(expression));
      }
      filterAnd(fn) {
        if (this.op !== "AND") {
          return super.filterAnd(fn);
        }
        const args = this.args.filter(fn);
        if (!args)
          return;
        if (args.length() === 1) {
          return args.first();
        }
        return this.changeArgs(args);
      }
      decomposeViaOr(options = {}) {
        if (this.op !== "OR")
          return super.decomposeViaOr(options);
        if (options.flatten) {
          return this.args.values.flatMap((v) => v.decomposeViaOr(options));
        } else {
          return this.args.values.map((v) => v.changeParens([]));
        }
      }
    };
    SqlMulti = _SqlMulti;
    SqlMulti.type = "multi";
    SqlBase.register(SqlMulti);
  }
});

// src/sql/sql-unary/sql-unary.ts
var _SqlUnary, SqlUnary;
var init_sql_unary = __esm({
  "src/sql/sql-unary/sql-unary.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    _SqlUnary = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlUnary.type);
        this.op = options.op;
        this.argument = options.argument;
      }
      static not(ex) {
        return new _SqlUnary({
          op: "NOT",
          argument: ex.type === "multi" || ex.type === "comparison" ? ex.ensureParens() : ex
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.op = this.op;
        value.argument = this.argument;
        return value;
      }
      _toRawString() {
        return [this.getKeyword("op", this.op), this.getSpace("postOp"), this.argument.toString()].join("");
      }
      changeArgument(argument) {
        const value = this.valueOf();
        value.argument = argument;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const argument = this.argument._walkHelper(nextStack, fn, postorder);
        if (!argument)
          return;
        if (argument !== this.argument) {
          ret = ret.changeArgument(argument);
        }
        return ret;
      }
      negate() {
        return this.argument;
      }
    };
    SqlUnary = _SqlUnary;
    SqlUnary.type = "unary";
    SqlBase.register(SqlUnary);
  }
});

// src/sql/sql-comparison/sql-between-part.ts
var _SqlBetweenPart, SqlBetweenPart;
var init_sql_between_part = __esm({
  "src/sql/sql-comparison/sql-between-part.ts"() {
    "use strict";
    init_sql_base();
    _SqlBetweenPart = class extends SqlBase {
      constructor(options) {
        super(options, _SqlBetweenPart.type);
        this.symmetric = options.symmetric;
        this.start = options.start;
        this.end = options.end;
      }
      static create(start, end) {
        return new _SqlBetweenPart({
          start,
          end
        });
      }
      static symmetric(start, end) {
        return new _SqlBetweenPart({
          symmetric: true,
          start,
          end
        });
      }
      valueOf() {
        const value = super.valueOf();
        if (this.symmetric)
          value.symmetric = true;
        value.start = this.start;
        value.end = this.end;
        return value;
      }
      _toRawString() {
        const rawParts = [];
        if (this.symmetric) {
          rawParts.push(this.getKeyword("symmetric", _SqlBetweenPart.DEFAULT_SYMMETRIC_KEYWORD), this.getSpace("postSymmetric"));
        }
        rawParts.push(this.start.toString(), this.getSpace("preAnd"), this.getKeyword("and", _SqlBetweenPart.DEFAULT_AND_KEYWORD), this.getSpace("postAnd"), this.end.toString());
        return rawParts.join("");
      }
      changeStart(start) {
        const value = this.valueOf();
        value.start = start;
        return SqlBase.fromValue(value);
      }
      changeEnd(end) {
        const value = this.valueOf();
        value.end = end;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const start = ret.start._walkHelper(nextStack, fn, postorder);
        if (!start)
          return;
        if (start !== ret.start) {
          ret = ret.changeStart(start);
        }
        const end = ret.end._walkHelper(nextStack, fn, postorder);
        if (!end)
          return;
        if (end !== ret.end) {
          ret = ret.changeEnd(end);
        }
        return ret;
      }
    };
    SqlBetweenPart = _SqlBetweenPart;
    SqlBetweenPart.type = "betweenPart";
    SqlBetweenPart.DEFAULT_SYMMETRIC_KEYWORD = "SYMMETRIC";
    SqlBetweenPart.DEFAULT_AND_KEYWORD = "AND";
    SqlBase.register(SqlBetweenPart);
  }
});

// src/sql/sql-comparison/sql-like-part.ts
var _SqlLikePart, SqlLikePart;
var init_sql_like_part = __esm({
  "src/sql/sql-comparison/sql-like-part.ts"() {
    "use strict";
    init_sql_base();
    init_sql_literal();
    _SqlLikePart = class extends SqlBase {
      constructor(options) {
        super(options, _SqlLikePart.type);
        this.like = options.like;
        this.escape = options.escape;
      }
      static create(like, escape) {
        const likeEx = typeof like === "string" ? SqlLiteral.create(like) : like;
        const escapeEx = typeof escape === "string" ? SqlLiteral.create(escape) : escape;
        return new _SqlLikePart({
          like: likeEx,
          escape: escapeEx
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.like = this.like;
        value.escape = this.escape;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.like.toString(),
          this.getSpace("preEscape"),
          this.getKeyword("escape", _SqlLikePart.DEFAULT_ESCAPE_KEYWORD),
          this.getSpace("postEscape"),
          this.escape.toString()
        ];
        return rawParts.join("");
      }
      changeLike(like) {
        const value = this.valueOf();
        value.like = like;
        return SqlBase.fromValue(value);
      }
      changeEscape(escape) {
        const value = this.valueOf();
        value.escape = escape;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const like = ret.like._walkHelper(nextStack, fn, postorder);
        if (!like)
          return;
        if (like !== ret.like) {
          ret = ret.changeLike(like);
        }
        const escape = ret.escape._walkHelper(nextStack, fn, postorder);
        if (!escape)
          return;
        if (escape !== ret.escape) {
          ret = ret.changeEscape(escape);
        }
        return ret;
      }
    };
    SqlLikePart = _SqlLikePart;
    SqlLikePart.type = "likePart";
    SqlLikePart.DEFAULT_ESCAPE_KEYWORD = "ESCAPE";
    SqlBase.register(SqlLikePart);
  }
});

// src/sql/sql-record/sql-record.ts
var _SqlRecord, SqlRecord;
var init_sql_record = __esm({
  "src/sql/sql-record/sql-record.ts"() {
    "use strict";
    init_utils();
    init_sql_base();
    init_sql_expression();
    init_utils2();
    _SqlRecord = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlRecord.type);
        this.expressions = options.expressions;
      }
      static create(expressions) {
        if (expressions instanceof _SqlRecord)
          return expressions;
        return new _SqlRecord({
          expressions: !expressions || isEmptyArray(expressions) ? void 0 : SeparatedArray.fromArray(expressions, Separator.COMMA)
        });
      }
      static createWithoutKeyword(expressions) {
        if (expressions instanceof _SqlRecord)
          return expressions;
        return new _SqlRecord({
          keywords: {
            row: ""
          },
          expressions: !expressions || isEmptyArray(expressions) ? void 0 : SeparatedArray.fromArray(expressions, Separator.COMMA)
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expressions = this.expressions;
        return value;
      }
      _toRawString() {
        var _a;
        const rawParts = [
          this.getKeyword("row", ((_a = this.expressions) == null ? void 0 : _a.length()) === 1 ? _SqlRecord.DEFAULT_ROW_KEYWORD : "", "postRow", "")
        ];
        rawParts.push("(", this.getSpace("postLeftParen", ""));
        if (this.expressions) {
          rawParts.push(this.expressions.toString(Separator.COMMA), this.getSpace("postExpressions", ""));
        }
        rawParts.push(")");
        return rawParts.join("");
      }
      changeExpressions(expressions) {
        const value = this.valueOf();
        if (!expressions || isEmptyArray(expressions)) {
          delete value.expressions;
        } else {
          value.expressions = SeparatedArray.fromArray(expressions, Separator.COMMA);
        }
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.expressions) {
          const expressions = SqlBase.walkSeparatedArray(this.expressions, nextStack, fn, postorder);
          if (!expressions)
            return;
          if (expressions !== this.expressions) {
            ret = ret.changeExpressions(expressions);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        if (!this.expressions)
          return this;
        const value = this.valueOf();
        value.expressions = this.expressions.clearSeparators();
        return SqlBase.fromValue(value);
      }
      prepend(expression) {
        const { expressions } = this;
        const ex = SqlExpression.wrap(expression);
        return this.changeExpressions(expressions ? expressions.prepend(ex) : SeparatedArray.fromSingleValue(ex));
      }
      append(expression) {
        const { expressions } = this;
        const ex = SqlExpression.wrap(expression);
        return this.changeExpressions(expressions ? expressions.append(ex) : SeparatedArray.fromSingleValue(ex));
      }
      unwrapIfSingleton() {
        const { expressions } = this;
        if ((expressions == null ? void 0 : expressions.length()) !== 1 || this.keywords.row)
          return this;
        return expressions.get(0).addParens(this.getSpace("postLeftParen", ""), this.getSpace("postExpressions", ""));
      }
    };
    SqlRecord = _SqlRecord;
    SqlRecord.type = "record";
    SqlRecord.DEFAULT_ROW_KEYWORD = "ROW";
    SqlBase.register(SqlRecord);
  }
});

// src/sql/sql-comparison/sql-comparison.ts
var ANTI_OP, _SqlComparison, SqlComparison;
var init_sql_comparison = __esm({
  "src/sql/sql-comparison/sql-comparison.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_sql_literal();
    init_sql_record();
    init_sql_between_part();
    init_sql_like_part();
    ANTI_OP = {
      "=": "<>",
      "<>": "=",
      "<": ">=",
      ">": "<=",
      "<=": ">",
      ">=": "<"
    };
    _SqlComparison = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlComparison.type);
        this.op = options.op;
        this.negated = options.negated;
        this.lhs = options.lhs;
        this.decorator = options.decorator;
        this.rhs = options.rhs;
      }
      static equal(lhs, rhs) {
        return new _SqlComparison({
          op: "=",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static unequal(lhs, rhs) {
        return new _SqlComparison({
          op: "<>",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static lessThan(lhs, rhs) {
        return new _SqlComparison({
          op: "<",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static greaterThan(lhs, rhs) {
        return new _SqlComparison({
          op: ">",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static lessThanOrEqual(lhs, rhs) {
        return new _SqlComparison({
          op: "<=",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static greaterThanOrEqual(lhs, rhs) {
        return new _SqlComparison({
          op: ">=",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlExpression.wrap(rhs)
        });
      }
      static isNull(lhs) {
        return new _SqlComparison({
          op: "IS",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlLiteral.NULL
        });
      }
      static isNotNull(lhs) {
        return _SqlComparison.isNull(lhs).negate();
      }
      static in(lhs, values) {
        return new _SqlComparison({
          op: "IN",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlRecord.createWithoutKeyword(values.map((v) => SqlExpression.wrap(v)))
        });
      }
      static notIn(lhs, values) {
        return _SqlComparison.in(lhs, values).negate();
      }
      static like(lhs, rhs, escape) {
        const rhsEx = SqlExpression.wrap(rhs);
        return new _SqlComparison({
          op: "LIKE",
          lhs: SqlExpression.wrap(lhs),
          rhs: typeof escape === "undefined" ? rhsEx : SqlLikePart.create(rhsEx, escape)
        });
      }
      static notLike(lhs, rhs, escape) {
        return _SqlComparison.like(lhs, rhs, escape).negate();
      }
      static between(lhs, start, end) {
        return new _SqlComparison({
          op: "BETWEEN",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlBetweenPart.create(SqlExpression.wrap(start), SqlExpression.wrap(end))
        });
      }
      static notBetween(lhs, start, end) {
        return _SqlComparison.between(lhs, start, end).negate();
      }
      static betweenSymmetric(lhs, start, end) {
        return new _SqlComparison({
          op: "BETWEEN",
          lhs: SqlExpression.wrap(lhs),
          rhs: SqlBetweenPart.symmetric(SqlExpression.wrap(start), SqlExpression.wrap(end))
        });
      }
      static notBetweenSymmetric(lhs, start, end) {
        return _SqlComparison.betweenSymmetric(lhs, start, end).negate();
      }
      valueOf() {
        const value = super.valueOf();
        value.op = this.op;
        if (this.negated)
          value.negated = true;
        value.lhs = this.lhs;
        value.decorator = this.decorator;
        value.rhs = this.rhs;
        return value;
      }
      _toRawString() {
        const { lhs, op, negated, decorator, rhs } = this;
        const opIsIs = op === "IS";
        const rawParts = [lhs.toString()];
        if (negated && !opIsIs) {
          rawParts.push(this.getSpace("not"), this.getKeyword("not", _SqlComparison.DEFAULT_NOT_KEYWORD));
        }
        rawParts.push(this.getSpace("preOp"), this.getKeyword("op", op), this.getSpace("postOp"));
        if (negated && opIsIs) {
          rawParts.push(this.getKeyword("not", _SqlComparison.DEFAULT_NOT_KEYWORD), this.getSpace("not"));
        }
        if (decorator) {
          rawParts.push(this.getKeyword("decorator", decorator), this.getSpace("postDecorator"));
        }
        rawParts.push(rhs.toString());
        return rawParts.join("");
      }
      changeLhs(lhs) {
        const value = this.valueOf();
        value.lhs = lhs;
        return SqlBase.fromValue(value);
      }
      changeRhs(rhs) {
        const value = this.valueOf();
        value.rhs = rhs;
        return SqlBase.fromValue(value);
      }
      negate() {
        let { op, negated, decorator } = this;
        switch (op) {
          case "=":
          case "<>":
          case "<":
          case ">":
          case "<=":
          case ">=":
            op = ANTI_OP[op];
            if (decorator) {
              decorator = decorator === "ALL" ? "ANY" : "ALL";
            }
            break;
          default:
            negated = !negated;
            break;
        }
        const value = this.valueOf();
        value.op = op;
        value.negated = negated;
        if (!value.negated) {
          value.keywords = this.getKeywordsWithout("not");
        }
        value.decorator = decorator;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const lhs = this.lhs._walkHelper(nextStack, fn, postorder);
        if (!lhs)
          return;
        if (lhs !== this.lhs) {
          ret = ret.changeLhs(lhs);
        }
        const rhs = this.rhs._walkHelper(nextStack, fn, postorder);
        if (!rhs)
          return;
        if (rhs !== this.rhs) {
          ret = ret.changeRhs(rhs);
        }
        return ret;
      }
      getLikeMatchPattern() {
        const { op, rhs } = this;
        if (op !== "LIKE")
          return;
        if (rhs instanceof SqlLiteral) {
          return rhs.getStringValue();
        } else if (rhs instanceof SqlLikePart && rhs.like instanceof SqlLiteral) {
          return rhs.like.getStringValue();
        }
        return;
      }
      getSpecialLikeType() {
        const likeMatchPattern = this.getLikeMatchPattern();
        if (typeof likeMatchPattern !== "string")
          return;
        if (likeMatchPattern.endsWith("%")) {
          if (likeMatchPattern.startsWith("%")) {
            return "includes";
          } else {
            return "prefix";
          }
        } else if (likeMatchPattern.startsWith("%")) {
          return "postfix";
        } else {
          return "exact";
        }
      }
    };
    SqlComparison = _SqlComparison;
    SqlComparison.type = "comparison";
    SqlComparison.DEFAULT_NOT_KEYWORD = "NOT";
    SqlBase.register(SqlComparison);
  }
});

// src/sql/sql-case/sql-when-then-part.ts
var _SqlWhenThenPart, SqlWhenThenPart;
var init_sql_when_then_part = __esm({
  "src/sql/sql-case/sql-when-then-part.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_utils2();
    _SqlWhenThenPart = class extends SqlBase {
      constructor(options) {
        super(options, _SqlWhenThenPart.type);
        this.whenExpressions = options.whenExpressions;
        this.thenExpression = options.thenExpression;
      }
      static create(whenExpressions, thenExpression) {
        return new _SqlWhenThenPart({
          whenExpressions: SeparatedArray.fromArray(whenExpressions instanceof SqlExpression ? [whenExpressions] : whenExpressions),
          thenExpression
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.whenExpressions = this.whenExpressions;
        value.thenExpression = this.thenExpression;
        return value;
      }
      _toRawString() {
        return [
          this.getKeyword("when", _SqlWhenThenPart.DEFAULT_WHEN_KEYWORD),
          this.getSpace("postWhen"),
          this.whenExpressions.toString(Separator.COMMA),
          this.getSpace("postWhenExpressions"),
          this.getKeyword("then", _SqlWhenThenPart.DEFAULT_THEN_KEYWORD),
          this.getSpace("postThen"),
          this.thenExpression.toString()
        ].join("");
      }
      changeWhenExpressions(whenExpressions) {
        const value = this.valueOf();
        value.whenExpressions = SeparatedArray.fromArray(whenExpressions);
        return SqlBase.fromValue(value);
      }
      changeWhenExpression(whenExpression) {
        return this.changeWhenExpressions([whenExpression]);
      }
      changeThenExpression(thenExpression) {
        const value = this.valueOf();
        value.thenExpression = thenExpression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const whenExpressions = SqlBase.walkSeparatedArray(this.whenExpressions, nextStack, fn, postorder);
        if (!whenExpressions)
          return;
        if (whenExpressions !== this.whenExpressions) {
          ret = ret.changeWhenExpressions(whenExpressions);
        }
        const thenExpression = this.thenExpression._walkHelper(nextStack, fn, postorder);
        if (!thenExpression)
          return;
        if (thenExpression !== this.thenExpression) {
          ret = ret.changeThenExpression(thenExpression);
        }
        return ret;
      }
    };
    SqlWhenThenPart = _SqlWhenThenPart;
    SqlWhenThenPart.type = "whenThenPart";
    SqlWhenThenPart.DEFAULT_WHEN_KEYWORD = "WHEN";
    SqlWhenThenPart.DEFAULT_THEN_KEYWORD = "THEN";
    SqlBase.register(SqlWhenThenPart);
  }
});

// src/sql/sql-case/sql-case.ts
var _SqlCase, SqlCase;
var init_sql_case = __esm({
  "src/sql/sql-case/sql-case.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_utils2();
    init_sql_when_then_part();
    _SqlCase = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlCase.type);
        this.caseExpression = options.caseExpression;
        this.whenThenParts = options.whenThenParts;
        this.elseExpression = options.elseExpression;
      }
      static ifThenElse(conditionExpression, thenExpression, elseExpression) {
        return new _SqlCase({
          whenThenParts: SeparatedArray.fromSingleValue(SqlWhenThenPart.create(conditionExpression, thenExpression)),
          elseExpression
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.caseExpression = this.caseExpression;
        value.whenThenParts = this.whenThenParts;
        value.elseExpression = this.elseExpression;
        return value;
      }
      _toRawString() {
        const rawParts = [
          this.getKeyword("case", _SqlCase.DEFAULT_CASE_KEYWORD),
          this.getSpace("postCase")
        ];
        if (this.caseExpression) {
          rawParts.push(this.caseExpression.toString(), this.getSpace("postCaseExpression"));
        }
        rawParts.push(this.whenThenParts.toString(" "));
        if (this.elseExpression) {
          rawParts.push(this.getSpace("preElse"), this.getKeyword("else", _SqlCase.DEFAULT_ELSE_KEYWORD), this.getSpace("postElse"), this.elseExpression.toString());
        }
        rawParts.push(this.getSpace("preEnd"), this.getKeyword("end", _SqlCase.DEFAULT_END_KEYWORD));
        return rawParts.join("");
      }
      changeCaseExpression(caseExpression) {
        const value = this.valueOf();
        value.caseExpression = caseExpression;
        return SqlBase.fromValue(value);
      }
      changeWhenThenParts(whenThenParts) {
        const value = this.valueOf();
        value.whenThenParts = whenThenParts;
        return SqlBase.fromValue(value);
      }
      changeElseExpression(elseExpression) {
        const value = this.valueOf();
        value.elseExpression = elseExpression;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.caseExpression) {
          const caseExpression = this.caseExpression._walkHelper(nextStack, fn, postorder);
          if (!caseExpression)
            return;
          if (caseExpression !== this.caseExpression) {
            ret = ret.changeCaseExpression(caseExpression);
          }
        }
        const whenThenParts = SqlBase.walkSeparatedArray(this.whenThenParts, nextStack, fn, postorder);
        if (!whenThenParts)
          return;
        if (whenThenParts !== this.whenThenParts) {
          ret = ret.changeWhenThenParts(whenThenParts);
        }
        if (this.elseExpression) {
          const elseExpression = this.elseExpression._walkHelper(nextStack, fn, postorder);
          if (!elseExpression)
            return;
          if (elseExpression !== this.elseExpression) {
            ret = ret.changeElseExpression(elseExpression);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.whenThenParts = this.whenThenParts.clearSeparators();
        return SqlBase.fromValue(value);
      }
    };
    SqlCase = _SqlCase;
    SqlCase.type = "case";
    SqlCase.DEFAULT_CASE_KEYWORD = "CASE";
    SqlCase.DEFAULT_ELSE_KEYWORD = "ELSE";
    SqlCase.DEFAULT_END_KEYWORD = "END";
    SqlBase.register(SqlCase);
  }
});

// src/sql/sql-alias/sql-alias.ts
var _SqlAlias, SqlAlias;
var init_sql_alias = __esm({
  "src/sql/sql-alias/sql-alias.ts"() {
    "use strict";
    init_sql_base();
    init_sql_column();
    init_sql_expression();
    init_utils2();
    _SqlAlias = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlAlias.type);
        this.expression = options.expression;
        this.alias = options.alias;
        this.columns = options.columns;
      }
      static create(expression, alias, forceQuotes) {
        if (expression instanceof _SqlAlias) {
          return expression.changeAlias(alias, forceQuotes);
        }
        if (expression.type === "query") {
          expression = expression.ensureParens();
        }
        return new _SqlAlias({
          expression,
          alias: typeof alias === "string" ? RefName.alias(alias, forceQuotes) : alias
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.expression = this.expression;
        value.alias = this.alias;
        value.columns = this.columns;
        return value;
      }
      _toRawString() {
        const rawParts = [this.expression.toString()];
        if (this.keywords["as"] !== "") {
          rawParts.push(this.getSpace("preAs"), this.getKeyword("as", _SqlAlias.DEFAULT_AS_KEYWORD));
        }
        rawParts.push(this.getSpace("preAlias"), this.alias.toString());
        if (this.columns) {
          rawParts.push(this.getSpace("preColumns"), this.columns.toString());
        }
        return rawParts.join("");
      }
      changeExpression(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      changeAlias(alias, forceQuotes = false) {
        const newAlias = typeof alias === "string" ? forceQuotes ? RefName.alias(alias, forceQuotes) : this.alias.changeNameAsAlias(alias) : alias;
        const value = this.valueOf();
        value.alias = newAlias;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expression = this.expression._walkHelper(nextStack, fn, postorder);
        if (!expression)
          return;
        if (expression !== this.expression) {
          ret = ret.changeExpression(expression);
        }
        return ret;
      }
      ifUnnamedAliasAs(_alias, _forceQuotes) {
        return this;
      }
      convertToTable() {
        const { expression } = this;
        if (expression instanceof SqlColumn) {
          return this.changeExpression(expression.convertToTable());
        }
        return this;
      }
      getAliasName() {
        return this.alias.name;
      }
      getOutputName() {
        return this.alias.name;
      }
      getUnderlyingExpression() {
        return this.expression;
      }
    };
    SqlAlias = _SqlAlias;
    SqlAlias.type = "alias";
    SqlAlias.DEFAULT_AS_KEYWORD = "AS";
    SqlBase.register(SqlAlias);
  }
});

// src/sql/sql-labeled-expression/sql-labeled-expression.ts
var _SqlLabeledExpression, SqlLabeledExpression;
var init_sql_labeled_expression = __esm({
  "src/sql/sql-labeled-expression/sql-labeled-expression.ts"() {
    "use strict";
    init_sql_base();
    init_sql_expression();
    init_utils2();
    _SqlLabeledExpression = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlLabeledExpression.type);
        this.label = options.label;
        this.expression = options.expression;
      }
      static create(label, expression, forceQuotes) {
        if (expression instanceof _SqlLabeledExpression) {
          return expression.changeLabel(label, forceQuotes);
        }
        return new _SqlLabeledExpression({
          expression,
          label: typeof label === "string" ? RefName.alias(label, forceQuotes) : label
        });
      }
      valueOf() {
        const value = super.valueOf();
        value.label = this.label;
        value.expression = this.expression;
        return value;
      }
      _toRawString() {
        return [
          this.label.toString(),
          this.getSpace("preArrow"),
          "=>",
          this.getSpace("postArrow"),
          this.expression.toString()
        ].join("");
      }
      changeExpression(expression) {
        const value = this.valueOf();
        value.expression = expression;
        return SqlBase.fromValue(value);
      }
      changeLabel(label, forceQuotes = false) {
        const newLabel = typeof label === "string" ? forceQuotes ? RefName.alias(label, forceQuotes) : this.label.changeNameAsAlias(label) : label;
        const value = this.valueOf();
        value.label = newLabel;
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const expression = this.expression._walkHelper(nextStack, fn, postorder);
        if (!expression)
          return;
        if (expression !== this.expression) {
          ret = ret.changeExpression(expression);
        }
        return ret;
      }
      getLabelName() {
        return this.label.name;
      }
      getUnderlyingExpression() {
        return this.expression;
      }
    };
    SqlLabeledExpression = _SqlLabeledExpression;
    SqlLabeledExpression.type = "labeledExpression";
    SqlBase.register(SqlLabeledExpression);
  }
});

// src/sql/sql-window-spec/sql-window-spec.ts
var _SqlWindowSpec, SqlWindowSpec;
var init_sql_window_spec = __esm({
  "src/sql/sql-window-spec/sql-window-spec.ts"() {
    "use strict";
    init_sql_base();
    init_utils2();
    _SqlWindowSpec = class extends SqlBase {
      constructor(options) {
        super(options, _SqlWindowSpec.type);
        this.windowName = options.windowName;
        this.orderByClause = options.orderByClause;
        this.partitionByClause = options.partitionByClause;
      }
      valueOf() {
        const value = super.valueOf();
        value.windowName = this.windowName;
        value.orderByClause = this.orderByClause;
        value.partitionByClause = this.partitionByClause;
        return value;
      }
      _toRawString() {
        const rawParts = ["(", this.getSpace("postLeftParen")];
        if (this.windowName) {
          rawParts.push(this.windowName.toString(), this.getSpace("postWindowName"));
        }
        if (this.orderByClause) {
          rawParts.push(this.orderByClause.toString(), this.getSpace("postOrderBy"));
        }
        if (this.partitionByClause) {
          rawParts.push(this.partitionByClause.toString(), this.getSpace("postPartitionBy"));
        }
        rawParts.push(")");
        return rawParts.join("");
      }
      changeWindowName(windowName) {
        const value = this.valueOf();
        if (windowName) {
          value.windowName = RefName.create(windowName);
        } else {
          delete value.windowName;
          value.spacing = this.getSpacingWithout("postWindowName");
        }
        return SqlBase.fromValue(value);
      }
      changeOrderByClause(orderByClause) {
        if (this.orderByClause === orderByClause)
          return this;
        const value = this.valueOf();
        if (orderByClause) {
          value.orderByClause = orderByClause;
        } else {
          delete value.orderByClause;
          value.spacing = this.getSpacingWithout("preOrderByClause");
        }
        return SqlBase.fromValue(value);
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        if (this.orderByClause) {
          const orderByClause = this.orderByClause._walkHelper(nextStack, fn, postorder);
          if (!orderByClause)
            return;
          if (orderByClause !== this.orderByClause) {
            ret = ret.changeOrderByClause(orderByClause);
          }
        }
        return ret;
      }
    };
    SqlWindowSpec = _SqlWindowSpec;
    SqlWindowSpec.type = "windowSpec";
    SqlBase.register(SqlWindowSpec);
  }
});

// src/sql/sql-values/sql-values.ts
var _SqlValues, SqlValues;
var init_sql_values = __esm({
  "src/sql/sql-values/sql-values.ts"() {
    "use strict";
    init_utils();
    init_sql_base();
    init_sql_clause2();
    init_sql_expression();
    init_utils2();
    _SqlValues = class extends SqlExpression {
      constructor(options) {
        super(options, _SqlValues.type);
        this.records = options.records;
        this.orderByClause = options.orderByClause;
        this.limitClause = options.limitClause;
        this.offsetClause = options.offsetClause;
      }
      static create(records) {
        if (records instanceof _SqlValues)
          return records;
        return new _SqlValues({
          records: SeparatedArray.fromArray(records)
        }).ensureParens();
      }
      valueOf() {
        const value = super.valueOf();
        value.records = this.records;
        value.orderByClause = this.orderByClause;
        value.limitClause = this.limitClause;
        value.offsetClause = this.offsetClause;
        return value;
      }
      _toRawString() {
        const { records, orderByClause, limitClause, offsetClause } = this;
        const multiline = records.length() > 1;
        const rawParts = [
          this.getKeyword("values", _SqlValues.DEFAULT_VALUES_KEYWORD),
          this.getSpace("postValues", multiline ? "\n" : " "),
          records.toString(multiline ? Separator.COMMA_NEW_LINE : Separator.COMMA)
        ];
        if (orderByClause) {
          rawParts.push(this.getSpace("preOrderByClause", "\n"), orderByClause.toString());
        }
        if (limitClause) {
          rawParts.push(this.getSpace("preLimitClause", "\n"), limitClause.toString());
        }
        if (offsetClause) {
          rawParts.push(this.getSpace("preOffsetClause", "\n"), offsetClause.toString());
        }
        return rawParts.join("");
      }
      changeRecords(records) {
        const value = this.valueOf();
        value.records = SeparatedArray.fromArray(records, Separator.COMMA);
        return SqlBase.fromValue(value);
      }
      changeOrderByClause(orderByClause) {
        if (this.orderByClause === orderByClause)
          return this;
        const value = this.valueOf();
        if (orderByClause) {
          value.orderByClause = orderByClause;
        } else {
          delete value.orderByClause;
          value.spacing = this.getSpacingWithout("preOrderByClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOrderByExpressions(orderByExpressions) {
        if (!orderByExpressions || isEmptyArray(orderByExpressions)) {
          return this.changeOrderByClause(void 0);
        } else {
          return this.changeOrderByClause(this.orderByClause ? this.orderByClause.changeExpressions(orderByExpressions) : SqlOrderByClause.create(orderByExpressions));
        }
      }
      changeOrderByExpression(orderByExpression) {
        if (!orderByExpression)
          return this.changeOrderByClause(void 0);
        return this.changeOrderByExpressions([orderByExpression]);
      }
      changeLimitClause(limitClause) {
        if (this.limitClause === limitClause)
          return this;
        const value = this.valueOf();
        if (limitClause) {
          value.limitClause = limitClause;
        } else {
          delete value.limitClause;
          value.spacing = this.getSpacingWithout("preLimitClause");
        }
        return SqlBase.fromValue(value);
      }
      changeLimitValue(limitValue) {
        if (typeof limitValue === "undefined")
          return this.changeLimitClause(void 0);
        return this.changeLimitClause(this.limitClause ? this.limitClause.changeLimit(limitValue) : SqlLimitClause.create(limitValue));
      }
      changeOffsetClause(offsetClause) {
        if (this.offsetClause === offsetClause)
          return this;
        const value = this.valueOf();
        if (offsetClause) {
          value.offsetClause = offsetClause;
        } else {
          delete value.offsetClause;
          value.spacing = this.getSpacingWithout("preOffsetClause");
        }
        return SqlBase.fromValue(value);
      }
      changeOffsetValue(offsetValue) {
        if (typeof offsetValue === "undefined")
          return this.changeOffsetClause(void 0);
        return this.changeOffsetClause(this.offsetClause ? this.offsetClause.changeOffset(offsetValue) : SqlOffsetClause.create(offsetValue));
      }
      _walkInner(nextStack, fn, postorder) {
        let ret = this;
        const records = SqlBase.walkSeparatedArray(this.records, nextStack, fn, postorder);
        if (!records)
          return;
        if (records !== this.records) {
          ret = ret.changeRecords(records);
        }
        if (this.orderByClause) {
          const orderByClause = this.orderByClause._walkHelper(nextStack, fn, postorder);
          if (!orderByClause)
            return;
          if (orderByClause !== this.orderByClause) {
            ret = ret.changeOrderByClause(orderByClause);
          }
        }
        if (this.limitClause) {
          const limitClause = this.limitClause._walkHelper(nextStack, fn, postorder);
          if (!limitClause)
            return;
          if (limitClause !== this.limitClause) {
            ret = ret.changeLimitClause(limitClause);
          }
        }
        if (this.offsetClause) {
          const offsetClause = this.offsetClause._walkHelper(nextStack, fn, postorder);
          if (!offsetClause)
            return;
          if (offsetClause !== this.offsetClause) {
            ret = ret.changeOffsetClause(offsetClause);
          }
        }
        return ret;
      }
      clearOwnSeparators() {
        const value = this.valueOf();
        value.records = this.records.clearSeparators();
        return SqlBase.fromValue(value);
      }
    };
    SqlValues = _SqlValues;
    SqlValues.type = "values";
    SqlValues.DEFAULT_VALUES_KEYWORD = "VALUES";
    SqlBase.register(SqlValues);
  }
});

// src/sql/shortcuts/column.ts
function C(name) {
  return SqlColumn.create(name);
}
var init_column = __esm({
  "src/sql/shortcuts/column.ts"() {
    "use strict";
    init_sql_column();
    C.optionalQuotes = (name) => {
      return SqlColumn.optionalQuotes(name);
    };
  }
});

// src/sql/shortcuts/function.ts
var F;
var init_function = __esm({
  "src/sql/shortcuts/function.ts"() {
    "use strict";
    init_sql_function();
    F = function(name, ...args) {
      return SqlFunction.simple(name, args);
    };
    F.cast = SqlFunction.cast;
    F.floor = SqlFunction.floor;
    F.timeFloor = SqlFunction.timeFloor;
    F.timeCeil = SqlFunction.timeCeil;
    F.timeShift = SqlFunction.timeShift;
    F.stringFormat = SqlFunction.stringFormat;
    F.array = SqlFunction.array;
    F.regexpLike = SqlFunction.regexpLike;
    F.count = SqlFunction.count;
    F.countDistinct = SqlFunction.countDistinct;
    F.sum = SqlFunction.sum;
    F.min = SqlFunction.min;
    F.max = SqlFunction.max;
    F.avg = SqlFunction.avg;
  }
});

// src/sql/shortcuts/literal.ts
function L(value) {
  return SqlLiteral.create(value);
}
var init_literal = __esm({
  "src/sql/shortcuts/literal.ts"() {
    "use strict";
    init_sql_literal();
    L.NULL = SqlLiteral.NULL;
    L.FALSE = SqlLiteral.FALSE;
    L.TRUE = SqlLiteral.TRUE;
  }
});

// src/sql/shortcuts/namespace.ts
function N(name) {
  return SqlNamespace.create(name);
}
var init_namespace = __esm({
  "src/sql/shortcuts/namespace.ts"() {
    "use strict";
    init_sql_namespace();
    N.optionalQuotes = (name) => {
      return SqlNamespace.optionalQuotes(name);
    };
  }
});

// src/sql/shortcuts/table.ts
function T(name) {
  return SqlTable.create(name);
}
var init_table = __esm({
  "src/sql/shortcuts/table.ts"() {
    "use strict";
    init_sql_table();
    T.optionalQuotes = (name) => {
      return SqlTable.optionalQuotes(name);
    };
  }
});

// src/sql/shortcuts/index.ts
var init_shortcuts = __esm({
  "src/sql/shortcuts/index.ts"() {
    "use strict";
    init_column();
    init_function();
    init_literal();
    init_namespace();
    init_table();
  }
});

// src/sql/index.ts
var sql_exports = {};
__export(sql_exports, {
  C: () => C,
  F: () => F,
  L: () => L,
  N: () => N,
  RefName: () => RefName,
  SeparatedArray: () => SeparatedArray,
  Separator: () => Separator,
  SqlAlias: () => SqlAlias,
  SqlBase: () => SqlBase,
  SqlBetweenPart: () => SqlBetweenPart,
  SqlCase: () => SqlCase,
  SqlClause: () => SqlClause,
  SqlClusteredByClause: () => SqlClusteredByClause,
  SqlColumn: () => SqlColumn,
  SqlColumnDeclaration: () => SqlColumnDeclaration,
  SqlColumnList: () => SqlColumnList,
  SqlComparison: () => SqlComparison,
  SqlExpression: () => SqlExpression,
  SqlExtendClause: () => SqlExtendClause,
  SqlFromClause: () => SqlFromClause,
  SqlFunction: () => SqlFunction,
  SqlGroupByClause: () => SqlGroupByClause,
  SqlHavingClause: () => SqlHavingClause,
  SqlInsertClause: () => SqlInsertClause,
  SqlInterval: () => SqlInterval,
  SqlJoinPart: () => SqlJoinPart,
  SqlLabeledExpression: () => SqlLabeledExpression,
  SqlLikePart: () => SqlLikePart,
  SqlLimitClause: () => SqlLimitClause,
  SqlLiteral: () => SqlLiteral,
  SqlMulti: () => SqlMulti,
  SqlNamespace: () => SqlNamespace,
  SqlOffsetClause: () => SqlOffsetClause,
  SqlOrderByClause: () => SqlOrderByClause,
  SqlOrderByExpression: () => SqlOrderByExpression,
  SqlPartitionedByClause: () => SqlPartitionedByClause3,
  SqlPlaceholder: () => SqlPlaceholder,
  SqlQuery: () => SqlQuery,
  SqlRecord: () => SqlRecord,
  SqlReplaceClause: () => SqlReplaceClause,
  SqlStar: () => SqlStar,
  SqlTable: () => SqlTable,
  SqlType: () => SqlType,
  SqlUnary: () => SqlUnary,
  SqlValues: () => SqlValues,
  SqlWhenThenPart: () => SqlWhenThenPart,
  SqlWhereClause: () => SqlWhereClause,
  SqlWindowSpec: () => SqlWindowSpec,
  SqlWithClause: () => SqlWithClause,
  SqlWithPart: () => SqlWithPart,
  SqlWithQuery: () => SqlWithQuery,
  T: () => T,
  clampIndex: () => clampIndex,
  getEscapedRegExp: () => getEscapedRegExp,
  insert: () => insert,
  needsUnicodeEscape: () => needsUnicodeEscape,
  normalizeIndex: () => normalizeIndex,
  sqlEscapeUnicode: () => sqlEscapeUnicode,
  trimString: () => trimString
});
var init_sql = __esm({
  "src/sql/index.ts"() {
    "use strict";
    init_utils2();
    init_sql_base();
    init_sql_type();
    init_sql_expression();
    init_sql_column_list();
    init_sql_placeholder();
    init_sql_literal();
    init_sql_interval();
    init_sql_namespace();
    init_sql_table();
    init_sql_star();
    init_sql_column();
    init_sql_multi();
    init_sql_unary();
    init_sql_between_part();
    init_sql_like_part();
    init_sql_comparison();
    init_sql_when_then_part();
    init_sql_case();
    init_sql_alias();
    init_sql_labeled_expression();
    init_sql_window_spec();
    init_sql_clause2();
    init_sql_record();
    init_sql_values();
    init_sql_function();
    init_sql_query();
    init_sql_with_query();
    init_shortcuts();
  }
});

// src/introspect/introspect.ts
function guessTypeFromValue(v) {
  if (v instanceof Date)
    return "TIMESTAMP";
  if (v === true || v === false)
    return "BOOLEAN";
  return;
}
var Introspect;
var init_introspect = __esm({
  "src/introspect/introspect.ts"() {
    "use strict";
    init_sql();
    init_utils();
    Introspect = class {
      static getTableIntrospectionQuery() {
        return `SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA = 'druid' AND TABLE_TYPE = 'TABLE'`;
      }
      static decodeTableIntrospectionResult(queryResult) {
        if (queryResult.isEmpty())
          return [];
        return queryResult.rows.map((r) => ({ name: r[0] }));
      }
      static getTableColumnIntrospectionQuery(thing) {
        let tableName;
        if (thing instanceof SqlTable) {
          tableName = thing.getName();
        } else {
          tableName = thing;
        }
        return SqlQuery.parse(`SELECT COLUMN_NAME, DATA_TYPE FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = 'druid' AND TABLE_NAME = ${SqlLiteral.create(tableName)}`);
      }
      static decodeTableColumnIntrospectionResult(queryResult) {
        if (queryResult.isEmpty())
          return [];
        if (queryResult.getHeaderNames().join(",") !== "COLUMN_NAME,DATA_TYPE") {
          throw new Error("invalid result shape, bad header");
        }
        return queryResult.rows.map((r) => ({ name: r[0], type: r[1] }));
      }
      static getQueryColumnIntrospectionQuery(query) {
        return query.makeExplain();
      }
      static getQueryColumnSampleQuery(query) {
        return query.changeLimitValue(1);
      }
      static decodeColumnTypesFromPlan(queryPlanResult) {
        if (queryPlanResult.isEmpty())
          return [];
        const { rows } = queryPlanResult;
        if (queryPlanResult.getHeaderNames()[0] !== "PLAN") {
          throw new Error("invalid result shape, bad header");
        }
        if (rows.length !== 1)
          throw new Error("invalid result shape, bad number of results");
        const plan = rows[0][0];
        let jsonPlan;
        try {
          jsonPlan = JSON.parse(plan);
        } catch {
          const m = plan.match(/ signature=\[\{([^}]*)}]/m);
          if (!m)
            throw new Error("could not find signature");
          return m[1].match(/:[A-Z]+(?:<[A-Za-z]+>)?(?:, |$)/g).map((t) => {
            return t.replace(/[:, ]/g, "");
          });
        }
        if (!Array.isArray(jsonPlan) || jsonPlan.length === 0) {
          throw new Error("could not find signature in JSON plan");
        }
        const signature = jsonPlan[0].signature;
        if (!signature)
          throw new Error("could not find signature in JSON plan");
        return signature.map((s) => s.type);
      }
      static decodeQueryColumnIntrospectionResult(queryPlanResult, sampleRowResult) {
        const { sqlQuery } = queryPlanResult;
        if (queryPlanResult.isEmpty())
          return [];
        if (!sqlQuery)
          throw new Error("must have a sqlQuery");
        const types = Introspect.decodeColumnTypesFromPlan(queryPlanResult);
        if (sampleRowResult) {
          if (sampleRowResult.header.length !== types.length) {
            throw new Error(`number of columns in the sample row (${sampleRowResult.header.length}) does not match the number of types (${types.length})`);
          }
          const sampleRow = sampleRowResult.rows[0];
          return filterMap(sampleRowResult.header, (column, i) => {
            const columnName = column.name;
            if (SqlQuery.isPhonyOutputName(columnName))
              return;
            let type = sampleRow ? guessTypeFromValue(sampleRow[i]) : void 0;
            if (!type) {
              type = types[i];
              if (!type)
                return;
              if (columnName === "__time" && type === "LONG") {
                type = "TIMESTAMP";
              }
            }
            return { name: columnName, type };
          });
        } else {
          if (sqlQuery.hasStarInSelect()) {
            throw new Error("a query with a star must have sampleRowResult set");
          }
          const outputColumns = sqlQuery.getOutputColumns();
          if (outputColumns.length !== types.length) {
            throw new Error("number of output columns does not match the number of types");
          }
          return filterMap(outputColumns, (name, i) => {
            if (!sqlQuery.isRealOutputColumnAtSelectIndex(i))
              return;
            let type = types[i];
            if (!type)
              return;
            if (name === "__time" && type === "LONG") {
              type = "TIMESTAMP";
            }
            return { name: String(name), type };
          });
        }
      }
      static decodeColumnIntrospectionResult(queryResult, sampleRowResult) {
        const headerNames = queryResult.getHeaderNames();
        if (headerNames[0] === "COLUMN_NAME" && headerNames[1] === "DATA_TYPE") {
          return Introspect.decodeTableColumnIntrospectionResult(queryResult);
        } else if (headerNames[0] === "PLAN") {
          return Introspect.decodeQueryColumnIntrospectionResult(queryResult, sampleRowResult);
        } else {
          throw new Error(`unknown header shape: '${headerNames.join("', '")}'`);
        }
      }
    };
  }
});

// src/query-result/column.ts
var Column;
var init_column2 = __esm({
  "src/query-result/column.ts"() {
    "use strict";
    Column = class {
      static fromName(name) {
        return new Column({ name: String(name) });
      }
      static fromColumnNames(names) {
        return names.map(Column.fromName);
      }
      static fromColumnNamesAndTypeArrays(names, types = [], sqlTypes = []) {
        return names.map((name, i) => new Column({ name: String(name), nativeType: types[i], sqlType: sqlTypes[i] }));
      }
      static fromColumnNamesAndTypeArray(names, typeArray = []) {
        return names.map((name, i) => {
          var _a, _b;
          return new Column({
            name: String(name),
            nativeType: (_a = typeArray[i]) == null ? void 0 : _a.type,
            sqlType: (_b = typeArray[i]) == null ? void 0 : _b.sqlType
          });
        });
      }
      constructor(options) {
        this.name = options.name;
        this.sqlType = options.sqlType;
        this.nativeType = options.nativeType;
      }
      isTimeColumn() {
        const { name, sqlType } = this;
        return name === "__time" && sqlType === "TIMESTAMP";
      }
    };
  }
});

// src/query-result/query-result.ts
function isObject(obj) {
  return Object.prototype.toString.call(obj) === "[object Object]";
}
function isIsoDate(thing) {
  return typeof thing === "string" && FULL_ISO_REGEXP.test(thing) && !isNaN(new Date(thing).valueOf());
}
function isAllGranularity(granularity) {
  if (typeof granularity === "string") {
    return granularity.toLowerCase() === "all";
  }
  if (isObject(granularity) && typeof granularity.type === "string") {
    return granularity.type.toLowerCase() === "all";
  }
  return false;
}
var FULL_ISO_REGEXP, _QueryResult, QueryResult;
var init_query_result = __esm({
  "src/query-result/query-result.ts"() {
    "use strict";
    init_utils();
    init_column2();
    FULL_ISO_REGEXP = /^\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d.\d\d\dZ$/;
    _QueryResult = class {
      static shouldIncludeTimestamp(queryPayload) {
        return Boolean(queryPayload.granularity && !isAllGranularity(queryPayload.granularity));
      }
      static hasHeader(queryPayload) {
        return typeof queryPayload.query === "string" && queryPayload.header === true;
      }
      static hasTypeHeader(queryPayload, header) {
        return _QueryResult.hasHeader(queryPayload) && queryPayload.typesHeader === true && header["x-druid-sql-header-included"] === "yes";
      }
      static hasSqlTypeHeader(queryPayload, header) {
        return _QueryResult.hasHeader(queryPayload) && queryPayload.sqlTypesHeader === true && header["x-druid-sql-header-included"] === "yes";
      }
      static fromQueryAndRawResult(queryPayload, data, header = {}) {
        return _QueryResult.fromRawResult(data, _QueryResult.shouldIncludeTimestamp(queryPayload), _QueryResult.hasHeader(queryPayload), _QueryResult.hasTypeHeader(queryPayload, header), _QueryResult.hasSqlTypeHeader(queryPayload, header));
      }
      static fromRawResult(data, includeTimestampIfExists, hasHeader, hasTypeHeader, hasSqlTypeHeader) {
        if (typeof data === "string") {
          if (!data.endsWith("\n")) {
            throw new Error(`Query results were truncated midstream! This may indicate a server-side error or a client-side issue. Try re-running your query, or using a lower limit or a longer timeout.`);
          }
          const trimmedDataString = data.trimEnd();
          if (trimmedDataString === "") {
            data = [];
          } else {
            data = trimmedDataString.split("\n").map((line, i) => {
              try {
                return _QueryResult.jsonParse(line);
              } catch {
                throw new Error(`Unparsable row on line ${i + 1} in query result: '${line}'.`);
              }
            });
          }
        }
        let resultContext;
        if (isObject(data) && data.results) {
          if (isObject(data.context)) {
            resultContext = data.context;
          }
          data = data.results;
        }
        if (Array.isArray(data)) {
          const firstRow = data[0];
          if (!firstRow)
            return _QueryResult.BLANK.changeResultContext(resultContext);
          if (Array.isArray(firstRow)) {
            let rowsToSkip = 0;
            let header;
            if (hasHeader) {
              rowsToSkip++;
              let types;
              if (hasTypeHeader) {
                types = data[rowsToSkip];
                rowsToSkip++;
              }
              let sqlTypes;
              if (hasSqlTypeHeader) {
                sqlTypes = data[rowsToSkip];
                rowsToSkip++;
              }
              header = Column.fromColumnNamesAndTypeArrays(firstRow, types, sqlTypes);
            } else {
              header = Column.fromColumnNames(firstRow.map((_d, i) => i));
            }
            return new _QueryResult({
              header,
              rows: rowsToSkip ? data.slice(rowsToSkip) : data,
              resultContext
            });
          }
          if (!isObject(firstRow)) {
            throw new Error(`Unexpected query result, array of non objects or arrays.`);
          }
          if (typeof firstRow.timestamp === "string" && firstRow.result) {
            const firstRowResult = firstRow.result;
            if (isObject(firstRowResult)) {
              if (Array.isArray(firstRowResult.results)) {
                return _QueryResult.fromRawResult(data.flatMap((d) => d.result.results), includeTimestampIfExists, hasHeader).changeResultContext(resultContext);
              }
              if (isObject(firstRowResult.pagingIdentifiers) && Array.isArray(firstRowResult.dimensions) && Array.isArray(firstRowResult.metrics) && Array.isArray(firstRowResult.events)) {
                const selectHeader = ["timestamp"].concat(firstRowResult.dimensions, firstRowResult.metrics);
                return new _QueryResult({
                  header: Column.fromColumnNames(selectHeader),
                  rows: data.flatMap((r) => r.result.events.map((r2) => selectHeader.map((h) => r2.event[h]))),
                  resultContext
                });
              }
              const header = Object.keys(firstRowResult);
              return new _QueryResult({
                header: Column.fromColumnNames(includeTimestampIfExists ? ["timestamp"].concat(header) : header),
                rows: data.map((r) => {
                  const { timestamp, result } = r;
                  const rest = header.map((h) => result[h]);
                  return includeTimestampIfExists ? [timestamp].concat(rest) : rest;
                }),
                resultContext
              });
            }
            if (Array.isArray(firstRowResult)) {
              const firstSubRow = data.find((r) => r.result[0]);
              if (!firstSubRow)
                return _QueryResult.BLANK;
              const header = Object.keys(firstSubRow.result[0]);
              return new _QueryResult({
                header: Column.fromColumnNames(includeTimestampIfExists ? ["timestamp"].concat(header) : header),
                rows: data.flatMap((r) => {
                  const { timestamp, result } = r;
                  return result.map((subResult) => {
                    const rest = header.map((h) => subResult[h]);
                    return includeTimestampIfExists ? [timestamp].concat(rest) : rest;
                  });
                }),
                resultContext
              });
            }
          }
          if (typeof firstRow.timestamp === "string" && isObject(firstRow.event)) {
            const header = Object.keys(firstRow.event);
            return new _QueryResult({
              header: Column.fromColumnNames(includeTimestampIfExists ? ["timestamp"].concat(header) : header),
              rows: data.map((r) => {
                const { timestamp, event } = r;
                const rest = header.map((h) => event[h]);
                return includeTimestampIfExists ? [timestamp].concat(rest) : rest;
              }),
              resultContext
            });
          }
          if (Array.isArray(firstRow.columns) && Array.isArray(firstRow.events)) {
            const firstRowColumnsKey = firstRow.columns.join("#");
            const headerNames = firstRow.columns;
            const headerNameToIndex = /* @__PURE__ */ new Map();
            for (let i = 0; i < headerNames.length; i++) {
              headerNameToIndex.set(headerNames[i], i);
            }
            const headerRemaps = data.map(({ columns }, i) => {
              if (i === 0 || columns.join("#") === firstRowColumnsKey)
                return;
              const remap = /* @__PURE__ */ new Map();
              for (let j = 0; j < columns.length; j++) {
                const column = columns[j];
                if (!headerNameToIndex.has(column)) {
                  headerNameToIndex.set(column, headerNames.length);
                  headerNames.push(column);
                }
                remap.set(column, j);
              }
              return remap;
            });
            const header = Column.fromColumnNames(headerNames);
            const firstSubRow = data.find((r) => r.events[0]);
            if (!firstSubRow)
              return new _QueryResult({ header, rows: [] });
            const firstSubRowEvents = firstSubRow.events[0];
            if (Array.isArray(firstSubRowEvents)) {
              return new _QueryResult({
                header,
                rows: data.flatMap(({ events }, i) => {
                  const headerRemap = headerRemaps[i];
                  if (!headerRemap)
                    return events;
                  return events.map((event) => {
                    return headerNames.map((column) => {
                      if (!headerRemap.has(column))
                        return null;
                      return event[headerRemap.get(column)];
                    });
                  });
                }),
                resultContext
              });
            }
            if (isObject(firstSubRowEvents)) {
              return new _QueryResult({
                header,
                rows: data.flatMap(({ events }) => events.map((event) => headerNames.map((h) => event[h]))),
                resultContext
              });
            }
            throw new Error(`Unexpected scan like results.`);
          }
          if (typeof firstRow.id === "string" && isObject(firstRow.columns)) {
            const flatArray = data.flatMap(({ columns }) => Object.keys(columns).map((k) => __spreadValues({ column: k }, columns[k])));
            return _QueryResult.fromObjectArray(flatArray).changeResultContext(resultContext);
          }
          return _QueryResult.fromObjectArray(data, hasHeader, hasTypeHeader || hasSqlTypeHeader).changeResultContext(resultContext);
        }
        throw new Error("Unrecognizable query return shape, not an array.");
      }
      static fromObjectArray(array, hasHeader, hasTypes) {
        const firstRow = array[0];
        if (!firstRow)
          return _QueryResult.BLANK;
        const header = Object.keys(firstRow);
        return new _QueryResult({
          header: Column.fromColumnNamesAndTypeArray(header, hasHeader && hasTypes ? header.map((h) => firstRow[h]) : void 0),
          rows: (hasHeader ? array.slice(1) : array).map((r) => header.map((h) => r[h]))
        });
      }
      constructor(value) {
        this.header = value.header;
        this.rows = value.rows;
        this.query = value.query;
        this.sqlQuery = value.sqlQuery;
        this.queryId = value.queryId;
        this.sqlQueryId = value.sqlQueryId;
        this.resultContext = value.resultContext;
        this.queryDuration = value.queryDuration;
      }
      valueOf() {
        return {
          header: this.header,
          rows: this.rows,
          query: this.query,
          sqlQuery: this.sqlQuery,
          queryId: this.queryId,
          sqlQueryId: this.sqlQueryId,
          resultContext: this.resultContext,
          queryDuration: this.queryDuration
        };
      }
      changeQueryDuration(queryDuration) {
        const value = this.valueOf();
        value.queryDuration = queryDuration;
        return new _QueryResult(value);
      }
      attachQuery(query, sqlQuery) {
        const value = this.valueOf();
        value.query = query;
        value.sqlQuery = sqlQuery;
        return new _QueryResult(value);
      }
      attachQueryId(queryId, sqlQueryId) {
        const value = this.valueOf();
        value.queryId = queryId;
        value.sqlQueryId = sqlQueryId;
        return new _QueryResult(value);
      }
      changeResultContext(resultContext) {
        if (this.resultContext === resultContext)
          return this;
        const value = this.valueOf();
        value.resultContext = resultContext;
        return new _QueryResult(value);
      }
      getHeaderNames() {
        return this.header.map((h) => h.name);
      }
      isEmpty() {
        return !this.rows.length;
      }
      getNumResults() {
        return this.rows.length;
      }
      toObjectArray() {
        const headerNames = this.getHeaderNames();
        const n = headerNames.length;
        return this.rows.map((row) => {
          const obj = {};
          for (let i = 0; i < n; i++) {
            obj[headerNames[i]] = row[i];
          }
          return obj;
        });
      }
      getColumnByIndex(columnIndex) {
        if (!this.header[columnIndex])
          return;
        return this.rows.map((row) => row[columnIndex]);
      }
      getColumnByName(columnName) {
        const columnIndex = this.header.findIndex((h) => h.name === columnName);
        if (columnIndex < 0)
          return;
        return this.getColumnByIndex(columnIndex);
      }
      getSqlOuterLimit() {
        const { query } = this;
        if (!query)
          return;
        const { context } = query;
        if (typeof context !== "object")
          return;
        return context.sqlOuterLimit;
      }
      guessDateColumnIndexes() {
        const { header, rows } = this;
        if (!rows.length)
          return [];
        const indexes = header.map((_x, i) => i);
        for (const row of rows) {
          let i = 0;
          while (i < indexes.length) {
            const index = indexes[i];
            if (isIsoDate(row[index])) {
              i++;
            } else {
              indexes.splice(i, 1);
            }
          }
          if (!indexes.length)
            break;
        }
        return indexes;
      }
      inflateDatesForIndexes(indexes) {
        if (!indexes.length)
          return this;
        const value = this.valueOf();
        value.rows = this.rows.map((row) => {
          row = row.slice();
          for (const index of indexes) {
            row[index] = new Date(row[index]);
          }
          return row;
        });
        return new _QueryResult(value);
      }
      inflateDatesByGuessing() {
        return this.inflateDatesForIndexes(this.guessDateColumnIndexes());
      }
      inflateDatesFromSqlTypes() {
        return this.inflateDatesForIndexes(filterMap(this.header, (c, i) => c.sqlType === "TIMESTAMP" ? i : void 0));
      }
    };
    QueryResult = _QueryResult;
    QueryResult.jsonParse = JSON.parse;
    QueryResult.BLANK = new QueryResult({ header: [], rows: [] });
  }
});

// src/query-runner/query-runner.ts
var QueryRunner;
var init_query_runner = __esm({
  "src/query-runner/query-runner.ts"() {
    "use strict";
    init_query_result();
    init_sql();
    QueryRunner = class {
      static now() {
        return Date.now();
      }
      static isEmptyContext(context) {
        return !context || Object.keys(context).length === 0;
      }
      constructor(options = {}) {
        this.executor = options.executor;
        this.inflateDateStrategy = options.inflateDateStrategy || "fromSqlTypes";
      }
      getExecutor() {
        const executor = this.executor || QueryRunner.defaultQueryExecutor;
        if (!executor) {
          throw new Error(`Query executor must be provided or a default must be defined`);
        }
        return executor;
      }
      async runQuery(options) {
        const {
          query,
          extraQueryContext,
          queryParameters,
          resultFormat,
          header,
          typesHeader,
          sqlTypesHeader,
          cancelToken
        } = options;
        let isSql;
        let jsonQuery;
        let parsedQuery;
        if (typeof query === "string") {
          isSql = true;
          jsonQuery = {
            query,
            resultFormat: resultFormat ?? "array",
            header: header ?? true,
            typesHeader: (header && typesHeader) ?? true,
            sqlTypesHeader: (header && sqlTypesHeader) ?? true
          };
          try {
            parsedQuery = SqlQuery.parse(query);
          } catch {
          }
        } else if (query instanceof SqlQuery) {
          isSql = true;
          parsedQuery = query;
          jsonQuery = {
            query: String(query),
            resultFormat: resultFormat ?? "array",
            header: header ?? true,
            typesHeader: (header && typesHeader) ?? true,
            sqlTypesHeader: (header && sqlTypesHeader) ?? true
          };
        } else {
          jsonQuery = query;
          isSql = !jsonQuery.queryType && typeof jsonQuery.query === "string";
          if (isSql) {
            try {
              parsedQuery = SqlQuery.parse(jsonQuery.query);
            } catch {
            }
          }
        }
        if (!QueryRunner.isEmptyContext(extraQueryContext)) {
          jsonQuery = __spreadProps(__spreadValues({}, jsonQuery), { context: __spreadValues(__spreadValues({}, jsonQuery.context || {}), extraQueryContext) });
        }
        if (typeof jsonQuery.query === "string" && Array.isArray(queryParameters) && queryParameters.length) {
          jsonQuery = __spreadProps(__spreadValues({}, jsonQuery), { parameters: queryParameters });
        }
        const startTime = QueryRunner.now();
        const dataAndHeaders = await this.getExecutor()(jsonQuery, isSql, cancelToken);
        const endTime = QueryRunner.now();
        if (cancelToken)
          cancelToken.throwIfRequested();
        const result = QueryResult.fromQueryAndRawResult(jsonQuery, dataAndHeaders.data, dataAndHeaders.headers).attachQuery(jsonQuery, parsedQuery).attachQueryId(dataAndHeaders.headers["x-druid-query-id"], dataAndHeaders.headers["x-druid-sql-query-id"]).changeQueryDuration(endTime - startTime);
        switch (this.inflateDateStrategy) {
          case "fromSqlTypes":
            return result.inflateDatesFromSqlTypes();
          case "guess":
            return result.inflateDatesByGuessing();
          default:
            return result;
        }
      }
    };
  }
});

// src/filter-pattern/common.ts
function extractOuterNot(ex) {
  let negated = false;
  if (ex instanceof SqlUnary && ex.op === "NOT") {
    negated = true;
    ex = ex.argument.changeParens([]);
  }
  return [negated, ex];
}
function sqlRecordGetLiteralValues(record) {
  const exs = record.expressions ? record.expressions.values : [];
  const values = filterMap(exs, (ex) => ex instanceof SqlLiteral ? ex.value : void 0);
  return exs.length === values.length ? values : void 0;
}
function castAsVarchar(ex) {
  return SqlFunction.cast(ex, "VARCHAR");
}
function unwrapCastAsVarchar(ex) {
  if (ex instanceof SqlFunction && ex.getEffectiveFunctionName() === "CAST") {
    const t = ex.getArg(1);
    if (t instanceof SqlType && t.getEffectiveType() === "VARCHAR") {
      return ex.getArg(0);
    }
  }
  return ex;
}
function oneOf(thing, ...options) {
  return options.includes(thing);
}
var init_common = __esm({
  "src/filter-pattern/common.ts"() {
    "use strict";
    init_sql();
    init_utils();
  }
});

// src/filter-pattern/pattern-contains.ts
var CONTAINS_PATTERN_DEFINITION;
var init_pattern_contains = __esm({
  "src/filter-pattern/pattern-contains.ts"() {
    "use strict";
    init_sql();
    init_common();
    CONTAINS_PATTERN_DEFINITION = {
      name: "Contains",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlFunction))
          return;
        if (ex.getEffectiveFunctionName() !== "ICONTAINS_STRING")
          return;
        let columnEx = ex.getArg(0);
        if (!columnEx)
          return;
        columnEx = unwrapCastAsVarchar(columnEx);
        if (!(columnEx instanceof SqlColumn))
          return;
        const needle = ex.getArgAsString(1);
        if (!needle)
          return;
        return {
          type: "contains",
          negated,
          column: columnEx.getName(),
          contains: needle
        };
      },
      isValid(pattern) {
        return Boolean(pattern.contains);
      },
      toExpression(pattern) {
        return F("ICONTAINS_STRING", castAsVarchar(C(pattern.column)), L(pattern.contains)).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.column} ~ '${pattern.contains}'`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(pattern) {
        return pattern.contains;
      }
    };
  }
});

// src/filter-pattern/pattern-custom.ts
var CUSTOM_PATTERN_DEFINITION;
var init_pattern_custom = __esm({
  "src/filter-pattern/pattern-custom.ts"() {
    "use strict";
    init_sql();
    init_common();
    CUSTOM_PATTERN_DEFINITION = {
      name: "Custom",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        return {
          type: "custom",
          negated,
          expression: ex
        };
      },
      isValid(pattern) {
        return Boolean(pattern.expression);
      },
      toExpression(pattern) {
        return (pattern.expression || SqlLiteral.TRUE).applyIf(pattern.negated, (ex) => ex.not());
      },
      formatWithoutNegation(pattern) {
        return String(pattern.expression);
      },
      getColumn(pattern) {
        var _a;
        return (_a = pattern.expression) == null ? void 0 : _a.getFirstColumnName();
      },
      getThing(_pattern) {
        return;
      }
    };
  }
});

// src/filter-pattern/pattern-mv-contains.ts
var MV_CONTAINS_PATTERN_DEFINITION;
var init_pattern_mv_contains = __esm({
  "src/filter-pattern/pattern-mv-contains.ts"() {
    "use strict";
    init_sql();
    init_utils();
    init_common();
    MV_CONTAINS_PATTERN_DEFINITION = {
      name: "MV contains",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlFunction))
          return;
        if (ex.getEffectiveFunctionName() !== "MV_CONTAINS")
          return;
        const columnEx = ex.getArg(0);
        if (!(columnEx instanceof SqlColumn))
          return;
        const valuesEx = ex.getArg(1);
        if (!(valuesEx instanceof SqlFunction) || valuesEx.getEffectiveFunctionName() !== "ARRAY")
          return;
        const valueArgs = valuesEx.getArgArray();
        const values = filterMap(valueArgs, (valueArg) => valueArg instanceof SqlLiteral ? valueArg.value : void 0);
        if (valueArgs.length !== values.length)
          return;
        return {
          type: "mvContains",
          negated,
          column: columnEx.getName(),
          values
        };
      },
      isValid(pattern) {
        return Array.isArray(pattern.values) && Boolean(pattern.values.length);
      },
      toExpression(pattern) {
        return F("MV_CONTAINS", C(pattern.column), F.array(...pattern.values)).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.column} on of ${pattern.values.map((v) => v === "" ? "empty" : String(v)).join(", ")}`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(pattern) {
        return pattern.values.length ? String(pattern.values[0]) : void 0;
      }
    };
  }
});

// src/filter-pattern/pattern-number-range.ts
var NUMBER_RANGE_PATTERN_DEFINITION;
var init_pattern_number_range = __esm({
  "src/filter-pattern/pattern-number-range.ts"() {
    "use strict";
    init_sql();
    init_common();
    NUMBER_RANGE_PATTERN_DEFINITION = {
      name: "Number range",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlMulti))
          return;
        if (ex.args.values.length !== 2)
          return;
        if (!ex.args.values.every((v) => v instanceof SqlComparison))
          return;
        const values = ex.args.values;
        const left = values.find((v) => {
          return (v.op === ">" || v.op === ">=") && v.rhs instanceof SqlLiteral || (v.op === "<" || v.op === "<=") && v.rhs instanceof SqlColumn;
        });
        const right = values.find((v) => {
          return (v.op === ">" || v.op === ">=") && v.rhs instanceof SqlColumn || (v.op === "<" || v.op === "<=") && v.rhs instanceof SqlLiteral;
        });
        if (!left || !right)
          return;
        if (left.getFirstColumnName() !== right.getFirstColumnName())
          return;
        const column = left.getFirstColumnName();
        if (!column)
          return;
        const leftValue = left.rhs instanceof SqlLiteral ? left.rhs.value : left.lhs instanceof SqlLiteral ? left.lhs.value : void 0;
        const rightValue = right.rhs instanceof SqlLiteral ? right.rhs.value : right.lhs instanceof SqlLiteral ? right.lhs.value : void 0;
        if (typeof leftValue !== "number" || typeof rightValue !== "number")
          return;
        if (isNaN(leftValue) || isNaN(rightValue))
          return;
        return {
          type: "numberRange",
          column,
          negated,
          start: leftValue,
          end: rightValue,
          startBound: left.op === ">" ? "(" : "[",
          endBound: right.op === "<" ? ")" : "]"
        };
      },
      isValid(_pattern) {
        return true;
      },
      toExpression(pattern) {
        return SqlMulti.and(pattern.startBound === "(" ? SqlComparison.greaterThan(C(pattern.column), L(pattern.start)) : SqlComparison.greaterThanOrEqual(C(pattern.column), L(pattern.start)), pattern.endBound === ")" ? SqlComparison.lessThan(C(pattern.column), L(pattern.end)) : SqlComparison.lessThanOrEqual(C(pattern.column), L(pattern.end))).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.startBound}${pattern.start}, ${pattern.end}${pattern.endBound}`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(_pattern) {
        return;
      }
    };
  }
});

// src/filter-pattern/pattern-regexp.ts
var REGEXP_PATTERN_DEFINITION;
var init_pattern_regexp = __esm({
  "src/filter-pattern/pattern-regexp.ts"() {
    "use strict";
    init_sql();
    init_common();
    REGEXP_PATTERN_DEFINITION = {
      name: "Regular expression",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlFunction))
          return;
        const name = ex.getEffectiveFunctionName();
        if (name !== "REGEXP_LIKE")
          return;
        let a0 = ex.getArg(0);
        if (!a0)
          return;
        a0 = unwrapCastAsVarchar(a0);
        if (!(a0 instanceof SqlColumn))
          return;
        const a1 = ex.getArg(1);
        if (!(a1 instanceof SqlLiteral))
          return;
        return {
          type: "regexp",
          negated,
          column: a0.getName(),
          regexp: a1.value
        };
      },
      isValid(pattern) {
        return Boolean(pattern.regexp);
      },
      toExpression(pattern) {
        return F("REGEXP_LIKE", castAsVarchar(C(pattern.column)), L(pattern.regexp)).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.column} ~ /${pattern.regexp}/`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(pattern) {
        return pattern.regexp;
      }
    };
  }
});

// src/filter-pattern/pattern-time-interval.ts
function invalidDate(date) {
  return isNaN(date.valueOf());
}
var TIME_INTERVAL_PATTERN_DEFINITION;
var init_pattern_time_interval = __esm({
  "src/filter-pattern/pattern-time-interval.ts"() {
    "use strict";
    init_sql();
    init_common();
    TIME_INTERVAL_PATTERN_DEFINITION = {
      name: "Time interval",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlFunction))
          return;
        const name = ex.getEffectiveFunctionName();
        if (name !== "TIME_IN_INTERVAL")
          return;
        const a0 = ex.getArg(0);
        if (!(a0 instanceof SqlColumn))
          return;
        const a1 = ex.getArgAsString(1);
        if (!a1)
          return;
        const intervalParts = a1.split("/");
        if (intervalParts.length !== 2)
          return;
        const start = new Date(intervalParts[0]);
        if (invalidDate(start))
          return;
        const end = new Date(intervalParts[1]);
        if (invalidDate(end))
          return;
        return {
          type: "timeInterval",
          negated,
          column: a0.getName(),
          start,
          end
        };
      },
      isValid(_pattern) {
        return true;
      },
      toExpression(pattern) {
        return F("TIME_IN_INTERVAL", C(pattern.column), L(`${pattern.start.toISOString()}/${pattern.end.toISOString()}`)).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        let startString = pattern.start.toISOString().replace(/Z$/, "");
        let endString = pattern.end.toISOString().replace(/Z$/, "");
        if (startString.endsWith(".000") && endString.endsWith(".000")) {
          startString = startString.replace(/\.000$/, "");
          endString = endString.replace(/\.000$/, "");
        }
        if (startString.endsWith(":00") && endString.endsWith(":00")) {
          startString = startString.replace(/:00$/, "");
          endString = endString.replace(/:00$/, "");
        }
        return `${startString}/${endString}`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(_pattern) {
        return;
      }
    };
  }
});

// src/filter-pattern/pattern-time-relative.ts
function getAnchor(anchor) {
  if (anchor === "currentTimestamp") {
    return new SqlFunction({
      functionName: new RefName({ name: "CURRENT_TIMESTAMP", quotes: false }),
      specialParen: "none"
    });
  } else {
    return new SqlFunction({
      functionName: new RefName({ name: "MAX_DATA_TIME", quotes: false })
    });
  }
}
var TIME_RELATIVE_PATTERN_DEFINITION;
var init_pattern_time_relative = __esm({
  "src/filter-pattern/pattern-time-relative.ts"() {
    "use strict";
    init_sql();
    init_common();
    TIME_RELATIVE_PATTERN_DEFINITION = {
      name: "Time relative",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlMulti) || ex.numArgs() !== 2)
          return;
        const a = ex.getArg(0);
        if (!(a instanceof SqlComparison) || a.op !== "<=")
          return;
        const b = ex.getArg(1);
        if (!(b instanceof SqlComparison) || b.op !== "<")
          return;
        const columnRef = a.rhs;
        if (!(columnRef instanceof SqlColumn) || !columnRef.equals(b.lhs))
          return;
        const timeShift = a.lhs;
        if (!(timeShift instanceof SqlFunction) || timeShift.getEffectiveFunctionName() !== "TIME_SHIFT") {
          return;
        }
        let anchorFn = timeShift.getArg(0);
        if (!b.rhs.equals(anchorFn))
          return;
        const rangeDuration = timeShift.getArgAsString(1);
        if (!rangeDuration)
          return;
        const rangeStep = -timeShift.getArgAsNumber(2);
        if (!(rangeStep > 0))
          return;
        let shiftDuration;
        let shiftStep;
        if (anchorFn instanceof SqlFunction && anchorFn.getEffectiveFunctionName() === "TIME_SHIFT") {
          shiftDuration = anchorFn.getArgAsString(1);
          if (!shiftDuration)
            return;
          shiftStep = anchorFn.getArgAsNumber(2);
          if (!shiftStep)
            return;
          anchorFn = anchorFn.getArg(0);
        }
        let alignType;
        let alignDuration;
        if (anchorFn instanceof SqlFunction && oneOf(anchorFn.getEffectiveFunctionName(), "TIME_FLOOR", "TIME_CEIL")) {
          alignType = anchorFn.getEffectiveFunctionName() === "TIME_FLOOR" ? "floor" : "ceil";
          alignDuration = anchorFn.getArgAsString(1);
          if (!alignDuration)
            return;
          anchorFn = anchorFn.getArg(0);
        }
        if (!(anchorFn instanceof SqlFunction))
          return;
        let anchor;
        switch (anchorFn.getEffectiveFunctionName()) {
          case "CURRENT_TIMESTAMP":
            anchor = "currentTimestamp";
            break;
          case "MAX_DATA_TIME":
            anchor = "maxDataTime";
            break;
          default:
            return;
        }
        return {
          type: "timeRelative",
          negated,
          column: columnRef.getName(),
          anchor,
          rangeDuration,
          rangeStep: rangeStep !== 1 ? rangeStep : void 0,
          alignDuration,
          alignType,
          shiftDuration,
          shiftStep
        };
      },
      isValid(_pattern) {
        return true;
      },
      toExpression(pattern) {
        let anchor = getAnchor(pattern.anchor);
        if (pattern.alignType && pattern.alignDuration) {
          anchor = F(pattern.alignType === "floor" ? "TIME_FLOOR" : "TIME_CEIL", anchor, pattern.alignDuration);
        }
        if (pattern.shiftDuration && pattern.shiftStep) {
          anchor = F.timeShift(anchor, pattern.shiftDuration, pattern.shiftStep);
        }
        const column = C(pattern.column);
        return F.timeShift(anchor, pattern.rangeDuration, -(pattern.rangeStep || 1)).lessThanOrEqual(column).and(column.lessThan(anchor)).ensureParens().applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.column} in ${pattern.rangeDuration}`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(_pattern) {
        return;
      }
    };
  }
});

// src/filter-pattern/utils.ts
function xor(a, b) {
  return Boolean(a ? !b : b);
}
var init_utils3 = __esm({
  "src/filter-pattern/utils.ts"() {
    "use strict";
  }
});

// src/filter-pattern/pattern-values.ts
var VALUES_PATTERN_DEFINITION;
var init_pattern_values = __esm({
  "src/filter-pattern/pattern-values.ts"() {
    "use strict";
    init_sql();
    init_common();
    init_utils3();
    VALUES_PATTERN_DEFINITION = {
      name: "Values",
      fit(possibleEx) {
        const [negated, ex] = extractOuterNot(possibleEx);
        if (!(ex instanceof SqlComparison))
          return;
        const { lhs, rhs, op } = ex;
        switch (op) {
          case "=":
            if (lhs instanceof SqlColumn && rhs instanceof SqlLiteral) {
              return {
                type: "values",
                negated,
                column: lhs.getName(),
                values: [rhs.value]
              };
            }
            return;
          case "<>":
            if (lhs instanceof SqlColumn && rhs instanceof SqlLiteral) {
              return {
                type: "values",
                negated: !negated,
                column: lhs.getName(),
                values: [rhs.value]
              };
            }
            return;
          case "IN":
            if (lhs instanceof SqlColumn && rhs instanceof SqlRecord) {
              const values = sqlRecordGetLiteralValues(rhs);
              if (values) {
                return {
                  type: "values",
                  negated: xor(ex.negated, negated),
                  column: lhs.getName(),
                  values
                };
              }
            }
            return;
          default:
            return;
        }
      },
      isValid(pattern) {
        return Array.isArray(pattern.values) && Boolean(pattern.values.length);
      },
      toExpression(pattern) {
        return C(pattern.column).apply((ex) => pattern.values.length <= 1 ? ex.equal(pattern.values[0] ?? SqlPlaceholder.PLACEHOLDER) : ex.in(pattern.values)).applyIf(pattern.negated, (ex) => ex.negate());
      },
      formatWithoutNegation(pattern) {
        return `${pattern.column}: ${pattern.values.map((v) => v === "" ? "empty" : String(v)).join(", ")}`;
      },
      getColumn(pattern) {
        return pattern.column;
      },
      getThing(pattern) {
        return pattern.values.length ? String(pattern.values[0]) : void 0;
      }
    };
  }
});

// src/filter-pattern/index.ts
function fitFilterPatterns(ex) {
  const wholeFit = fitFilterPattern(ex);
  if (wholeFit.type !== "custom")
    return [wholeFit];
  return ex.decomposeViaAnd({ flatten: false }).map(fitFilterPattern);
}
function fitFilterPattern(ex) {
  for (const type of FILTER_PATTERN_TYPES) {
    const pattern = TYPE_TO_DESCRIPTION[type].fit(ex);
    if (pattern)
      return pattern;
  }
  return {
    type: "custom",
    negated: false,
    expression: ex
  };
}
function filterPatternsToExpression(patterns) {
  return SqlExpression.and(...patterns.map(filterPatternToExpression));
}
function filterPatternToExpression(pattern) {
  return TYPE_TO_DESCRIPTION[pattern.type].toExpression(pattern);
}
function formatPatternWithoutNegation(pattern) {
  return TYPE_TO_DESCRIPTION[pattern.type].formatWithoutNegation(pattern);
}
function getColumnFromPattern(pattern) {
  return TYPE_TO_DESCRIPTION[pattern.type].getColumn(pattern);
}
function getThingFromPattern(pattern) {
  return TYPE_TO_DESCRIPTION[pattern.type].getThing(pattern);
}
function changeFilterPatternType(pattern, newType) {
  const column = getColumnFromPattern(pattern);
  const thing = getThingFromPattern(pattern);
  switch (newType) {
    case "values":
      return {
        type: "values",
        negated: pattern.negated,
        column: column || "?",
        values: thing ? [thing] : []
      };
    case "contains":
      return {
        type: "contains",
        negated: pattern.negated,
        column: column || "?",
        contains: thing || ""
      };
    case "regexp":
      return {
        type: "regexp",
        negated: pattern.negated,
        column: column || "?",
        regexp: thing || ""
      };
    case "timeInterval":
      return {
        type: "timeInterval",
        negated: pattern.negated,
        column: column || "?",
        start: new Date("2020-01-01Z"),
        end: new Date("2022-01-01Z")
      };
    case "timeRelative":
      return {
        type: "timeRelative",
        negated: pattern.negated,
        column: column || "?",
        anchor: "currentTimestamp",
        rangeDuration: "P1D"
      };
    case "numberRange":
      return {
        type: "numberRange",
        negated: pattern.negated,
        column: column || "?",
        start: 0,
        end: 100,
        startBound: "(",
        endBound: ")"
      };
    case "mvContains":
      return {
        type: "mvContains",
        negated: pattern.negated,
        column: column || "?",
        values: thing ? [thing] : []
      };
    case "custom":
      return {
        type: "custom",
        negated: false,
        expression: filterPatternToExpression(pattern)
      };
  }
}
function patternTypeToName(type) {
  return TYPE_TO_DESCRIPTION[type].name;
}
function initPatternForColumn(columnName, columnType) {
  switch (columnType) {
    case "TIMESTAMP":
      return {
        type: "timeInterval",
        negated: false,
        column: columnName,
        start: new Date("2020-01-01Z"),
        end: new Date("2022-01-01Z")
      };
    default:
      return {
        type: "values",
        negated: false,
        column: columnName,
        values: []
      };
  }
}
var FILTER_PATTERN_TYPES, TYPE_TO_DESCRIPTION;
var init_filter_pattern = __esm({
  "src/filter-pattern/index.ts"() {
    "use strict";
    init_sql();
    init_pattern_contains();
    init_pattern_custom();
    init_pattern_mv_contains();
    init_pattern_number_range();
    init_pattern_regexp();
    init_pattern_time_interval();
    init_pattern_time_relative();
    init_pattern_values();
    FILTER_PATTERN_TYPES = [
      "values",
      "contains",
      "regexp",
      "timeInterval",
      "timeRelative",
      "numberRange",
      "mvContains",
      "custom"
    ];
    TYPE_TO_DESCRIPTION = {
      values: VALUES_PATTERN_DEFINITION,
      contains: CONTAINS_PATTERN_DEFINITION,
      regexp: REGEXP_PATTERN_DEFINITION,
      timeInterval: TIME_INTERVAL_PATTERN_DEFINITION,
      timeRelative: TIME_RELATIVE_PATTERN_DEFINITION,
      numberRange: NUMBER_RANGE_PATTERN_DEFINITION,
      mvContains: MV_CONTAINS_PATTERN_DEFINITION,
      custom: CUSTOM_PATTERN_DEFINITION
    };
  }
});

// src/index.ts
var src_exports = {};
__export(src_exports, {
  C: () => C,
  Column: () => Column,
  F: () => F,
  FILTER_PATTERN_TYPES: () => FILTER_PATTERN_TYPES,
  Introspect: () => Introspect,
  L: () => L,
  N: () => N,
  QueryResult: () => QueryResult,
  QueryRunner: () => QueryRunner,
  RefName: () => RefName,
  SeparatedArray: () => SeparatedArray,
  Separator: () => Separator,
  SqlAlias: () => SqlAlias,
  SqlBase: () => SqlBase,
  SqlBetweenPart: () => SqlBetweenPart,
  SqlCase: () => SqlCase,
  SqlClause: () => SqlClause,
  SqlClusteredByClause: () => SqlClusteredByClause,
  SqlColumn: () => SqlColumn,
  SqlColumnDeclaration: () => SqlColumnDeclaration,
  SqlColumnList: () => SqlColumnList,
  SqlComparison: () => SqlComparison,
  SqlExpression: () => SqlExpression,
  SqlExtendClause: () => SqlExtendClause,
  SqlFromClause: () => SqlFromClause,
  SqlFunction: () => SqlFunction,
  SqlGroupByClause: () => SqlGroupByClause,
  SqlHavingClause: () => SqlHavingClause,
  SqlInsertClause: () => SqlInsertClause,
  SqlInterval: () => SqlInterval,
  SqlJoinPart: () => SqlJoinPart,
  SqlLabeledExpression: () => SqlLabeledExpression,
  SqlLikePart: () => SqlLikePart,
  SqlLimitClause: () => SqlLimitClause,
  SqlLiteral: () => SqlLiteral,
  SqlMulti: () => SqlMulti,
  SqlNamespace: () => SqlNamespace,
  SqlOffsetClause: () => SqlOffsetClause,
  SqlOrderByClause: () => SqlOrderByClause,
  SqlOrderByExpression: () => SqlOrderByExpression,
  SqlPartitionedByClause: () => SqlPartitionedByClause3,
  SqlPlaceholder: () => SqlPlaceholder,
  SqlQuery: () => SqlQuery,
  SqlRecord: () => SqlRecord,
  SqlReplaceClause: () => SqlReplaceClause,
  SqlStar: () => SqlStar,
  SqlTable: () => SqlTable,
  SqlType: () => SqlType,
  SqlUnary: () => SqlUnary,
  SqlValues: () => SqlValues,
  SqlWhenThenPart: () => SqlWhenThenPart,
  SqlWhereClause: () => SqlWhereClause,
  SqlWindowSpec: () => SqlWindowSpec,
  SqlWithClause: () => SqlWithClause,
  SqlWithPart: () => SqlWithPart,
  SqlWithQuery: () => SqlWithQuery,
  T: () => T,
  changeFilterPatternType: () => changeFilterPatternType,
  clampIndex: () => clampIndex,
  cleanObject: () => cleanObject,
  compact: () => compact,
  dedupe: () => dedupe,
  filterMap: () => filterMap,
  filterPatternToExpression: () => filterPatternToExpression,
  filterPatternsToExpression: () => filterPatternsToExpression,
  fitFilterPattern: () => fitFilterPattern,
  fitFilterPatterns: () => fitFilterPatterns,
  formatPatternWithoutNegation: () => formatPatternWithoutNegation,
  getEscapedRegExp: () => getEscapedRegExp,
  initPatternForColumn: () => initPatternForColumn,
  insert: () => insert,
  isEmptyArray: () => isEmptyArray,
  needsUnicodeEscape: () => needsUnicodeEscape,
  normalizeIndex: () => normalizeIndex,
  objectMap: () => objectMap,
  patternTypeToName: () => patternTypeToName,
  sane: () => sane,
  sqlEscapeUnicode: () => sqlEscapeUnicode,
  trimString: () => trimString
});
module.exports = __toCommonJS(src_exports);
var init_src = __esm({
  "src/index.ts"() {
    init_utils();
    init_sql();
    init_introspect();
    init_column2();
    init_query_result();
    init_query_runner();
    init_filter_pattern();
  }
});
init_src();
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  C,
  Column,
  F,
  FILTER_PATTERN_TYPES,
  Introspect,
  L,
  N,
  QueryResult,
  QueryRunner,
  RefName,
  SeparatedArray,
  Separator,
  SqlAlias,
  SqlBase,
  SqlBetweenPart,
  SqlCase,
  SqlClause,
  SqlClusteredByClause,
  SqlColumn,
  SqlColumnDeclaration,
  SqlColumnList,
  SqlComparison,
  SqlExpression,
  SqlExtendClause,
  SqlFromClause,
  SqlFunction,
  SqlGroupByClause,
  SqlHavingClause,
  SqlInsertClause,
  SqlInterval,
  SqlJoinPart,
  SqlLabeledExpression,
  SqlLikePart,
  SqlLimitClause,
  SqlLiteral,
  SqlMulti,
  SqlNamespace,
  SqlOffsetClause,
  SqlOrderByClause,
  SqlOrderByExpression,
  SqlPartitionedByClause,
  SqlPlaceholder,
  SqlQuery,
  SqlRecord,
  SqlReplaceClause,
  SqlStar,
  SqlTable,
  SqlType,
  SqlUnary,
  SqlValues,
  SqlWhenThenPart,
  SqlWhereClause,
  SqlWindowSpec,
  SqlWithClause,
  SqlWithPart,
  SqlWithQuery,
  T,
  changeFilterPatternType,
  clampIndex,
  cleanObject,
  compact,
  dedupe,
  filterMap,
  filterPatternToExpression,
  filterPatternsToExpression,
  fitFilterPattern,
  fitFilterPatterns,
  formatPatternWithoutNegation,
  getEscapedRegExp,
  initPatternForColumn,
  insert,
  isEmptyArray,
  needsUnicodeEscape,
  normalizeIndex,
  objectMap,
  patternTypeToName,
  sane,
  sqlEscapeUnicode,
  trimString
});
//# sourceMappingURL=druid-query-toolkit.js.map